/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    Cache.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept. 9th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.util.Hashtable;
import java.io.Serializable;
import java.util.Enumeration;

public class Cache implements Serializable
{
    public static Hashtable cache;  // hashtable holding the cache entries
    int cache_size;                 // the max number of cache entries

/******************************************************************************
 **
 **     Purpose:               Constructor - sets the cache size and instan-
 **                                          tiates the cache hashtable.
 **    
 **     Parameters:            int cache_size - the number of cache entries
 **
 **     Author/Last Modified:  Ram Periakaruppan, Sept 9th, 1999
 **
 *****************************************************************************/

    public Cache(int cache_size)
    {
	this.cache_size = cache_size;
	cache = new Hashtable();
    }

/******************************************************************************
 **
 **     Method name:             deleteCache
 **
 **     Purpose:                 Clears all the contents of the cache hashtable
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 9th, 1999
 **
 *****************************************************************************/
    public static void deleteCache()
    {
	cache.clear();
    }

/******************************************************************************
 **
 **     Method name:             lookup
 **
 **     Purpose:                 Lookup a IP address in the cache hashtable and
 **                              return record if found.
 **    
 **     Parameters:              String ipaddr - the IP address to serach for
 **
 **     Return Values:           LocationDetails - a LocationDeatils object
 **                                     that stores all details about the
 **                                     requested IP address.
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 9th, 1999
 **
 *****************************************************************************/

    public LocationDetails lookup(String ipaddr)
    {
	return (LocationDetails)cache.get(ipaddr);
    }

/******************************************************************************
 **
 **     Method name:             add
 **
 **     Purpose:                 Method is invoked to add an entry to the cache
 **                              
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 9th, 1999
 **
 *****************************************************************************/

    public void add(String key, LocationDetails data)
    {
	if (cache.size() == cache_size)
	{
	    String keys[] = new String[cache_size];
	    int hits[] = new int[cache_size];
	    int k=0;
	    for (Enumeration e = cache.keys(); e.hasMoreElements() ;) 
	    {
		keys[k] = (String)e.nextElement();
		hits[k] = ((LocationDetails)cache.get(keys[k])).getHits();
		k++;
	    }

	    String tmpKey;
	    int tmpHit;
	    for (int i=0; i<cache_size; i++)
	    {
		for (int j=i; j<cache_size; j++)
		{
		    if (hits[i] > hits[j])
		    {
			tmpHit = hits[i];
			hits[i] = hits[j];
			hits[j] = tmpHit;

			tmpKey = keys[i];
			keys[i] = keys[j];
			keys[j] = tmpKey;
		    }
		}
	    }

	    // remove top five elements
	    for (int i=0; i<5; i++)
	    {
		cache.remove(keys[i]);
	    }
	}
	
	if (!cache.contains(key))
	{
	    cache.put(key, data);
	}
    }
}
