/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    ToolBar.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept. 13th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import com.sleepycat.db.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;


public class DbNew extends JDialog implements ActionListener
{
    JFrame frame;
    private JDialog dbNewDialog;
    JLabel dbNameLabel;
    JLabel warningLabel;
    JLabel textFileLabel;
    JLabel contentsLabel;
    JTextField dbTField;
    JTextField textFileTField;
    JComboBox contentsCombo;
    JButton newButton;
    JButton cancelButton;
    JButton chooseFileButton;

    private static final int SUCCESS = 5;
    private static final int FAILURE = -1;

    
    public final static Dimension TEXTFIELD = new Dimension(100,25);
    private String newline = "\n";
    private static final String databaseFieldString = "Database Name";
    private static final String textFileFieldString = "Input Text File";
    private static final String contentsFieldString = "Contents";


    public DbNew(JFrame f) 
    {
        this.frame = f;
 	dbNewDialog = new JDialog(frame, "Database Creation", false);
	dbNewDialog.setSize(525, 300);
	dbNewDialog.setResizable(false);

	// Labels
	dbNameLabel = new JLabel(databaseFieldString);
	textFileLabel = new JLabel(textFileFieldString);
	contentsLabel = new JLabel(contentsFieldString);
	warningLabel = new JLabel("* Database will be created and placed "
                                  + "under the `db' subdirectory");
	warningLabel.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));
	

	// TextFields

	dbTField = new JTextField(15);
        dbTField.setActionCommand(databaseFieldString);
        dbTField.addActionListener(this);


	textFileTField = new JTextField(15);
        textFileTField.setActionCommand(textFileFieldString);
        textFileTField.addActionListener(this);

	// Combo

	contentsCombo = new JComboBox();
	contentsCombo.setEditable(false);
	contentsCombo.addItem("Hostnames Only");
	contentsCombo.addItem("IP Addresses Only");
	contentsCombo.addItem("IP addresses and Hostnames");

	// Buttons

	chooseFileButton = new JButton("Choose File...");
	chooseFileButton.setMnemonic('F');
	chooseFileButton.setBorder(BorderFactory.createRaisedBevelBorder());
	chooseFileButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	chooseFileButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
	    {
                    JFileChooser chooser = new JFileChooser(GTrace.DbDir);
                    chooser.setMultiSelectionEnabled(false);
                    chooser.addChoosableFileFilter(new GTraceFileFilter(
                                             new String[] {"txt"}, "(*.txt)"));
	    
                    int option = chooser.showOpenDialog(frame);
                    if (option == JFileChooser.APPROVE_OPTION)
                    {
                        if (chooser.getSelectedFile() != null)
                        {
                            String OPENDIR =
                               chooser.getCurrentDirectory().getAbsolutePath();
                            String fname =
                              chooser.getCurrentDirectory().getAbsolutePath() +
                                System.getProperty("file.separator") + 
                                chooser.getSelectedFile().getName();
                            textFileTField.setText(fname); 
                        }
                    }

	    }
	});


	cancelButton = new JButton("Cancel");
	cancelButton.setMnemonic('C');
	cancelButton.setBorder(BorderFactory.createRaisedBevelBorder());
	cancelButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
	    {
		dbNewDialog.setVisible(false);
		dbNewDialog.dispose();
	    }
	});

	newButton = new JButton("Create");
	newButton.setMnemonic('C');
	newButton.setBorder(BorderFactory.createRaisedBevelBorder());
	newButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	newButton.addActionListener(this);


        JPanel dbPane = new JPanel();
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

        dbPane.setLayout(gridbag);

        //JLabel[] labels = {dbNameLabel, textFileLabel};
        //JTextField[] textFields = {textFileTField, dbTField};
	//addLabelTextRows(labels, textFields, gridbag, dbPane);
	

	// ROW 1
	//c.weightx = 0.5;
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(dbNameLabel, c);
	dbPane.add(dbNameLabel);

	c.gridx = 1;
	c.ipady = 8;  
	c.gridwidth = 2; 
	c.insets = new Insets(20,0,0,0); 
	gridbag.setConstraints(dbTField, c);
	dbPane.add(dbTField);


	// ROW 2
	//c.weightx = 0.0;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridx = 0;
	c.gridy = 1;
	c.insets = new Insets(0,45,0,0); 
	gridbag.setConstraints(warningLabel, c);
	dbPane.add(warningLabel);

	// ROW 3 
	//c.weightx = 0.5;
	c.gridx = 0;
	c.gridy = 2;
	c.gridwidth = 1;
	c.insets = new Insets(25,20,0,10); 
	gridbag.setConstraints(textFileLabel, c);
	dbPane.add(textFileLabel);

	
	c.gridx = 1;
	c.insets = new Insets(25,0,0,0); 
	gridbag.setConstraints(textFileTField, c);
	dbPane.add(textFileTField);
	

	c.gridx = 2;
	c.ipadx = 20;  
	c.insets = new Insets(25,0,0,0); 
	c.anchor = GridBagConstraints.WEST;
	gridbag.setConstraints(chooseFileButton, c);
	dbPane.add(chooseFileButton);
	
	// ROW 4
	c.gridx = 0;
	c.gridy = 3;
	c.ipadx = 0;  
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(contentsLabel, c);
	dbPane.add(contentsLabel);

	
	c.gridx = 1;
	c.insets = new Insets(20,0,0,0); 
	gridbag.setConstraints(contentsCombo, c);
	dbPane.add(contentsCombo);

	// ROW 5
	c.gridy = 4;
	c.ipadx = 30;  
	c.ipady = 10;
	c.insets = new Insets(30,30,0,0); 
	gridbag.setConstraints(newButton, c);
	dbPane.add(newButton);

	c.anchor = GridBagConstraints.EAST;
	c.insets = new Insets(30,0,0,30); 
	gridbag.setConstraints(cancelButton, c);
	dbPane.add(cancelButton);


	dbNewDialog.getContentPane().add(dbPane);
	dbNewDialog.setLocationRelativeTo(frame);
	dbNewDialog.setVisible(true);
    }


    public void actionPerformed(ActionEvent e)
    {
        if (dbTField.getText().length() == 0)
        {
                JOptionPane.showMessageDialog(frame,
                                    "You must specify a database name",  
                                    "New Database Error",
                                    JOptionPane.ERROR_MESSAGE);
        }
        else if (textFileTField.getText().length() == 0)
        {
                JOptionPane.showMessageDialog(frame,
                                    "You must specify the input textfile",  
                                    "New Database Error",
                                    JOptionPane.ERROR_MESSAGE);
        }
        else
        {
            File tmpf = new File(GTrace.DbDir + "__db_lock.share");
            if (tmpf.exists())
            {
                tmpf.delete();
            }

            tmpf = new File(GTrace.DbDir + "__db_mpool.share");
            if (tmpf.exists())
            {
                tmpf.delete();
            }
            
            try
            {
                newDb newdb = new newDb(textFileTField.getText(),
                                        GTrace.DbDir+ dbTField.getText());
                int status = newdb.createDb();

                if (status != FAILURE)
                {
                    dbNewDialog.setVisible(false);
                    dbNewDialog.dispose();

                    JOptionPane.showMessageDialog(frame,
                                    "Database successfully created",  
                                    "Record Added",
                                    JOptionPane.INFORMATION_MESSAGE);
                }
                
            }
            catch(DbException dbe)
            {
                JOptionPane.showMessageDialog(frame,
                                    "Error creating database",
                                    "New Database Error",
                                    JOptionPane.ERROR_MESSAGE);
                
            }

        }
    }

    public class newDb extends DbEnv
    {
        private String SrcFileName;
        private String DbFileName;
        
        public newDb(String sfname, String dbname) throws DbException
        {
            // delete the __mpool files if they exist each time
            
            super(GTrace.DbDir,null, Db.DB_CREATE |
                  Db.DB_INIT_MPOOL | Db.DB_INIT_CDB | Db.DB_THREAD);

            SrcFileName = sfname;
            DbFileName = dbname;

            try 
            {
                set_error_stream(System.err);
                set_errpfx("GTraceDb");
            }
            catch (Exception e) 
            {
                System.err.println(e.toString());
                System.exit(1);
            }
        }

        public int createDb() throws DbException
        {
            String fline;
            String token;
            RandomAccessFile fptr=null;
            String skey;
            String sdata;
            Dbt key;
            Dbt data;

            String newGtraceConf = new String();
            String str;
            int index;
 
            
            File tmpf = new File(DbFileName);
            if (tmpf.exists())
            {
                tmpf.delete();
            }

            Db Dbptr = Db.open(DbFileName, Db.DB_HASH, Db.DB_CREATE
                               | Db.DB_THREAD, 0644, this, null);

            try
            {
                fptr = new RandomAccessFile(SrcFileName,"r"); 

                for(;;)
                {
                    fline = fptr.readLine();

                    if (fline == null)
                    {
                        break;
                    }

                    if (!fline.startsWith("#")  && fline.trim().length() > 0)
                    {
                        StringTokenizer tokenizer = new StringTokenizer(fline);
		    
                        try
                        {
                            skey = tokenizer.nextToken();
                            sdata = new String("");

                            while(tokenizer.hasMoreTokens())
                            {
                                token = tokenizer.nextToken() + "\t"; 
                                sdata = sdata.concat(token);
                            }
			
                            key = new Dbt(skey.getBytes());
                            key.set_size(skey.length());
                            data = new Dbt(sdata.getBytes());
                            data.set_size(sdata.length());

                            try
                            {
                                int err;
                                if ((err = Dbptr.put(null, key, data, 0))
                                                            == Db.DB_KEYEXIST) 
                                {
                                    System.out.println("Key " + key
                                                       + " already exists.");
                                }
                            }
                            catch (DbException dbe)
                            {
                                System.out.println(dbe.toString());
                            }

                            System.out.println(skey + " : " + sdata);
                        }
                        catch(NoSuchElementException nsee) { }
                    }
                }
                Dbptr.close(0);
            }
            catch(IOException fnfe)
            {
                JOptionPane.showMessageDialog(frame,
                                              SrcFileName + " not found",
                                              "Delete Error",
                                              JOptionPane.ERROR_MESSAGE);
                return FAILURE;
            }

  try
  {
      File gtraceFile = new File(GTrace.HOMEDIR + GTrace.separator
                                 + "GTrace.properties");
	    
      RandomAccessFile rafptr = new RandomAccessFile(gtraceFile, "r");

      while((str = rafptr.readLine()) != null)
      {
          if (str.startsWith("DBS"))
          {
              String comboTxt = (String)contentsCombo.getSelectedItem();
              String contents = new String();
                        
              if (comboTxt.compareTo("Hostnames Only") == 0)
              {
                  contents = "hostnames";
              }
              else if (comboTxt.compareTo("IP Addresses Only") == 0)
              {
                  contents = "ipaddr";
              }
              else if (comboTxt.compareTo("IP addresses and Hostnames") == 0)
              {
                  contents = "hostnames/ipaddr";
              }

              newGtraceConf = newGtraceConf.concat(str +
                                                   dbTField.getText() + ","
                                                   + contents
                                                   + ";\n");
          }
          else
          {
              newGtraceConf = newGtraceConf.concat(str + "\n");
          }
      }

      gtraceFile.delete();
      rafptr.close();
      rafptr = new RandomAccessFile(gtraceFile, "rw");
      rafptr.writeBytes(newGtraceConf);
      rafptr.close();
      return SUCCESS;
  }
            catch(IOException ioe)
            {
                JOptionPane.showMessageDialog(frame,
                   "Could not update GTrace.properties configuration file.\n" +
                   "Try manually adding the database and its contents to "
                   + "the file.\n",
                   "Database error",
                   JOptionPane.ERROR_MESSAGE);
		return FAILURE;

            }
        }
    }
}
