/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    DbQAMD.java
**
**     Purpose:                 Implements the query, add, modify and delete
**                              options for the databases
**
**     Author/Date:             Ram Periakaruppan, Sept. 13th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
****************************************************************************/
package caida.GTrace;

import com.sleepycat.db.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

public class DbQAMD extends JDialog implements ActionListener
{
    JFrame frame;
    private JDialog dbQAMDDialog;
    JLabel databaseLabel;
    JLabel keyLabel;
    JLabel dataLabel;
    JTextField keyTField;
    JTextArea dataTArea;
    JComboBox databaseCombo;
    JButton cancelButton;
    JButton updateButton;
    JRadioButton queryButton;
    JRadioButton addButton;
    JRadioButton modifyButton;
    JRadioButton deleteButton;


    private static final String databaseFieldString = "Database";
    private static final String keyFieldString = "Key";
    private static final String dataFieldString = "Data";
    private static final String queryString = "Query";
    private static final String addString = "Add";
    private static final String modifyString = "Modify";
    private static final String deleteString = "Delete";

    //database status constants
    static int ADD = 1;
    static int MODIFY = 2;
    static int DELETE = 3;
    private static final int ADD_FAILURE = 0;
    private static final int MODIFY_FAILURE = 0;
    private static final int DELETE_FAILURE = 0;
    private static final int KEY_ALREADY_EXITS = 1;
    private static final int KEY_LENGTH_ERROR = 2;
    private static final int KEY_DATA_LENGTH_ERROR = 3;
    private static final int KEY_NOT_FOUND = 4;
    private static final int SUCCESS = 5;
    private static final int FAILURE = -1;

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Purpose:                 Constructs the dialog and adds the 
**                              components to the dialog
**    
**     Parameters:              JFrame frame - the parent frame on which the
**                                             dialog is displayed on.
**
**     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
**
******************************************************************************/
    
    public DbQAMD(JFrame frame) 
    {
        this.frame = frame;

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
 	dbQAMDDialog = new JDialog(frame, "Database", false);
	dbQAMDDialog.setSize(400, 360);
	dbQAMDDialog.setResizable(false);

	// Labels
	databaseLabel = new JLabel(databaseFieldString);
	keyLabel = new JLabel(keyFieldString);
	dataLabel = new JLabel(dataFieldString);
	

	// TextFields
	keyTField = new JTextField(15);
        keyTField.setActionCommand(keyFieldString);
        keyTField.addActionListener(this);


	// Combo
	databaseCombo = new JComboBox();
	databaseCombo.setEditable(false);

        /* add in list of databases to combo */
	File dbpath = new File(GTrace.DbDir);
	String dbs[] = dbpath.list(new DbFileFilter());
	for (int i=0; i<dbs.length; i++)
	{
	    databaseCombo.addItem(dbs[i]);
	}

        // Text area
        dataTArea= new JTextArea();
        JScrollPane dataTAreaScrollPane = new JScrollPane(dataTArea,
                                   JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                   JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        dataTAreaScrollPane.setPreferredSize(new Dimension(250, 100));

        
	// Buttons
        queryButton = new JRadioButton(queryString);
        queryButton.setMnemonic(KeyEvent.VK_Q);
        queryButton.setActionCommand(queryString);
        queryButton.setSelected(true);
        queryButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    dataTArea.setEditable(false);
                }
	});

        
        addButton = new JRadioButton(addString);
        addButton.setMnemonic(KeyEvent.VK_A);
        addButton.setActionCommand(addString);
        addButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    dataTArea.setText("");
                    dataTArea.setEditable(true);
                }
	});
        
        modifyButton = new JRadioButton(modifyString);
        modifyButton.setMnemonic(KeyEvent.VK_M);
        modifyButton.setActionCommand(modifyString);
        modifyButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    dataTArea.setText("");
                    dataTArea.setEditable(true);
                }
	});
        
        deleteButton = new JRadioButton(deleteString);
        deleteButton.setMnemonic(KeyEvent.VK_D);
        deleteButton.setActionCommand(deleteString);
        deleteButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    dataTArea.setEditable(false);
                }
	});
        
        // Group the radio buttons.
        ButtonGroup group = new ButtonGroup();
        group.add(queryButton);
        group.add(addButton);
        group.add(modifyButton);
        group.add(deleteButton);
        
	cancelButton = new JButton("Cancel");
	cancelButton.setMnemonic(KeyEvent.VK_C);
	cancelButton.setBorder(BorderFactory.createRaisedBevelBorder());
	cancelButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    dbQAMDDialog.setVisible(false);
                    dbQAMDDialog.dispose();
                }
	});

	updateButton = new JButton("Update");
	updateButton.setMnemonic(KeyEvent.VK_U);
	updateButton.setBorder(BorderFactory.createRaisedBevelBorder());
	updateButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	updateButton.addActionListener(this);


        JPanel dbPane = new JPanel();
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

        dbPane.setLayout(gridbag);

        // Put the radio buttons in a column in a panel
        JPanel radioPanel = new JPanel();
        radioPanel.setLayout(new FlowLayout());
        radioPanel.add(queryButton);
        radioPanel.add(addButton);
        radioPanel.add(modifyButton);
        radioPanel.add(deleteButton);

        // ROW 1
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
        c.gridwidth = 2;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(radioPanel, c);
	dbPane.add(radioPanel);

        
	// ROW 2
	c.gridx = 0;
	c.gridy = 1;
        c.gridwidth = 1;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(databaseLabel, c);
	dbPane.add(databaseLabel);

	c.gridx = 1;
	c.insets = new Insets(20,0,0,0); 
	gridbag.setConstraints(databaseCombo, c);
	dbPane.add(databaseCombo);

	// ROW 3
	c.gridx = 0;
	c.gridy = 2;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(keyLabel, c);
	dbPane.add(keyLabel);

	c.gridx = 1;
	c.ipady = 10;
	c.insets = new Insets(20,0,0,0); 
	gridbag.setConstraints(keyTField, c);
	dbPane.add(keyTField);

	
	// ROW 4 
	c.gridx = 0;
	c.gridy = 3;
	c.ipady = 0;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(20,20,0,10); 
	gridbag.setConstraints(dataLabel, c);
	dbPane.add(dataLabel);
	
	
	c.gridx = GridBagConstraints.RELATIVE;
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.insets = new Insets(20,0,0,0); 
	gridbag.setConstraints(dataTAreaScrollPane, c);
	dbPane.add(dataTAreaScrollPane);
	
	
	// ROW 5
	c.gridx = 1;
	c.gridy = 4;
	c.ipadx = 30;  
	c.ipady = 8;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,10,0,0); 
	gridbag.setConstraints(updateButton, c);
	dbPane.add(updateButton);

	c.anchor = GridBagConstraints.EAST;
	c.insets = new Insets(20,0,0,50); 
	gridbag.setConstraints(cancelButton, c);
	dbPane.add(cancelButton);

	dbQAMDDialog.getContentPane().add(dbPane);
	dbQAMDDialog.setLocationRelativeTo(frame);
	dbQAMDDialog.setVisible(true);
    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 10th, 1999
 **
 *****************************************************************************/
    
    public void actionPerformed(ActionEvent e)
    {

        if (queryButton.isSelected())
        {
            try
            {
                queryDb qdb = new queryDb();
                String record = qdb.getRecord(GTrace.DbDir
                                    + (String)databaseCombo.getSelectedItem(),
                                    keyTField.getText());

                if (record == null)
                {
                    JOptionPane.showMessageDialog(frame,
                                                  "Missing key field value",  
                                                  "Query Error",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }
                
                //record.replace('\t',' ');
                int linwidth[] = Util.getNumLinesMaxWidth(record);

                dataTArea.setEditable(false);
		dataTArea.setRows(linwidth[0]);
		dataTArea.setColumns(linwidth[1]);
                dataTArea.setText(record);
            }
            catch(DbException dbe)
            {
                dataTArea.setText("Error opening database");
            }
        }
        else if (addButton.isSelected())
        {
            try
            {
                amdDb amddb = new amdDb();
                int err = amddb.record(ADD,GTrace.DbDir
                                  + (String)databaseCombo.getSelectedItem(),
                                  keyTField.getText(), dataTArea.getText());
                switch(err)
                {
                case KEY_DATA_LENGTH_ERROR:
                    JOptionPane.showMessageDialog(frame,
                                             "Missing key/data field value",  
                                             "Add Error",
                                             JOptionPane.ERROR_MESSAGE);
                    break;
                case KEY_ALREADY_EXITS:
                    JOptionPane.showMessageDialog(frame,
                                           "The key specified already exists",
                                           "Add Error",
                                           JOptionPane.ERROR_MESSAGE);
                    break;
                case ADD_FAILURE:
                    JOptionPane.showMessageDialog(frame,
                                                  "Could not add record",  
                                                  "Add Error",
                                                  JOptionPane.ERROR_MESSAGE);
                    break;
                case SUCCESS:
                    JOptionPane.showMessageDialog(frame,
                                      "Record successfully added to database",
                                      "Record Added",
                                      JOptionPane.INFORMATION_MESSAGE);
                }
            }
            catch(DbException dbe)
            {
                dataTArea.setText("Error opening database");
            }

        }
        else if (modifyButton.isSelected()
                 && dataTArea.getText().length() == 0)
        {
            if (keyTField.getText().length() == 0)
            {
                JOptionPane.showMessageDialog(frame,
                                              "Missing key field value",  
                                              "Delete Error",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }

            try
            {
                queryDb qdb = new queryDb();
                String record = qdb.getRecord(GTrace.DbDir
                                     + (String)databaseCombo.getSelectedItem(),
                                     keyTField.getText());

                int linwidth[] = Util.getNumLinesMaxWidth(record);

                dataTArea.setEditable(true);
                dataTArea.setRows(linwidth[0]);
                dataTArea.setColumns(linwidth[1]);
                dataTArea.setText(record);
            }
            catch(DbException dbe)
            {
                dataTArea.setText("Error opening database");
            }
        }
        else if (modifyButton.isSelected() && dataTArea.getText().length() > 0)
        {
            try
            {
                amdDb amddb = new amdDb();
                int err = amddb.record(MODIFY,GTrace.DbDir
                                    + (String)databaseCombo.getSelectedItem(),
                                    keyTField.getText(), dataTArea.getText());
                switch(err)
                {
                case KEY_LENGTH_ERROR:
                    JOptionPane.showMessageDialog(frame,
                                                  "Missing key field value",  
                                                  "Modify Error",
                                                  JOptionPane.ERROR_MESSAGE);
                    break;
                case MODIFY_FAILURE:
                    JOptionPane.showMessageDialog(frame,
                                                  "Could not modify record",  
                                                  "Modify Error",
                                                  JOptionPane.ERROR_MESSAGE);
                    break;
                case SUCCESS:
                    JOptionPane.showMessageDialog(frame,
                                   "Record successfully modified in database",
                                   "Record Modified",
                                   JOptionPane.INFORMATION_MESSAGE);
                }
            }
            catch(DbException dbe)
            {
                dataTArea.setText("Error opening database");
            }
        }
        else if (deleteButton.isSelected())
        {
            try
            {
                amdDb amddb = new amdDb();
                int err = amddb.record(DELETE,GTrace.DbDir
                                    + (String)databaseCombo.getSelectedItem(),
                                    keyTField.getText(), dataTArea.getText());

                if (err == KEY_LENGTH_ERROR)
                {
                    JOptionPane.showMessageDialog(frame,
                                    "Missing key field value",  
                                    "Delete Error",
                                    JOptionPane.ERROR_MESSAGE);
                }
                else if (err == KEY_NOT_FOUND)
                {
                    JOptionPane.showMessageDialog(frame,
                                    "The key specified does not exist",  
                                    "Delete Error",
                                    JOptionPane.ERROR_MESSAGE);
                }
                else if (err == DELETE_FAILURE)
                {
                    JOptionPane.showMessageDialog(frame,
                                    "Could not delete record",  
                                    "Delete Error",
                                    JOptionPane.ERROR_MESSAGE);
                }
                else if (err == SUCCESS)
                {
                    JOptionPane.showMessageDialog(frame,
                                 "Record successfully deleted from database",  
                                 "Record Deleted",
                                 JOptionPane.INFORMATION_MESSAGE);
                }
            }
            catch(DbException dbe)
            {
                dataTArea.setText("Error opening database");
            }
        }
        
    }

//---------------------------------------------------------------------------
//                             inner classes
//--------------------------------------------------------------------------- 
    public class DbFileFilter implements FilenameFilter
    {
	public boolean accept(File f, String fname)
	{
	    // check the extension of regular files
	    if (fname.toLowerCase().endsWith(".db"))
	    {
		return true;
	    }

	    return false;
	}
    }

    public class queryDb extends DbEnv
    {

	public queryDb() throws DbException
	{
	    super(System.getProperty(GTrace.DbDir),null, Db.DB_THREAD);
	    
	    try 
	    {
		set_error_stream(System.err);
		set_errpfx("Database Query :");
	    }
	    catch (Exception e) 
	    {
		System.err.println(e.toString());
		//System.exit(1);
	    }
	}


	public String getRecord(String DbFileName, String skey)
                                                           throws DbException
	{
	    Db Dbptr = Db.open(DbFileName, Db.DB_HASH, Db.DB_RDONLY,
                               0644, this, null);

            if (!(skey.length() > 0))
            {
                return null;
            }

	    Dbt key = new Dbt(skey.getBytes());
	    key.set_size(skey.length());
	    Dbt data = new Dbt();
	    data.set_flags(Db.DB_DBT_MALLOC);

	    switch( Dbptr.get(null,key,data,0) )
	    {
            case 0: 
                String key_string = new String(key.get_data(), 0,
                                               key.get_size());
                String data_string = new String(data.get_data(), 0,
                                                data.get_size());
                Dbptr.close(0);
                return data_string.replace('\t',' '); 
		    
            case Db.DB_NOTFOUND:
                Dbptr.close(0);
                return "Not Found";

            default:
                Dbptr.close(0);
                return "Not Found";
	    }
	}
    }

    public class amdDb extends DbEnv
    {
        
	public amdDb() throws DbException
	{
            super(System.getProperty("user.dir"),null, Db.DB_THREAD);

            try 
            {
                set_error_stream(System.err);
                set_errpfx("Database :");
            }
            catch (Exception e) 
            {
                System.err.println(e.toString());
                System.exit(1);
            }
	}


	public int record(int op, String DbFileName, String skey, String sdata)
                                                             throws DbException
	{
            StringTokenizer tokenizer;
            String tabData;
            String token;
            Dbt key;
            Dbt data;
            
            Db Dbptr = Db.open(DbFileName, Db.DB_HASH, Db.DB_THREAD,
                               0644, this, null);

            switch(op)
            {
            case 1:  //ADD
                if (skey.length() == 0 || sdata.length() == 0)
                {
                    return KEY_DATA_LENGTH_ERROR;
                }

                tokenizer = new StringTokenizer(sdata);
		    
                try
                {
                    tabData = new String("");

                    while(tokenizer.hasMoreTokens())
                    {
                        token = tokenizer.nextToken() + "\t"; 
                        tabData = tabData.concat(token);
                    }
			
                    key = new Dbt(skey.getBytes());
                    key.set_size(skey.length());
                    data = new Dbt(tabData.getBytes());
                    data.set_size(tabData.length());

                    try
                    {
                        int err;
                        if ((err = Dbptr.put(null, key, data, 0))
                                                            == Db.DB_KEYEXIST) 
                        {
                            System.err.println("Key " + key
                                                         + " already exists.");
                            return KEY_ALREADY_EXITS;
                        }
                    }
                    catch (DbException dbe)
                    {
                        System.err.println(dbe.toString());
                        return ADD_FAILURE;
                    }
                }
                catch(NoSuchElementException nsee) { }

                Dbptr.close(0);
                return SUCCESS;
            case 2: // MODIFY

                tokenizer = new StringTokenizer(sdata);
                tabData = new String("");		    

                try
                {
                    while(tokenizer.hasMoreTokens())
                    {
                        token = tokenizer.nextToken() + "\t"; 
                        tabData = tabData.concat(token);
                    }
			
                    key = new Dbt(skey.getBytes());
                    key.set_size(skey.length());
                    data = new Dbt(tabData.getBytes());
                    data.set_size(tabData.length());

                    try
                    {
                        Dbptr.del(null, key, 0);
                        Dbptr.put(null, key, data, 0);
                    }
                    catch (DbException dbe)
                    {
                        System.err.println(dbe.toString());
                        return MODIFY_FAILURE;
                    }
                }
                catch(NoSuchElementException nsee) { }

                Dbptr.close(0);
                return SUCCESS;

            case 3: //DELETE

                if (skey.length() ==0)
                {
                    return KEY_LENGTH_ERROR;
                }

                try
                {
                    queryDb qdb = new queryDb();
                    String record = qdb.getRecord(DbFileName, skey);

                    int linwidth[] = Util.getNumLinesMaxWidth(record);

                    dataTArea.setEditable(false);
                    dataTArea.setRows(linwidth[0]);
                    dataTArea.setColumns(linwidth[1]);
                    dataTArea.setText(record);
                }
                catch(DbException dbe)
                {
                    dataTArea.setText("Error opening database");
                }

                if (dataTArea.getText().compareTo("Not Found") != 0)
                {
                    Object[] options = {"Delete",
                                        "Cancel"};
                    int n = JOptionPane.showOptionDialog(frame,
                               "Are you sure you want to delete this Record?",
                               "Delete Record",
                               JOptionPane.YES_NO_OPTION,
                               JOptionPane.QUESTION_MESSAGE,
                               null,
                               options,
                               options[1]);

                    if (n == 0)
                    {
                        key = new Dbt(skey.getBytes());
                        key.set_size(skey.length());

                        Dbptr.del(null, key, 0);
                    }

                    Dbptr.close(0);
                    return SUCCESS;
                }
            }
            return FAILURE;
        }
    }
}
