/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    DbRecord.java
**
**     Purpose:                 Used to store contents of a Database record
**                              
**     Author/Date:             Ram Periakaruppan, Oct. 17th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.io.Serializable;

public class DbRecord implements Serializable
{
    String name;
    String key;
    String record;
    
/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables to the
 **                              parameters specified.
 **    
 **     Parameters:              String name - name of database
 **                              String key - the key
 **                              String record - the data
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public DbRecord(String name, String key, String record)
    {
	this.name = name;
	this.key = key;
	this.record = record;
    }

/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables to the
 **                              parameters specified.
 **    
 **     Parameters:              String key - the key
 **                              String record - the data
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public DbRecord(String key, String record)
    {
	this.name = null;
	this.key = key;
	this.record = record;
    }

//---------------------------------------------------------------------------
//                             setxxx Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Method name:             setName
 **
 **     Purpose:                 set the database/textfile name
 **    
 **     Parameters:              String name - db/txt file name
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public void setName(String name)
    {
	this.name = name;
    }

//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Method name:             getName
 **
 **     Purpose:                 returns the database/textfile name
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String 
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public String getName()
    {
	return name;
    }

/******************************************************************************
 **
 **     Method name:             getKey
 **
 **     Purpose:                 returns the key
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - key
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public String getKey()
    {
	return key;
    }

/******************************************************************************
 **
 **     Method name:             getRecord
 **
 **     Purpose:                 returns the data assoiciated with the key
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - the data record
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public String getRecord()
    {
	return record;
    }

/******************************************************************************
 **
 **     Method name:             toString
 **
 **     Purpose:                 returns a string representation of the
 **                              contents of this object.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public String toString()
    {
	String retString;

	retString = new String(
			       "Database/Textfile Details:\n\n" +
			       "Name\t: " + name + "\n" +
			       "Key\t: " + key + "\n" + 
			       "Data\t: " + record.replace('\t',' ') + "\n");
	
	return retString;
    }
}
	
    
