/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    DomainParsingFile.java
**
**     Purpose:                 
**                              
**     Author/Date:             Ram Periakaruppan, Sept 15th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.util.Vector;
import java.util.Hashtable;

import java.io.RandomAccessFile;

import com.oroinc.text.regex.*;
import com.sleepycat.db.DbException;

public class DomainParsingFile
{
    private String fileName;
    private String className;
    private Vector regexs = new Vector();
    private Hashtable data = new Hashtable();

    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Purpose:                 
 **    
 **     Parameters:             
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 16th, 1999
 **
 *****************************************************************************/

    public DomainParsingFile(String className, String fileName)
    {
	this.className = className;
	this.fileName = fileName;
    }

//---------------------------------------------------------------------------
//                             public utility Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Method name:             regexParser
 **
 **     Purpose:                 
 **    
 **     Parameters:              String regex
 **                              String input
 **                              String sub
 **
 **     Return Values:           String
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 16th, 1999
 **
 *****************************************************************************/

    public String regexParser(String regex, String input, String sub)
    {
        int limit, interps;
        PatternMatcher matcher = new Perl5Matcher();
        Pattern pattern = null;
        PatternCompiler compiler = new Perl5Compiler();
        String result;
	 
        limit = com.oroinc.text.regex.Util.SUBSTITUTE_ALL;
        interps = com.oroinc.text.regex.Util.INTERPOLATE_ALL; 
	 
        try
        {
            pattern = compiler.compile(regex);
        }
        catch(MalformedPatternException mpe)
        {
            // indicate bad pattern to user or write to a file the error 
            // message that the pattern generated.
            return null;
        }    
          
        result = com.oroinc.text.regex.Util.substitute(matcher, pattern, 
                                          new Perl5Substitution(sub, interps),
                                          input, limit); 
        return result.trim();
    }

/******************************************************************************
 **
 **     Method name:             readDomainParsingFile
 **
 **     Purpose:                 
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **
 **     Side Effects:            throws Exception
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 16th, 1999
 **
 *****************************************************************************/

    private void readDomainParsingFile() throws Exception
    {
	String line;
	boolean dataSection = false;
	String dataTokens[];
	RandomAccessFile fptr = new RandomAccessFile(fileName, "r");

	while ((line = fptr.readLine()) != null)
	{
	    if (!dataSection && line.startsWith("#Data"))
	    {
		dataSection = true;
		continue;
	    }

	    if (!line.startsWith("#") && line.length() > 0)
	    {
		if (!dataSection)
		{
		    regexs.addElement(line);
		}
		else
		{
		    dataTokens = Util.tokenizeToString(line,"=");
		    if (dataTokens.length == 2)
		    {
			data.put(dataTokens[0],dataTokens[1]);
		    }
		}
	    }
	}
    }
	

/******************************************************************************
 **
 **     Method name:             lookup
 **
 **     Purpose:                 
 **    
 **     Parameters:              String hostname
 **
 **     Return Values:           RegexRecord
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 16th, 1999
 **
 *****************************************************************************/
    
    public RegexRecord lookup(String hostname)
    {
	String result;
	DbRecord dbRecord = null;
	String regex;
	String loc = new String("");

	if (regexs.isEmpty())
	{
	    try
	    {
		readDomainParsingFile();
	    }
	    catch(Exception e)
	    {
		return null;
	    }
	}
	
	int size = regexs.size();
	for (int i=0; i<size; i++)
	{
	    regex = ((String)regexs.elementAt(i)).toLowerCase();
  if (regex.startsWith("s/"))
  {
      regex = regex.substring(1);
      String regexTokens[] = Util.tokenizeToString(regex,"/");

      if (regexTokens.length == 3)
      {
          String dbTokens[] = Util.tokenizeToString(regexTokens[2],",");
		    
          result = regexParser(regexTokens[0],hostname,regexTokens[1]);
          //System.err.println("Result - " + result);
          for (int j=0; j<dbTokens.length; j++)
          {
              if (dbTokens[j].compareTo("this") == 0)
              {
                  loc = (String)data.get(result);
                  //System.err.println("loc = " + loc);
                  if (loc != null)
                  {
                      if (loc.startsWith("L:") || loc.startsWith("l:"))
                      {
                          return new RegexRecord(regex, result, loc);
                      }
                      else
                      {
   try
   {
       if (className.compareTo("GTrace") == 0)
       {
           dbRecord = ((DbTxt)GTrace.dbtxts.get(GTrace.CITIES)).lookup(loc);
       }
       else if (className.compareTo("LookupServer") == 0)
       {
           dbRecord = ((DbTxt)LookupServer.dbtxts.get(
                                            LookupServer.CITIES)).lookup(loc);
       }
					
   }
   catch(DbException dbe) { }
				    
                          if (dbRecord != null)
                          {
                              return new RegexRecord(regex, result,
                                                     loc, dbRecord);
                          }
                      }
                  }
              }
              else if (dbTokens[j].compareTo("country") == 0)
              {
                  return new RegexRecord(regex, result, "country:"+result);
              }
              else
              {
  try
  {
      if (className.compareTo("GTrace") == 0)
      {
          dbRecord = ((DbTxt)GTrace.dbtxts.get(dbTokens[j])).lookup(result);
      }
      else if (className.compareTo("LookupServer") == 0)
      {
          dbRecord = ((DbTxt)LookupServer.dbtxts.get(
                                                dbTokens[j])).lookup(result);
      }
  }
  catch(DbException dbe) { }
			    
                  if (dbRecord != null)
                  {
                      return new RegexRecord(regex, result, "", dbRecord);
                  }
              }
          }
      }
  }
	}
	
	return null;
    }
}
