/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    GTrace.java
**
**     Purpose:                 This is the main GTrace application file. 
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**      The Graphical User Interface code is based on the SwingSet example 
**          that comes with the JFC/Swing distribution.
**
******************************************************************************/
package caida.GTrace;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.PixelGrabber;

import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.io.RandomAccessFile;
import java.io.FileOutputStream;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.MissingResourceException;
import java.util.ResourceBundle;


public class GTrace extends JPanel implements ActionListener
{
    // The Frame
    public static JFrame frame;

    // The width and height of the frame
    public static int WIDTH = 600;
    public static int HEIGHT = 450;
    public static int INITIAL_WIDTH = 400;
    public static int INITIAL_HEIGHT = 200;

    // Track progress
    public static int totalPanels = 9;
    public static int currentProgressValue;
    public static JLabel progressLabel = null;
    public static JProgressBar progressBar = null;

    // Resources 
    public static char separator = File.separatorChar;
    private static Hashtable maps = new Hashtable();
    private static ResourceBundle mapResource;    
    private static ResourceBundle gtraceResource;    

    // Resource Bundle Constants
    private final String PARENT = new String(".PARENT");
    private final String CHILDREN = new String(".CHILDREN");
    private final String BOUNDARY = new String(".BOUNDARY");
    private final String BGCOLOR = new String(".BGCOLOR");
    private final String FGCOLOR = new String(".FGCOLOR");
    private final String PROJECTION = new String(".PROJECTION");

    // RootMap and CurrentMap
    private MapImage rootMap;     //startup Map
    public static JLabel curMap;    

    // Panel Components
    private JScrollPane routeScrollPane;
    private JSplitPane splitPane;    
    private JComboBox urlList;
    private JLabel latlonLabel;
    private JLabel latlonValue;
    public static JTextField statusBar;
    public static JScrollPane mapScrollPane;
    public static ComboModel comboModel;
    public static JButton resetButton;
    public Font defaultFont = new Font("Dialog", Font.PLAIN, 12);

    // the table model
    public GTraceTableModel dataModel;
    private JLabelIconRenderer iconColorWhite;
    static int count = 0;
    private int flag = 0;	
    static JTable table;

    // the colors of the lines, background and foreground of maps
    private static final Color map_background = new Color(192,192,192);
    private static final Color map_lines = new Color(128,128,128);
    private static final Color map_foreground = new Color(255,255,255);

    
    static Vector latlons = new Vector();
    boolean ComboMenuDown = false;
    boolean tracing=false;
    public static int currentThreadGroupNumber = 0;
    ThreadGroup traceThreadGroup = null;
    ParseTrace parseTrace = null;
    public static Cache cache;
    

    // the options menu item flags
    public static boolean status_bar=false;
    public static boolean showDistances = false;
    public boolean showToolTips = true;
    public boolean locationLegendVisible = false;
    public boolean mapLineLegendVisible = false;
    public static boolean disableVerifier = false;

    //Files
    public static String HOMEDIR;
    private String DPFSDIR;
    public static String cacheFile;
    public static String comboFile;
    public static String errorFile;
    public static Hashtable dbtxts = new Hashtable();
    public static Hashtable dpfs = new Hashtable();

    //Dialogs
    public static whois whoisDialog;
    private JDialog nslookupDialog;
    private JDialog pingDialog;
    private static LDetailDialog ldetailDialog;
    private JDialog locationLegendDialog;
    private JDialog mapLineLegendDialog;

    //Country center latlons
    public static Hashtable country_latlon = new Hashtable();

    //Colored balls
    public ImageIcon balls[] = new ImageIcon[6];
    public ImageIcon lines[] = new ImageIcon[4];

    //constants
    public static int NUMOFRTTS = 3;
    public static final int FAILURE = -1;
    public static final int CACHESIZE = 250;
    public static String DbDir;
    public static String VERSION;
    public static String AIRPORT;
    public static String CITIES;

    // Table column names
    public final static String PLOT = new String("Plot");
    public final static String HOP = new String("Hop");
    public final static String IPADDR = new String("IP Address");
    public final static String INTERFACE = new String("Interface Name");
    public final static String LOCATION = new String("Location");
    public final static String RTT = new String("RTT (ms)");

    // Internal Use --> Table Columns Numbers
    private final static int nPLOT = 0;
    private final static int nHOP = 1;
    private final static int nIPADDR = 2;
    private final static int nINTERFACE = 3;
    private final static int nLOCATION = 4;
    private final static int nRTT = 5;

    // The various search steps
    public static final int STEP2 = 20;
    public static final int STEP3 = 30;
    public static final int STEP4 = 40;
    public static final int STEP4STATE = 41;
    public static final int STEP4COUNTRY = 42;
    public static final int STEP5 = 50;
    public static final int STEP6 = 60;
    public static final int STEP7 = 70;

/*---------------------------------------------------------------------------
 *  check if both config files exit otherwise quit application.
 *---------------------------------------------------------------------------*/

    static 
    {
        try 
	{
            gtraceResource = ResourceBundle.getBundle("GTrace");
        } 
	catch (MissingResourceException mre) 
	{
            System.err.println("GTrace.properties not found");
            System.exit(FAILURE);
        }

        try 
	{
            mapResource = ResourceBundle.getBundle("maps.GTraceMaps");
        } 
	catch (MissingResourceException mre) 
	{
            System.err.println("GTraceMaps.properties not found");
            System.exit(FAILURE);
        }
    }

/******************************************************************************
**
**     Purpose:                 Initializes and creates the gtrace frame
**    
**     Parameters:              NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public GTrace()
    {    
	setFont(defaultFont);
	
	setBorder(BorderFactory.createEtchedBorder());
	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	
	currentProgressValue = 0;
	
	progressLabel.setText("Loading GTrace.properties");
	initGTrace();
	progressBar.setValue(++currentProgressValue);
	
	progressLabel.setText("Loading GTraceMaps.properties"); 
	initGTraceMaps();
	progressBar.setValue(++currentProgressValue);
	
	progressLabel.setText("Loading Maps....."); 
	loadMaps();
	progressBar.setValue(++currentProgressValue);
	
	progressLabel.setText("Loading Cache.....");
	loadCache();
	progressBar.setValue(++currentProgressValue);
	
	progressLabel.setText("Loading Traceroute Destination History.....");
	loadComboHistory();
	progressBar.setValue(++currentProgressValue);

	progressLabel.setText("Loading Images.....");
	loadImages();
	progressBar.setValue(++currentProgressValue);
	
	progressLabel.setText("Creating Menus"); 
	frame.setJMenuBar(new MenuBar(this,frame));
	progressBar.setValue(++currentProgressValue);

	curMap = new JLabel( rootMap );
	
	PictureListener pictureListener = new PictureListener();
	mapScrollPane = new JScrollPane( curMap );			  
	mapScrollPane.addMouseMotionListener(pictureListener);
	mapScrollPane.addMouseListener(pictureListener);           

	JLabel urlLabel = new JLabel( "Host/URL to Trace" );

	urlList = new JComboBox(comboModel);		
	urlList.setPreferredSize( new Dimension(250, 
					 urlList.getPreferredSize().height) );
	urlList.setEditable(true);        
	urlList.setEnabled(true);
	urlList.setMaximumRowCount(10);
	urlList.addActionListener(new ComboBoxListener());

	latlonLabel = new JLabel("Lat/Lon :");
	latlonValue = new JLabel("");
	
	JPanel urlPanel = new JPanel();        		
	BoxLayout urlLayout = new BoxLayout(urlPanel, BoxLayout.X_AXIS); 
	urlPanel.setLayout(urlLayout);
	urlPanel.setBorder(BorderFactory.createRaisedBevelBorder());
	urlPanel.add(Box.createHorizontalStrut(5));
	urlPanel.add(urlLabel);        		
	urlPanel.add(Box.createHorizontalStrut(5));
	urlPanel.add(urlList);        
	
	JPanel latlonPanel = new JPanel();
	BoxLayout latlonLayout = new BoxLayout(latlonPanel, BoxLayout.X_AXIS); 
	latlonPanel.setLayout(latlonLayout);
	latlonPanel.setBorder(BorderFactory.createRaisedBevelBorder());
	latlonPanel.add(Box.createHorizontalStrut(5));
	latlonPanel.add(latlonLabel);        		
	latlonPanel.add(Box.createHorizontalStrut(5));
	latlonPanel.add(latlonValue);        
	
	JPanel topPane = new JPanel();
	topPane.setLayout(new BorderLayout());
	topPane.add(urlPanel, BorderLayout.WEST);
	topPane.add(latlonPanel, BorderLayout.CENTER);
	topPane.setMaximumSize(new Dimension(Integer.MAX_VALUE, 
					     topPane.getMinimumSize().height));
	
	dataModel = new GTraceTableModel();				
	table = new JTable(dataModel);

	iconColorWhite = new JLabelIconRenderer(); 

	table.setDefaultRenderer(JLabel.class, 
				 new JLabelRenderer(frame.getContentPane()));
	setUpJLabelEditor(table);
	
	TableColumn column = null;
	
	for (int i = 0; i < 6; i++) 
	{
	    column = table.getColumnModel().getColumn(i);
	    if (i == 0) 
	    {
		column.setPreferredWidth(30); 
		column.setMaxWidth(30);
	    } 
	    else if(i == 1)
	    {
		column.setPreferredWidth(50);         
		column.setMaxWidth(50);         
	    }
	    else 
	    {
		column.setPreferredWidth(100);
	    }
	}

	routeScrollPane = new JScrollPane(table);

	splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
	splitPane.setTopComponent(mapScrollPane);
	splitPane.setBottomComponent(routeScrollPane);
	splitPane.setOneTouchExpandable(true);		
	splitPane.setDividerLocation(250);
	splitPane.setDividerSize(10);				
	
	JPanel splitPanel = new JPanel(new BorderLayout()); 
	splitPanel.add(splitPane,"Center");
	
	add(topPane);
	add(splitPanel);	
	frame.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));

        resetButton = new JButton();
	resetButton.addActionListener(this);

	statusBar = new JTextField();
	statusBar.setBorder(BorderFactory.createCompoundBorder(
			    BorderFactory.createRaisedBevelBorder(),
			    BorderFactory.createLoweredBevelBorder()));
	statusBar.setEditable(false);
    }     
    
//---------------------------------------------------------------------------
//                             init Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             initGTrace
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    private void initGTrace()
    {
	VERSION = new String("GTrace v" + gtraceResource.getString("VERSION"));

	HOMEDIR = gtraceResource.getString("HOMEDIR");
	cacheFile = new String(HOMEDIR + separator + "log" 
			       + separator + "GTraceCache");
	comboFile = new String(HOMEDIR + separator + "log" 
			       + separator + "GTraceCombo");
	errorFile = new String(HOMEDIR + separator + "log" 
			       + separator + "GTraceError"); 
	DbDir = new String(HOMEDIR + gtraceResource.getString("DATABASES") 
			   + separator);
	DPFSDIR = new String(gtraceResource.getString("DOMAINFILES"));

	File f = new File(HOMEDIR + DPFSDIR);

	String files[] = f.list();

	for (int i=0; i<files.length; i++)
	{
	     dpfs.put(files[i],new DomainParsingFile("GTrace",HOMEDIR+DPFSDIR
						    +separator+files[i]));
	}

	AIRPORT = gtraceResource.getString("AIRPORTS");
	CITIES = gtraceResource.getString("CITIES");
	dbtxts.put(CITIES, new DbTxt(DbDir, 
				     gtraceResource.getString("CITIES")));
	dbtxts.put(AIRPORT, new DbTxt(DbDir, AIRPORT));

	String dbs[] = Util.tokenizeToString(
				         gtraceResource.getString("DBS"),";");
	parseDbTxts(dbs);
	String txts[] = Util.tokenizeToString(
				   gtraceResource.getString("TEXTFILES"),";");
	parseDbTxts(txts);

	String traceroute = gtraceResource.getString("TRACEROUTE");
	int qindex;
	try
	{
	    if ( (qindex=traceroute.indexOf("-q")) > 0)
	    {
		NUMOFRTTS = new Integer(
   	                  traceroute.substring(qindex).trim()).intValue();
	    }
	}
	catch(NumberFormatException nfe) { }

	try
	{
	    String line;
	    String linetokens[] = new String[3];
	    RandomAccessFile fptr = new RandomAccessFile(DbDir
							 +"country.txt","r");
	    while((line = fptr.readLine()) != null)
	    {
		linetokens = Util.tokenizeToString(line);
		country_latlon.put(linetokens[0],
				   new LatLon(linetokens[1],linetokens[2]));
	    }
	}
	catch(Exception e)
	{

	}
    }



/******************************************************************************
**
**     Method name:             initGTraceMaps
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
**
******************************************************************************/

    private void initGTraceMaps()
    {
	MapImage map;
	float mapBoundary[];
	String mapParent;
	String mapChildren[];
	String mapFGColor;
	String mapBGColor;
	String mapProjection;
	String listOfMaps[] = Util.tokenizeToString(
					  mapResource.getString("MAPS"),",");

	for (int i=0; i<listOfMaps.length; i++)
	{
	    try
	    {
		mapParent = getMapResource(listOfMaps[i] + PARENT);
		mapChildren = Util.tokenizeToString(
			        getMapResource(listOfMaps[i] + CHILDREN),","); 
								   
		mapBoundary = Util.tokenizeTofloat(
                                getMapResource(listOfMaps[i] + BOUNDARY),",");
								  
		mapFGColor = getMapResource(listOfMaps[i] + FGCOLOR); 
		mapBGColor = getMapResource(listOfMaps[i] + BGCOLOR); 
		mapProjection = getMapResource(listOfMaps[i] + PROJECTION); 
		map = new MapImage(listOfMaps[i],mapBoundary,mapParent,
				   mapChildren,mapFGColor,mapBGColor,
				   mapProjection);
		maps.put(listOfMaps[i],map);
	    }
	    catch(NumberFormatException nfe)
	    {
		//write listOfMaps[i] to GTraceError.log
	    }
	}
    }

//---------------------------------------------------------------------------
//                             private Utility Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
**
**     Method name:             parseDbTxts
**
**     Purpose:                 
**    
**     Parameters:              String dbs[]
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
**
******************************************************************************/

    private void parseDbTxts(String dbs[])
    {
	String item[] = new String[2];

	for (int i=0; i<dbs.length; i++)
	{
	    item = Util.tokenizeToString(dbs[i],",");
	    
	    if (item[1].compareTo("hostnames/ipaddr") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], 
					      DbTxt.HOSTNAMEIPADDR));
	    }
	    else if (item[1].compareTo("hostnames") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], DbTxt.HOSTNAME));
	    }
	    else  if (item[1].compareTo("ipaddr") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], DbTxt.IPADDR));
	    }
	}
    }
    

//---------------------------------------------------------------------------
//                             public utility Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             redoTrace
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 23th, 1999
**
******************************************************************************/

    public void redoTrace()
    {
	String dest = (String)comboModel.getSelectedItem();
	if (dest != null && dest.length() > 0)
	{
	    dataModel.clearTable();
	    currentThreadGroupNumber++;
	    latlons.removeAllElements();
	    MapImage.latlonValues.removeAllElements();
	    mapScrollPane.getViewport().getView().repaint();

	    try
	    {
		traceThreadGroup.destroy();
	    }
	    catch(IllegalThreadStateException itse) { }
	    catch(NullPointerException npe) { }
	
	    traceThreadGroup = new ThreadGroup("group"+
					       currentThreadGroupNumber);   
	    //parseTrace.killProcess();  
	    parseTrace = new ParseTrace("ParseThread", dest, 
					dataModel, traceThreadGroup);
	}
    }
    
/******************************************************************************
**
**     Method name:             readTraceFromFile
**
**     Purpose:                 
**    
**     Parameters:              String fname
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
**
******************************************************************************/

    public void readTraceFromFile(String fname)
    {
	dataModel.clearTable();
	currentThreadGroupNumber++;
	latlons.removeAllElements();
	MapImage.latlonValues.removeAllElements();
	mapScrollPane.getViewport().getView().repaint();
	
	try
	{
	    traceThreadGroup.destroy();
	}
	catch(IllegalThreadStateException itse) { }
	catch(NullPointerException npe) { }
	
	traceThreadGroup = new ThreadGroup("group"+currentThreadGroupNumber);

        if (fname.endsWith(".gtr"))
        {
            try
            {
                FileInputStream in = new FileInputStream(fname);
                ObjectInputStream s = new ObjectInputStream(in);
                SaveTrace st = (SaveTrace)s.readObject();
                setMap(st.getMapName());

                st.path = (Hashtable)s.readObject();

                int rows=st.getNumHops();

                for (int i=1; i<=rows; i++)
                {
                    dataModel.addRow(i, st.getRow(i-1),
                                     st.getTimeColor(i-1),
                                     st.getLocationColor(i-1),
                                     st.getLocationDetails(i-1),
                                     st.getStep(i-1),traceThreadGroup.getName());
                }
            }
            catch(Exception e) 
            { 
                System.err.println(e.toString());
            }
        }
        else
        {
            parseTrace = new ParseTrace(fname, dataModel, traceThreadGroup);
        }
    }
    
/******************************************************************************
**
**     Method name:             saveTraceToFile
**                              
**     Purpose:                 
**    
**     Parameters:              String fname
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
**
******************************************************************************/

    public void saveTraceToFile(String fname)
    {
	MapImage mapImage = (MapImage)curMap.getIcon();

        SaveTrace st = new SaveTrace(mapImage.getMapLabelFileName(),
                                     dataModel);

        try
        {
            FileOutputStream out = new FileOutputStream(fname);
            ObjectOutputStream s = new ObjectOutputStream(out);
            s.writeObject(st);
            s.writeObject(st.path);
            s.flush();
            s.close();
            out.close();
        }
        catch(Exception ep)
        {
            JOptionPane.showMessageDialog(frame,
               "Could not save the trace. There was an error creating\n" +
               "the file " + fname +
               "\nCheck to make sure that you have permission to\n" +
               "write to the directory you want to save in.",      
                   "Save Trace error",
                   JOptionPane.ERROR_MESSAGE);
        }
    }

/******************************************************************************
**
**     Method name:             showLocationLegend
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Nov. 2nd, 1999
**
******************************************************************************/


    public void showLocationLegend()
    {
        if (!locationLegendVisible)
        {
            locationLegendVisible = true;
            new LocationLegend();
        }
    }

/******************************************************************************
**
**     Method name:             showMapLineLegend
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Nov. 2nd, 1999
**
******************************************************************************/


    public void showMapLineLegend()
    {
        if (!mapLineLegendVisible)
        {
            mapLineLegendVisible = true;
            new MapLineLegend();
        }
    }
    
/******************************************************************************
**
**     Method name:             redrawMap
**
**     Purpose:
**
**     Parameters:              NONE
**
**     Return Values:           void
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public static void redrawMap()
     {
          Component c = mapScrollPane.getViewport().getView();
          c.repaint();
     }


//---------------------------------------------------------------------------
//                             load Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             loadCache
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    private void loadCache()
    {
	try
	{
	    FileInputStream in = new FileInputStream(cacheFile);
	    ObjectInputStream s = new ObjectInputStream(in);
	    cache = (Cache)s.readObject();
	    cache.cache = (Hashtable)s.readObject();
	    return;
	}
	catch(Exception e) 
	{ 
	    cache = new Cache(CACHESIZE);
	}
    }

/******************************************************************************
**
**     Method name:             loadComboHistory
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    private void loadComboHistory()
    {
	comboModel = new ComboModel(10);
	String line;
	
	try
	{
	    RandomAccessFile fptr = new RandomAccessFile(GTrace.comboFile,"r");
	    
	    while ((line = fptr.readLine()) != null)
	    {
		line.replace('\n',' ');
		comboModel.addComboHistory(line);
	    }
	    return;
	}
	catch(Exception e) { }
    }

/******************************************************************************
**
**     Method name:             loadImages
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 6th, 1999
**
******************************************************************************/

    private void loadImages()
    {
	balls[0] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "gr_ball.gif");
	balls[1] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "yl_ball.gif");
	balls[2] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "bl_ball.gif");
	balls[3] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "cy_ball.gif");
	balls[4] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "rd_ball.gif");
	balls[5] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "wh_ball.gif");

        lines[0] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "green_ln.gif");
	lines[1] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "cyan_ln.gif");
	lines[2] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "blue_ln.gif");
	lines[3] = new ImageIcon(HOMEDIR + separator + "images" + separator
                                 + "red_ln.gif");

    }

/******************************************************************************
**
**     Method name:             loadMaps
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
**
******************************************************************************/

    public void loadMaps()
    {
	rootMap = getMap(getMapResource("STARTUPMAP"));
	if (rootMap != null)
	{
	    rootMap.loadMap();
            progressBar.setValue(++currentProgressValue);
            
	    if ( rootMap.getMapName().compareTo("World50.gif") == 0)
	    {
		MapImage worldMap  = getMap("World100.gif");
		MapImage europeMap = getMap("Europe.gif"); 
		if (worldMap != null)
		{
		    worldMap.loadMap();
                    progressBar.setValue(++currentProgressValue);
		}
		if (europeMap != null)
		{
		    europeMap.loadMap();
		}
	    }
	}
	else
        {
	    System.err.println("GTrace : Error loading STARTUPMAP");
	    System.exit(-1);
	}
    }

//---------------------------------------------------------------------------
//                             ActionListener Interface
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/

    public void actionPerformed(ActionEvent e) 
    {
	dataModel.clearTable();
	latlons.removeAllElements();
	MapImage.latlonValues.removeAllElements();
	mapScrollPane.getViewport().getView().repaint();
	currentThreadGroupNumber++;
	urlList.setSelectedItem(new String(""));
	urlList.requestFocus();
    }
    

//---------------------------------------------------------------------------
//                             status bar Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             displayStatusBar
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public static void displayStatusBar()
    {
	statusBar.setText("");
	frame.getContentPane().add(statusBar, BorderLayout.SOUTH);
	frame.getContentPane().validate();
    }

/******************************************************************************
**
**     Method name:             removeStatusBar
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public static void removeStatusBar()
    {
	frame.remove(statusBar);
	frame.getContentPane().validate();
    }

/******************************************************************************
**
**     Method name:             displayTextSBar
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public static void displayTextSBar(String text)
    {
	if (status_bar)
	{
	    statusBar.setText(text);
	}
    }

/******************************************************************************
**
**     Method name:             statusBarUpdate
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public static void statusBarUpdate()
    {
	if (status_bar)
	{
	    status_bar = false;
	    removeStatusBar();
	}
	else
	{
	    status_bar = true;
	    displayStatusBar();
	}
    }

//---------------------------------------------------------------------------
//                             display Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             displayErrorDialog
**
**     Purpose:                 
**    
**     Parameters:              String msg
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    public static void displayErrorDialog(String msg)
    {
	JOptionPane.showMessageDialog(frame,
				      msg,
				      "",
				      JOptionPane.ERROR_MESSAGE);
    }


//---------------------------------------------------------------------------
//                             setxxx Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             setMap
**
**     Purpose:                 
**    
**     Parameters:              String MapName
**
**     Return Values:           void 
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 4th, 1999
**
******************************************************************************/

    public static void setMap(String MapName)
    {
	MapImage childImage = getMap(MapName+".gif");

        if (childImage == null)
        {
            childImage = getMap(MapName+".jpg");
        }
        
	if (childImage != null)
	{
	    frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
	    childImage.loadMap(); 
	    curMap.setIcon(childImage);
	    frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
    }


/******************************************************************************
**
**     Method name:             setUpJLabelEditor (class GTracePanel)
**
**     Purpose:                 
**    
**     Parameters:              JTable table
**
**     Return Values:           void
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     private void setUpJLabelEditor(JTable table) 
     {

	  final JButton button = new JButton() 
	  {
	       public void setText(String s) 
	       {
		    super.setText(s);
	       }
	  };
	 
	  final ipaddr_host_loc_Editor iphnlocEditor = 
	                                    new ipaddr_host_loc_Editor(button);
	  table.setDefaultEditor(JLabel.class, iphnlocEditor);
	
	  button.addActionListener(new ActionListener() 
	  {
	       public void actionPerformed(ActionEvent e) 
	       {
		   String event = e.getActionCommand();

		   if(event.compareTo(IPADDR) == 0)
		   {
		       whoisDialog.showDialog(Util.ipaddrToString(
						      button.getText()),false);
		   }
		   else if(event.compareTo(INTERFACE) == 0)
		   {
		       whoisDialog.showDialog(Util.getDomain(
						       button.getText()),true);
		   }
		   else if(event.startsWith(LOCATION))
		   {
		       String tokens[] = Util.tokenizeToString(event,"-");
		       int row = Util.stringToint(tokens[1]);
		       ldetailDialog.showDialog(
			  (GTraceTableModel.getLocationDetails(
                                                             row)).toString());
		   }
	       }
	  });
     }

/******************************************************************************
**
**     Method name:             incCurrentThreadGroupNumber
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
**
******************************************************************************/

    public void incCurrentThreadGroupNumber()
    {
	currentThreadGroupNumber++;

        if (parseTrace != null)
        {
            parseTrace.killProcess();
        }
    }


//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public static boolean getVerifierStatus()
    {
	return disableVerifier;
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
**
******************************************************************************/

    public static LatLon getCountryLatLon(String countryCode)
    {
	return (LatLon)country_latlon.get(countryCode.trim().toLowerCase());  
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    public static Vector getItems()
    {
	return comboModel.getItems();
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    public static String getCurrentGroupNumber()
    {
	return new String("group"+currentThreadGroupNumber);
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    public static String getGTraceResource(String key)
    {
	String data;

	try
	{
	    data = gtraceResource.getString(key);
	}
	catch(MissingResourceException mre)
	{
	    return null;
	}

	return data;
    }



/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 6th, 1999
**
******************************************************************************/

    private static String getMapResource(String key)
    {
	String data;

	try
	{
	    data = mapResource.getString(key);
	}
	catch(MissingResourceException mre)
	{
	    return null;
	}

	return data;
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
**
******************************************************************************/

    public static String[] getSortedMapKeys()
    {
	Vector keys = new Vector();
	String temp;

	for (Enumeration e = maps.keys() ; e.hasMoreElements() ;) 
	{
	    keys.addElement(e.nextElement());
	}

	int numOfKeys = keys.size();
	String key[] = new String[numOfKeys];
	for (int i=0; i < numOfKeys; i++)
	{
	    key[i] = (String)keys.elementAt(i);
	    //key[i] = key[i].substring(0,key[i].lastIndexOf('.'));
	}

	for (int i=0; i < numOfKeys; i++)
	{
	    for (int j=i; j < numOfKeys; j++)
	    {
		if (key[i].compareTo(key[j]) > 0)
		{
		    temp = key[i];
		    key[i] = key[j];
		    key[j] = temp;
		}
	    }
	}
	return key;
    }


/******************************************************************************
**
**     Method name:             realPoint  (class GTracePanel)
**
**     Purpose:                 
**    
**     Parameters:              Point p
**
**     Return Values:           Point 
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public static Point getRealPoint(Point p)
     {
	  int iconWidth = curMap.getIcon().getIconWidth();
	  int iconHeight = curMap.getIcon().getIconHeight(); 
	  int paneWidth = mapScrollPane.getViewportBorderBounds().width;
	  int paneHeight = mapScrollPane.getViewportBorderBounds().height;

	  if ( paneWidth < iconWidth && paneHeight < iconHeight )
	  {
	       return p;
	  }
	  else 
	  {
	       if (paneWidth > iconWidth && paneHeight > iconHeight)
	       {
		    p.x = p.x + (paneWidth-iconWidth)/2;
		    p.y = p.y + (paneHeight-iconHeight)/2;
		    return p;
	       }
	       else if(paneWidth > iconWidth)
	       {
		    p.x = p.x + (paneWidth-iconWidth)/2;
		    return p;
	       }
	       else if ( paneHeight > iconHeight )
	       {
		    p.y = p.y + (paneHeight-iconHeight)/2;
		    return p;
	       }
	  }
	  return null;
     }



/******************************************************************************
**
**     Method name:             getMap
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
**
******************************************************************************/

    public static MapImage getMap(String MapName)
    {
	return (MapImage)maps.get(MapName);
    }

/******************************************************************************
**
**     Method name:             getStartUpMap
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 6th, 1999
**
******************************************************************************/

    public static MapImage getStartUpMap()
    {
	return (MapImage)maps.get(getMapResource("STARTUPMAP"));
    }

/******************************************************************************
**
**     Method name:             main 
**
**     Purpose:                 
**    
**     Parameters:              String[] args
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**                              
**     Reference:               Code taken from SwingSet example
**
******************************************************************************/
    
     public static void main(String[] args)
     {
	 String vers = System.getProperty("java.version");
	 if (vers.compareTo("1.1.2") < 0) 
	 {
	     System.out.println("!!!WARNING: Swing must be run with a " +
				"1.1.2 or higher version VM!!!");
	 }


	 // set the Look and Feel to the System L&F 
	 try 
	 {
	     UIManager.setLookAndFeel(
				    UIManager.getSystemLookAndFeelClassName());
	 } 
	 catch (Exception exc) 
	 {
	     System.out.println("Error loading L&F: " + exc);
	 }

	 WindowListener wl = new WindowAdapter() {
	     public void windowClosing(WindowEvent e) {System.exit(0);}
	 };


	 frame = new JFrame("GTrace v1.0.0");
	 frame.addWindowListener(wl);
	 frame.getAccessibleContext().setAccessibleDescription(
						"A Graphical Traceroute Tool");

	 JOptionPane.setRootFrame(frame);
	 
	 JPanel progressPanel = new JPanel() {
	     public Insets getInsets() {
		 return new Insets(40,30,20,30);
	     }
	 };
	 progressPanel.setLayout(new BoxLayout(progressPanel, 
					                    BoxLayout.Y_AXIS));
	 frame.getContentPane().add(progressPanel, BorderLayout.CENTER);

	 Dimension d = new Dimension(400, 20);
	 GTrace.progressLabel = new JLabel("Loading, please wait...");
	 GTrace.progressLabel.setAlignmentX(CENTER_ALIGNMENT);
	 GTrace.progressLabel.setMaximumSize(d);
	 GTrace.progressLabel.setPreferredSize(d);
	 progressPanel.add(progressLabel);
	 progressPanel.add(Box.createRigidArea(new Dimension(1,20)));
	 
	 GTrace.progressBar = new JProgressBar(0, GTrace.totalPanels);
	 GTrace.progressBar.setStringPainted(true);
	 GTrace.progressLabel.setLabelFor(progressBar);
	 GTrace.progressBar.setAlignmentX(CENTER_ALIGNMENT);
	 GTrace.progressBar.getAccessibleContext().setAccessibleName(
						    "GTrace loading progress");
	 progressPanel.add(GTrace.progressBar);
	 
	 // show the progress frame
	 frame.setSize(INITIAL_WIDTH, INITIAL_HEIGHT);
	 Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	 frame.setLocation(screenSize.width/2 - INITIAL_WIDTH/2,
			   screenSize.height/2 - INITIAL_HEIGHT/2);
	 frame.show();
	 
	 frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

	 whoisDialog = new whois(frame);
	 ldetailDialog = new LDetailDialog(frame);
	 GTrace gtrace = new GTrace();
	 frame.getContentPane().removeAll();
	 frame.getContentPane().setLayout(new BorderLayout());
	 frame.getContentPane().add(gtrace, BorderLayout.CENTER);
	 frame.setLocation(screenSize.width/2 - WIDTH/2,
			   screenSize.height/2 - HEIGHT/2);
	 
	 frame.setSize(WIDTH, HEIGHT);
	 frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	 
	 frame.validate();
	 frame.repaint();
	 statusBarUpdate();
	 gtrace.requestDefaultFocus();
     }




//---------------------------------------------------------------------------
//                             Inner Classes
//---------------------------------------------------------------------------


/******************************************************************************
**
**     Inner Class name:        LocationLegend 
**
**     Purpose:                 
**    
**     Author/Last Modified:    Ram Periakaruppan, Nov. 1st, 1999
**
******************************************************************************/

    class MapLineLegend extends JDialog
    {
        JDialog mapLineLegendDialog;     //the dialog


        JLabel greenLine;      // Label displaying green line
        JLabel greenLabel;     // Label displaying text 
        JLabel cyanLine;       // Label displaying cyan line
        JLabel cyanLabel1;     // Label displaying text
        JLabel cyanLabel2;     // Label displaying text
        JLabel cyanLabel3;     // Label displaying text
        JLabel blueLine;       // Label displaying blue line
        JLabel blueLabel1;     // Label displaying text
        JLabel blueLabel2;     // Label displaying text
        JLabel blueLabel3;     // Label displaying text
        JLabel redLine;        // Label displaying red line
        JLabel redLabel1;      // Label displaying text
        JLabel redLabel2;      // Label displaying text

        JButton okButton;  // ok button


        // The width and height of the frame
        public final int WIDTH = 450;
        public final int HEIGHT = 320;

/******************************************************************************
 **
 **     Purpose:                 Constructs and laysout components in dialog
 **                              setting the name of the host to be looked up.
 **    
 **     Parameters:              JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **                              String name - the host/ipaddr to be looked up
 **
 **     Author/Last Modified:    Ram Periakaruppan, Nov. 2nd, 1999
 **
 *****************************************************************************/

        public MapLineLegend()
        {
/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
            mapLineLegendDialog = new JDialog(frame, "Map Line Color Legend", false);
            mapLineLegendDialog.setSize(WIDTH, HEIGHT);
            mapLineLegendDialog.setResizable(false);
            
/*---------------------------------------------------------------------------
 *  the panel contains all the components
 *---------------------------------------------------------------------------*/

            // labels
            greenLine = new JLabel(lines[0]);
            greenLabel = new JLabel("Both endpoints are authoritative locations"); 
            cyanLine = new JLabel(lines[1]);
            cyanLabel1 = new JLabel("One endpoint is authoritative the other is");
            cyanLabel2 = new JLabel("a guess whose location is not a country");
            cyanLabel3 = new JLabel("center, state center, or from whois");
            blueLine = new JLabel(lines[2]);
            blueLabel1 = new JLabel("Both endpoints are guesses and the locations");
            blueLabel2 = new JLabel("of both the endpoints are not a country center,");
            blueLabel3 = new JLabel("state center, or from whois");
            redLine = new JLabel(lines[3]);
            redLabel1 = new JLabel("One endpoint is a location that is a country");
            redLabel2 = new JLabel("center, state center or obtained from whois");


            // Buttons
            okButton = new JButton("Ok");
            okButton.setMnemonic('O');
            okButton.setBorder(BorderFactory.createRaisedBevelBorder());
            okButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) 
                    {
                        mapLineLegendDialog.setVisible(false);
                        mapLineLegendVisible = false;
                        mapLineLegendDialog.dispose();
                    }
            });
        


            JPanel lineLegendPane = new JPanel();
            GridBagLayout gridbag = new GridBagLayout();
            GridBagConstraints c = new GridBagConstraints();

            lineLegendPane.setLayout(gridbag);

            // ROW 1
            c.gridx = 0;
            c.gridy = 0;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(15,5,0,10); 
            gridbag.setConstraints(greenLine, c);
            lineLegendPane.add(greenLine);

            c.gridx = 1;
            c.insets = new Insets(15,0,0,0); 
            gridbag.setConstraints(greenLabel, c);
            lineLegendPane.add(greenLabel);

        
            // ROW 2
            c.gridx = 0;
            c.gridy = 1;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(cyanLine, c);
            lineLegendPane.add(cyanLine);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(cyanLabel1, c);
            lineLegendPane.add(cyanLabel1);

            // ROW 3
            c.gridx = 1;
            c.gridy = 2;
            c.insets = new Insets(0,0,5,0);
            c.anchor = GridBagConstraints.NORTHWEST;
            gridbag.setConstraints(cyanLabel2, c);
            lineLegendPane.add(cyanLabel2);

            // ROW 4
            c.gridx = 1;
            c.gridy = 3;
            c.insets = new Insets(0,0,5,0);
            gridbag.setConstraints(cyanLabel3, c);
            lineLegendPane.add(cyanLabel3);
        

            // ROW 5
            c.gridx = 0;
            c.gridy = 4;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(blueLine, c);
            lineLegendPane.add(blueLine);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(blueLabel1, c);
            lineLegendPane.add(blueLabel1);

            // ROW 6
            c.gridx = 1;
            c.gridy = 5;
            c.insets = new Insets(0,0,5,0);
            c.anchor = GridBagConstraints.NORTHWEST;
            gridbag.setConstraints(blueLabel2, c);
            lineLegendPane.add(blueLabel2);

            // ROW 7
            c.gridx = 1;
            c.gridy = 6;
            c.insets = new Insets(0,0,5,0);
            gridbag.setConstraints(blueLabel3, c);
            lineLegendPane.add(blueLabel3);
        

            // ROW 8
            c.gridx = 0;
            c.gridy = 7;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(redLine, c);
            lineLegendPane.add(redLine);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(redLabel1, c);
            lineLegendPane.add(redLabel1);

            // ROW 9
            c.gridx = 1;
            c.gridy = 8;
            c.insets = new Insets(0,0,5,0);
            c.anchor = GridBagConstraints.NORTHWEST;
            gridbag.setConstraints(redLabel2, c);
            lineLegendPane.add(redLabel2);

            // ROW 10
            c.gridx = 0;
            c.gridy = 9;
            c.ipadx = 30;  
            c.ipady = 10;
            c.gridwidth = 2;
            c.anchor = GridBagConstraints.CENTER;
            c.insets = new Insets(10,5,5,0); 
            gridbag.setConstraints(okButton, c);
            lineLegendPane.add(okButton);


/*---------------------------------------------------------------------------
 *  add the add panel to the content pane of the map dialog and set its
 *  postion relative to the parent frame.
 *---------------------------------------------------------------------------*/
        
            mapLineLegendDialog.getContentPane().add(lineLegendPane);
            mapLineLegendDialog.setLocationRelativeTo(frame);
            mapLineLegendDialog.setVisible(true);
        }
    }            
        
/******************************************************************************
**
**     Inner Class name:        LocationLegend 
**
**     Purpose:                 
**    
**     Author/Last Modified:    Ram Periakaruppan, Nov. 1st, 1999
**
******************************************************************************/

    class LocationLegend extends JDialog
    {
        JDialog locationLegendDialog;     //the dialog
        
        JLabel greenIcon;    // Label displaying green ball
        JLabel greenLabel;   // Label displaying text
        JLabel yellowIcon;   // Label displaying yellow ball
        JLabel yellowLabel;  // Label displaying text
        JLabel blueIcon;     // Label displaying blue ball
        JLabel blueLabel;    // Label displaying text
        JLabel cyanIcon;     // Label displaying cyan ball
        JLabel cyanLabel;    // Label displaying text
        JLabel redIcon;      // Label displaying red ball
        JLabel redLabel;     // Label displaying text
        JLabel grayIcon;     // Label displaying gray ball
        JLabel grayLabel;    // Label displaying text

        JButton okButton;  // ok button


        // The width and height of the frame
        public final int WIDTH = 250;
        public final int HEIGHT = 240;

/******************************************************************************
 **
 **     Purpose:                 Constructs and laysout components in dialog
 **                              setting the name of the host to be looked up.
 **    
 **     Parameters:              JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **                              String name - the host/ipaddr to be looked up
 **
 **     Author/Last Modified:    Ram Periakaruppan, Nov. 2nd, 1999
 **
 *****************************************************************************/

        public LocationLegend()
        {

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
            locationLegendDialog = new JDialog(frame, "Location Legend", false);
            locationLegendDialog.setSize(WIDTH, HEIGHT);
            locationLegendDialog.setResizable(false);
            
/*---------------------------------------------------------------------------
 *  the panel contains all the components
 *---------------------------------------------------------------------------*/

            // Labels
            greenIcon = new JLabel(balls[0]);
            greenLabel = new JLabel("LOC Record");
            yellowIcon = new JLabel(balls[1]);
            yellowLabel = new JLabel("Complete DB/TextFile Match");
            blueIcon = new JLabel(balls[2]);
            blueLabel = new JLabel("Domain Parsing File");
            cyanIcon = new JLabel(balls[3]);
            cyanLabel = new JLabel("Hostname Reduction Match");
            redIcon = new JLabel(balls[4]);
            redLabel = new JLabel("whois Record");
            grayIcon = new JLabel(balls[5]);
            grayLabel = new JLabel("Country Code");


            // Buttons
            okButton = new JButton("Ok");
            okButton.setMnemonic('O');
            okButton.setBorder(BorderFactory.createRaisedBevelBorder());
            okButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
            okButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) 
                    {
                        locationLegendDialog.setVisible(false);
                        locationLegendVisible = false;
                        locationLegendDialog.dispose();
                    }
            });

            JPanel ballLegendPane = new JPanel();
            GridBagLayout gridbag = new GridBagLayout();
            GridBagConstraints c = new GridBagConstraints();

            ballLegendPane.setLayout(gridbag);

            // ROW 1
            c.gridx = 0;
            c.gridy = 0;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(greenIcon, c);
            ballLegendPane.add(greenIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(greenLabel, c);
            ballLegendPane.add(greenLabel);

            // ROW 2
            c.gridx = 0;
            c.gridy = 1;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(yellowIcon, c);
            ballLegendPane.add(yellowIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(yellowLabel, c);
            ballLegendPane.add(yellowLabel);

            // ROW 3
            c.gridx = 0;
            c.gridy = 2;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(blueIcon, c);
            ballLegendPane.add(blueIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(blueLabel, c);
            ballLegendPane.add(blueLabel);

            // ROW 4
            c.gridx = 0;
            c.gridy = 3;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(cyanIcon, c);
            ballLegendPane.add(cyanIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(cyanLabel, c);
            ballLegendPane.add(cyanLabel);
        
            // ROW 5
            c.gridx = 0;
            c.gridy = 4;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(redIcon, c);
            ballLegendPane.add(redIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(redLabel, c);
            ballLegendPane.add(redLabel);

            // ROW 6
            c.gridx = 0;
            c.gridy = 5;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(5,5,5,10); 
            gridbag.setConstraints(grayIcon, c);
            ballLegendPane.add(grayIcon);

            c.gridx = 1;
            c.insets = new Insets(5,0,5,0); 
            gridbag.setConstraints(grayLabel, c);
            ballLegendPane.add(grayLabel);

            // ROW 7
            c.gridx = 0;
            c.gridy = 6;
            c.ipadx = 30;  
            c.ipady = 10;
            c.gridwidth = 2;
            c.anchor = GridBagConstraints.CENTER;
            c.insets = new Insets(10,5,5,0); 
            gridbag.setConstraints(okButton, c);
            ballLegendPane.add(okButton);

/*---------------------------------------------------------------------------
 *  add the add panel to the content pane of the map dialog and set its
 *  postion relative to the parent frame.
 *---------------------------------------------------------------------------*/
        
            locationLegendDialog.getContentPane().add(ballLegendPane);
            locationLegendDialog.setLocationRelativeTo(frame);
            locationLegendDialog.setVisible(true);
        }
    }
    
    
/******************************************************************************
**
**     Inner Class name:        ipaddr_host_loc_Editor 
**
**     Purpose:                 
**    
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     class ipaddr_host_loc_Editor extends DefaultCellEditor 
     {

/******************************************************************************
**
**     Constructor:             for class GTracePanel
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public ipaddr_host_loc_Editor(JButton b) 
	  {
	       super(new JCheckBox());
	       editorComponent = b;
	       setClickCountToStart(1); 

	       b.addActionListener(new ActionListener() 
	       {
		    public void actionPerformed(ActionEvent e) 
		    {
			 fireEditingStopped();
		    }
	       });
	  }

/******************************************************************************
**
**     Method name:             fireEditingStopped 
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  protected void fireEditingStopped() 
	  {
               super.fireEditingStopped();
	  }

/******************************************************************************
**
**     Method name:             getCellEditorValue  
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public Object getCellEditorValue() 
	  {
               return new JLabel(((JButton)editorComponent).getText());
	  }

/******************************************************************************
**
**     Method name:             getTableCellEditorComponent  
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public Component getTableCellEditorComponent(JTable table, 
						       Object value,
						       boolean isSelected,
						       int row,
						       int column) 
	  {
	      String colName = table.getColumnName(column);

	      if(colName.compareTo(IPADDR) == 0)
	      {
		  ((JButton)editorComponent).setActionCommand(IPADDR);
	      }
	      else if(colName.compareTo(INTERFACE) == 0)
	      {
		  ((JButton)editorComponent).setActionCommand(INTERFACE);
	      }
	      else if(colName.compareTo(LOCATION) == 0)
	      {
		  ((JButton)editorComponent).setActionCommand(LOCATION+"-"
                                              +(new Integer(row).toString()));
	      }

	      ((JButton)editorComponent).setText(((JLabel)value).getText());
	      return editorComponent;
	  }
     }   



//---------------------------------------------------------------------------
//                             xxxListeners
//---------------------------------------------------------------------------


/******************************************************************************
**
**     Inner Class name:        ComboBoxListener   
**
**     Purpose:                 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
    
     class ComboBoxListener implements ActionListener
     {       

	  public void actionPerformed(ActionEvent e)
	  {
	      
	      String dest = (String)urlList.getSelectedItem();		  
	      if (dest != null && dest.trim().length() > 0)
	      { 
		  dataModel.clearTable();
		  latlons.removeAllElements();
		  MapImage.latlonValues.removeAllElements();
		  mapScrollPane.getViewport().getView().repaint();
		  currentThreadGroupNumber++;
		  
		  try
		  {
		      traceThreadGroup.destroy();
		  }
		  catch(IllegalThreadStateException itse) { }
		  catch(NullPointerException npe) { }
		  
		  traceThreadGroup = new ThreadGroup("group"
                                                    +currentThreadGroupNumber);
		  
		  if (dest.startsWith("http://"))
		  {
		      try
		      {
			  URL url = new URL(dest);
			  dest = url.getHost();
			  parseTrace = new ParseTrace("ParseThread",
                                            dest, dataModel, traceThreadGroup);
		      }
		      catch(MalformedURLException mue)
		      {
			  JOptionPane.showMessageDialog(frame,
						   "Invalid URL entered.",
						   "",
						   JOptionPane.ERROR_MESSAGE);
		      }


		  }
		  else
		  {
		      parseTrace = new ParseTrace("ParseThread",
                                                  dest, dataModel,
                                                  traceThreadGroup);
		  }
	      }
	  }
     }

    
/******************************************************************************
**
**     Inner Class name:        JLabelIconRenderer   
**
**     Purpose:                 
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 9th, 1999
**
******************************************************************************/

    class JLabelIconRenderer extends ImageIcon
    {
	Color iconColor;
	
	public JLabelIconRenderer()
	{
	    super();
	    iconColor = Color.white;
	}

	public void paintIcon(Component c, Graphics g, int x, int y) 
	{	
	    int iconSize = 12;
	    g.setColor(iconColor);
	    g.fillOval(x+2,y-7,iconSize,iconSize);
	}    

    }


/******************************************************************************
**
**     Inner Class name:        JLabelRenderer   
**
**     Purpose:                 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     class JLabelRenderer extends JLabel implements TableCellRenderer
     {
	  Container parent;



/******************************************************************************
**
**     Constructor:             for class JLabelRenderer
**
**     Purpose:                 
**    
**     Parameters:              Container parent
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  JLabelRenderer(Container parent)
	  {
	       this.parent = parent;
	  }

/******************************************************************************
**
**     Method name:             getTableCellRendererComponent
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Return Values:           Component
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public Component getTableCellRendererComponent(JTable table,
                                                         Object text,
							 boolean isSelected,
                                                         boolean hasFocus,
							 int row, int column) 
	  {

	       JLabel temp = (JLabel)text;
	       setIcon(null);
	       setToolTipText(null);

	       String colName = table.getColumnName(column);

	       if(colName.compareTo(RTT) == 0)
	       {
		    frame.setCursor(Cursor.getPredefinedCursor(
                                                       Cursor.DEFAULT_CURSOR));
		    setHorizontalAlignment(SwingConstants.CENTER);
		    Color color = GTraceTableModel.getCellColor(row);
		    setForeground(color);

		    if (showToolTips)
		    {
			if(color == Color.black)
			{
			    setToolTipText("No Packets Timed Out");
			}
			else if(color == Color.blue)
			{
			    setToolTipText("One Packet Timed Out");
			}
			else if(color == Color.red)
			{
			    setToolTipText("Two Packets Timed Out");
			}
			else if(color == Color.green)
			{
			    setToolTipText("RTT obtained using PING");
			}
		    }
	       }
	       else if(colName.compareTo(IPADDR) == 0
                       || colName.compareTo(INTERFACE) == 0)
	       {
		    frame.setCursor(Cursor.getPredefinedCursor(
                                                          Cursor.HAND_CURSOR));
		    setHorizontalAlignment(SwingConstants.LEFT);
		    setForeground(Color.gray);
	       }
	       else if(colName.compareTo(LOCATION) == 0)
	       {
		    frame.setCursor(Cursor.getPredefinedCursor(
                                                          Cursor.HAND_CURSOR));
		    setHorizontalAlignment(SwingConstants.LEFT);
		    setForeground(GTraceTableModel.getLocationColor(row));
		    
		    if (showToolTips)
		    {
			setToolTipText("Click here for more details");
		    }

		    Integer typeOfLoc = GTraceTableModel.getTypeofLOC(row);
		    MapImage mapImage = (MapImage)curMap.getIcon();
		    
		    int numOfRows=table.getRowCount();
		    boolean dataChanged = false;
		    int numOfCols = table.getColumnCount();

		    String city;
		    JLabel label;

		    for (int i=0; i<numOfCols; i++)
		    {
			if (table.getColumnName(i).compareTo(LOCATION) == 0)
			{
			    for (int j=0; j <numOfRows; j++)
			    {
				city = GTraceTableModel.getLocationCity(j);
				
				if (city != null)
				{
				    if (city.compareTo("UNKNOWN") == 0)
				    {
					label = (JLabel)table.getValueAt(j,i);
				    }
				    else 
				    {
					label = new JLabel(
				  GTraceTableModel.getLocationLatLonString(j));
				    }
				}
				else 
				{
				    label = (JLabel)table.getValueAt(j,i);
				}

				try
				{
				    if ( label.getText().compareTo(
				      ((JLabel)latlons.elementAt(j)).getText())
					 != 0 )
				    {     
					latlons.setElementAt(label,j);
					dataChanged = true;
				    }
				}
				catch(ArrayIndexOutOfBoundsException aobe)
				{
				    latlons.addElement(label);
				    dataChanged = true;
				}
			    }
			    
			    break;
			}
		    }
				
		    if (dataChanged)
		    {
			 Component c = mapScrollPane.getViewport().getView();
			 mapImage.plotpoints(latlons, c);
		    }
		    
		    setIconTextGap(5);
		    
		    if (typeOfLoc.intValue() == 1)
		    {
			 setIcon(iconColorWhite);
		    }			
		    else if (typeOfLoc.intValue() == 0)
		    {
			 setIcon(iconColorWhite);

			 if (showToolTips)
			 {
			     setToolTipText("Private Internet Use");
			 }
			 else 
			 {
			     setToolTipText(null);
			 }
		    }		
		    else
		    {
			switch(typeOfLoc.intValue())
			{
			    case STEP2:
				setIcon(balls[0]);
				break;
			    case STEP3:
				setIcon(balls[1]);
				break;
			    case STEP4:
			    case STEP4STATE:
			    case STEP4COUNTRY:
				setIcon(balls[2]);
				break;
			    case STEP5:
				setIcon(balls[3]);
				break;
			    case STEP6:
				setIcon(balls[4]);
				break;
			    case STEP7:
				setIcon(balls[5]);
			}
		    }
	       }
	       else
	       {
		    frame.setCursor(Cursor.getPredefinedCursor(
						       Cursor.DEFAULT_CURSOR));
		    setHorizontalAlignment(SwingConstants.CENTER);
		    setForeground(Color.black);
	       }

	       setText(temp.getText());
	       return this;
	  }
     }


//---------------------------------------------------------------------------
//                             Mouse Handling
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Inner Class name:        PictureListener  
**
**     Purpose:                 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     class PictureListener extends MouseAdapter implements MouseMotionListener
     {       
          Point prevPoint = new Point();

/******************************************************************************
**
**     Method name:             realPoint (class PictureListener)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           Point  
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  private Point realPoint(MouseEvent e)
	  { 
	       JScrollBar hBar = mapScrollPane.getHorizontalScrollBar();
	       JScrollBar vBar = mapScrollPane.getVerticalScrollBar();

	       Point point = e.getPoint();
	       int x = point.x + hBar.getValue();
	       int y = point.y + vBar.getValue();
	      
	       int iconWidth = curMap.getIcon().getIconWidth();
	       int iconHeight = curMap.getIcon().getIconHeight(); 
	       int paneWidth = mapScrollPane.getViewportBorderBounds().width;
	       int paneHeight = mapScrollPane.getViewportBorderBounds().height;

	       int boundary[] = new int[4];
	       Point actPoint = new Point();

	       prevPoint.setLocation(point.x, point.y);
	       MapImage mapImage = (MapImage)curMap.getIcon();	

	       if ( paneWidth < iconWidth && paneHeight < iconHeight )
	       {
		    actPoint.x = x;
		    actPoint.y = y;
		    latlonValue.setText(mapImage.pointToLatLon(actPoint));
		    return actPoint;
	       }
	       else 
	       {
		    if (paneWidth > iconWidth && paneHeight > iconHeight)
		    {
		         boundary[0] = (paneWidth-iconWidth)/2;
			 boundary[1] = (paneHeight-iconHeight)/2;
			 boundary[2] = boundary[0] + iconWidth;
			 boundary[3] = boundary[1] + iconHeight;
		    }
		    else if(paneWidth > iconWidth)
		    {
		         boundary[0] = (paneWidth-iconWidth)/2;
			 boundary[1] = 1;
			 boundary[2] = boundary[0] + iconWidth;
			 boundary[3] = iconHeight;
		    }
		    else if ( paneHeight > iconHeight )
		    {
		         boundary[0] = 1;
			 boundary[1] = (paneHeight-iconHeight)/2;
			 boundary[2] = iconWidth;
			 boundary[3] = boundary[1] + iconHeight;
		    }
		 
		    if (x > boundary[0] && x < boundary[2] && y > boundary[1] 
			&& y < boundary[3])
		    {
			 actPoint.x = x - boundary[0];
			 actPoint.y = y - boundary[1];
			 latlonValue.setText(mapImage.pointToLatLon(actPoint));
			 return(actPoint);
		    }
		    else
		    {
			 latlonValue.setText("");
			 return null;
		    }
	       }
	  }

/******************************************************************************
**
**     Method name:             mouseClicked (class PictureListener)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           void 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	 
	 public void mouseClicked(MouseEvent e)
	 {
	     MapImage mapImage = (MapImage)curMap.getIcon();
	     
	     Point point = realPoint(e);
	     int[] pixels = new int[1];

	     if (point != null && !ComboMenuDown)
	     {    
		 try 
		 {
		     new PixelGrabber(mapImage.getImage(), 
				      point.x, point.y, 1, 
				      1, pixels, 0, 1).grabPixels();
		 } 
		 catch (Exception exp) { }

		 if (!(new Color(pixels[0])).equals(map_background))
		 {
		     if (e.getModifiers() == InputEvent.BUTTON1_MASK)	
		     {
			 String childMaps[];
			 childMaps = mapImage.getChildMap(point.x,point.y);
				     
			 float lon = mapImage.getLon(point);
			 float lat = mapImage.getLat(point);
			 int[] chldpixels = new int[1];
			      
			 if (childMaps.length == 1 && childMaps[0] != null)
			 {
			     frame.setCursor(Cursor.getPredefinedCursor(
							 Cursor.WAIT_CURSOR));
			     MapImage childImage = getMap(childMaps[0]);
			     if (childImage != null)
			     {
				 childImage.loadMap(); 
			     }
					     
			     Point childPoint = childImage.latLonToPoint(
						      mapImage.getLat(point),
						      mapImage.getLon(point) );
			     try 
			     {
				 new PixelGrabber(childImage.getImage(), 
						  childPoint.x, 
						  childPoint.y, 
						  1, 1, chldpixels, 
						  0, 1).grabPixels();
			     } 
			     catch (Exception exp) { }
			     
			     if (!(new Color(chldpixels[0])).equals(
							       map_background) 
				 || childImage.getDescription() == "World100")
			     {
				 curMap.setIcon(childImage);
			     }
			     frame.setCursor(
			        Cursor.getPredefinedCursor(
						      Cursor.DEFAULT_CURSOR));
			 }
			 else if (childMaps.length > 1)
			 { 				
			     frame.setCursor(
			       Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			     
			     for (int i=0; i < childMaps.length; i++)
			     {
				 MapImage childImage = getMap(childMaps[i]);
				 if (childImage != null)
				 {
				     childImage.loadMap();
				 }

				 Point childPoint = childImage.latLonToPoint(
						        mapImage.getLat(point),
						       mapImage.getLon(point));
				 try 
				 {
				     new PixelGrabber(childImage.getImage(), 
						      childPoint.x, 
						      childPoint.y, 1, 1, 
					        chldpixels, 0, 1).grabPixels();
				 } 
				 catch (Exception exp) { } 
				 
				 if (!(new Color(chldpixels[0])).equals(
							       map_background))
				 {
				     curMap.setIcon(childImage);
				     break;
				 }
			     }
			     frame.setCursor(
			    Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
			 }
		     }
		 }
		    
		 if(e.getModifiers() == InputEvent.BUTTON2_MASK 
		    ||  e.getModifiers() == InputEvent.BUTTON3_MASK)
		 {
		     String parentMap = mapImage.getParentMap();
		     if (parentMap != null)
		     { 				
			 frame.setCursor(
			       Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			 MapImage parentImage = getMap(parentMap);
			 if (parentImage != null)
			 {
			     parentImage.loadMap();
			     curMap.setIcon(parentImage);
			 }
			 frame.setCursor(
			    Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		     }
		 }
	     }
	 }

/******************************************************************************
**
**     Method name:             mouseMoved (class PictureListener)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           void  
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public void mouseMoved(MouseEvent e)
	  {	
	       int[] pixels = new int[1];

	       Point point = realPoint(e);
	       MapImage mapImage = (MapImage)curMap.getIcon();

	       ComboMenuDown = false;

	       if (point != null)
	       {
		    try 
		    {
			 new PixelGrabber(mapImage.getImage(), point.x, 
					  point.y, 1, 1, pixels, 
					  0, 1).grabPixels();
		    } 
		    catch (Exception exp) { }
		     
		    if (mapImage.withinRegion(point.x, point.y) 
			&& !(new Color(pixels[0])).equals(map_background))
		    {
			 frame.setCursor(
			       Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		    }
		    else
		    {
			frame.setCursor(
			  Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
		    }
	       }
	       else
	       {
		   frame.setCursor(
			    Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	       }
	  }
    
/******************************************************************************
**
**     Method name:             mouseDragged (class PictureListener)
**
**     Purpose:                 
**    
**     Parameters:              MouseEvent e
**
**     Return Values:           void  
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public void mouseDragged(MouseEvent e)
	  {
	       JScrollBar hBar = mapScrollPane.getHorizontalScrollBar();
	       JScrollBar vBar = mapScrollPane.getVerticalScrollBar();
	       
	       int xVal = (int)prevPoint.x;
	       int yVal = (int)prevPoint.y;				    
							  
	       if (e.getX() > xVal)
	       {
		    hBar.setValue(hBar.getValue() + e.getX() - xVal);
	       }
	       else if(e.getX() < xVal)
	       { 
		    hBar.setValue(hBar.getValue() - xVal + e.getX());
	       }
		   
	       if (e.getY() > yVal)
	       {
		    vBar.setValue(vBar.getValue() + e.getY() - yVal);
	       }
	       else if(e.getY() < yVal)   
	       {
		    vBar.setValue(vBar.getValue() - yVal + e.getY());
	       }
		    
	       prevPoint.setLocation(e.getX(), e.getY());
	  }
     }



/******************************************************************************
**
**     Inner Class name:        ComboModel  
**
**     Purpose:                 
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public class ComboModel extends AbstractListModel 
	                     implements MutableComboBoxModel
     {
	  Vector elements;
	  Object selectedElement;
	  int limit;      
    

/******************************************************************************
**
**     Constructor:             for inner class ComboModel
**
**     Purpose:                 Initializes the limit of the number of items 
**                              that be stored in the ComboBox and the Vector
**                              that holds the elements. 
**    
**     Parameters:              int maxNumOfItems --> Number of items in
**                                                    ComboBox
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public ComboModel(int maxNumOfItems) 
	  {
	       limit = maxNumOfItems;
	       elements = new Vector(limit);
	  }

//---------------------------------------------------------------------------
//                             setxxx Methods
//---------------------------------------------------------------------------


/******************************************************************************
**
**     Method name:             setSelectedItem  (class ComboModel)
**
**     Purpose:                 
**    
**     Parameters:              Object anObject
**
**     Return Values:           void
**
**     Notes:                   implements javax.swing.ComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	 
	  public void setSelectedItem(Object anObject) 
	  {        
	       String temp = (String)anObject;
	       if (!elements.contains(anObject) && anObject != null 
		   && temp.length() > 0)
	       {
		    addElement(anObject);
	       }
	       else
	       {
		    selectedElement = anObject;
		    fireContentsChanged(this, -1, -1);
	       }
	  }


//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

	 public Vector getItems()
	 {
	     return elements;
	 }

	 
/******************************************************************************
**
**     Method name:             getSelectedItem
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           Object
**
**     Notes:                   implements javax.swing.ComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public Object getSelectedItem() 
	  {
	       return selectedElement;
	  }

/******************************************************************************
**
**     Method name:             getSize
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**
**     Notes:                   implements javax.swing.ListModel
**                                            
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public int getSize() 
	  {
	       ComboMenuDown = true;
	       return elements.size();
	  }

/******************************************************************************
**
**     Method name:             getElementAt
**
**     Purpose:                 
**    
**     Parameters:              int index
**
**     Return Values:           Object
**
**     Notes:                   implements javax.swing.ListModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public Object getElementAt(int index) 
	  {
	       if ( index >= 0 && index < elements.size() )
	       {
		    return elements.elementAt(index);
	       }
	       else
	       {
		    return null;
	       }
	  }
    

/******************************************************************************
**
**     Method name:             getIndexOf
**
**     Purpose:                 
**    
**     Parameters:              Object anObject
**
**     Return Values:           Returns the index-position of the specified
**                              object in the list.
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public int getIndexOf(Object anObject) 
	  {
	       return elements.indexOf(anObject);
	  }
    

	  public void addComboHistory(Object element) 
	  {       
	       if (elements.size() == limit)
	       {
		    elements.removeElement(elements.lastElement());
	       }

	       elements.insertElementAt(element, 0);
	       fireIntervalAdded(this,elements.size()-1, elements.size()-1);
	  }

/******************************************************************************
**
**     Method name:             addElement
**
**     Purpose:                 
**    
**     Parameters:              Object anObject
**
**     Return Values:           void
**
**     Notes:                   implements javax.swing.MutableComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	 
	  public void addElement(Object anObject) 
	  {       
	       if (elements.size() == limit)
	       {
		    elements.removeElement(elements.lastElement());
	       }

	       elements.insertElementAt(anObject, 0);
	       fireIntervalAdded(this,elements.size()-1, elements.size()-1);
	       setSelectedItem( anObject );
	  }
    
/******************************************************************************
**
**     Method name:             insertElementAt
**
**     Purpose:                 
**    
**     Parameters:              Object anObject,int index
**
**     Return Values:           void
**
**     Notes:                   implements javax.swing.MutableComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public void insertElementAt(Object anObject,int index) 
	  {
	       if (elements.size() == limit)
	       {
		    //remove the last element if max limit is reached
		    elements.removeElement(elements.lastElement());
	       }
	       elements.insertElementAt(anObject,index);
	       fireIntervalAdded(this, index, index);          
	  }

/******************************************************************************
**
**     Method name:             removeElementAt
**
**     Purpose:                 
**    
**     Parameters:              int index
**
**     Return Values:           void
**
**     Notes:                   implements javax.swing.MutableComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public void removeElementAt(int index) 
	  {
	       if ( getElementAt( index ) == selectedElement ) 
	       {
		    if ( index == 0 ) 
		    {
			 if (getSize() == 1)
			 {
			      setSelectedItem(null);
			 }
			 else
			 {
			      setSelectedItem(getElementAt( index + 1 ));
			 }
		    }
		    else 
		    {
			 setSelectedItem( getElementAt( index - 1 ) );
		    }
	       }
	       elements.removeElementAt(index);
	       fireIntervalRemoved(this, index, index);
	  }

/******************************************************************************
**
**     Method name:             removeElement
**
**     Purpose:                 
**    
**     Parameters:              Object anObject
**
**     Return Values:           void
**
**     Notes:                   implements javax.swing.MutableComboBoxModel
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	  
	  public void removeElement(Object anObject) 
	  {
	       int index = elements.indexOf(anObject);
	       if ( index != -1 ) 
	       {
		    removeElementAt(index);
	       }
	  }
    
/******************************************************************************
**
**     Method name:             removeAllElements
**
**     Purpose:                 Empties the list.
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                            
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

	  public void removeAllElements() 
	  {
	       int firstIndex = 0;
	       int lastIndex = elements.size()-1;
	       elements.removeAllElements();
	       selectedElement = null;
	       fireIntervalRemoved(this, firstIndex, lastIndex);
	  }
     }
}

