/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    GTraceTableModel.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import javax.swing.JLabel;
import javax.swing.table.AbstractTableModel;

import java.awt.Color;
import java.util.Vector;

public class GTraceTableModel extends AbstractTableModel
{    
     public final int numOfColumns = 6;
     public final int PRIVATE = 80;
     public static final int TIMEOUT = 90;
    
     static Vector data = new Vector();
     static Vector time_color = new Vector();
     static Vector location_color = new Vector();
     static Vector location_details = new Vector();
     static Vector searchStep = new Vector();
     static Object[] emptyRow = new Object[6];

     final Color def_col = new Color(0,0,0);

     String[] header = new String[]{"Plot","Hop","IP Address",
				    "Interface Name","Location",
				    "RTT (ms)"};

     private final int HOP = 1;
     private final int RTT = 5;

     private static int numLocationsAdded = 0;
     private int currentHop = 0;

     static
     {
	  emptyRow[0] = new Boolean(false);
	  emptyRow[1] = new JLabel();
	  emptyRow[2] = new JLabel();
	  emptyRow[3] = new JLabel();
	  emptyRow[4] = new JLabel();
	  emptyRow[5] = new JLabel();
     }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
**
******************************************************************************/

    public Vector getLocationDetails()
    {
	return location_details;
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 18th, 1999
**
******************************************************************************/

    public static LocationDetails getLocationDetails(int rowNum)
    {
        try
        {
            return (LocationDetails)location_details.elementAt(rowNum);
        }
        catch(ArrayIndexOutOfBoundsException aiobe) { }

        return null;
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Vector getData()
    {
	return data;
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Vector getSearchStep()
    {
	return searchStep;
    }



/******************************************************************************
**
**     Method name:             getData  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           Color
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public static Object[] getData(int rowNum) 
     {
         try
         {
             return (Object[])data.elementAt(rowNum);
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }       

    
/******************************************************************************
**
**     Method name:             getCellColor  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           Color
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public static Color getCellColor(int rowNum) 
     { 
         try
         {
             return (Color)time_color.elementAt(rowNum);
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }       

/******************************************************************************
**
**     Method name:             getLocationColor  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           Color
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, Oct 6th, 1999
**
******************************************************************************/

     public static Color getLocationColor(int rowNum) 
     { 
         try
         {
             return (Color)location_color.elementAt(rowNum);
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }       

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
**
******************************************************************************/

     public static String getLocationLatLonString(int rowNum) 
     {
         try
         {
             return ((LocationDetails)location_details.elementAt(
                                                   rowNum)).getLatLonString();
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }       

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
**
******************************************************************************/

     public static String getLocationCity(int rowNum) 
     {
         try
         {
             return ((LocationDetails)location_details.elementAt
                            (rowNum)).getFormattedCity();
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }       


/******************************************************************************
**
**     Method name:             getTypeofLOC  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           Integer
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	
     public static Integer getTypeofLOC(int rowNum)
     {
         try
         {
             return (Integer)searchStep.elementAt(rowNum);
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;
     }

/******************************************************************************
**
**     Method name:             getColumnCount  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public int getColumnCount() 
     { 
	  return numOfColumns; 
     }   

/******************************************************************************
**
**     Method name:             getRowCount  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	
     public int getRowCount() 
     { 
	  return data.size();
     }			 

/******************************************************************************
**
**     Method name:             getValueAt  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum 
**                              int colNum
**
**     Return Values:           Object
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
	
     public Object getValueAt(int rowNum, int colNum) 
     {
         try
         {
             Object[] row = (Object[])data.elementAt(rowNum);

             if (row != null)
             {	
                 return row[colNum];
             }
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return null;		
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/

    public int getNumLocationsAdded(String threadGroup)
    {
	if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	{
            return numLocationsAdded;
	}

        return 0;
    }
    
/******************************************************************************
**
**     Method name:             getCheckBoxValueAt  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public static boolean getCheckBoxValueAt(int rowNum)
     {
         try
         {
             Object[] elementsInRow = (Object[])data.elementAt(rowNum);
             return ((Boolean)elementsInRow[0]).booleanValue();
         }
         catch(ArrayIndexOutOfBoundsException aiobe) { }

         return false;
     }

/******************************************************************************
**
**     Method name:             clearTable  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public void clearTable()
     {
	  if (getRowCount() > 0)
	  {
	       data.removeAllElements();
	       time_color.removeAllElements();
	       location_color.removeAllElements();
	       location_details.removeAllElements();
	       searchStep.removeAllElements();
	       numLocationsAdded = 0;
               currentHop = 0;
	       fireTableRowsDeleted(1,getRowCount());
	  }
     }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public boolean isTraceComplete(String threadGroup)
    {
	if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	{
	    if (currentHop == numLocationsAdded)
	    {
		return true;
	    }
	}
	
	return false;
    }

/******************************************************************************
**
**     Method name:             isCellEditable  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int row 
**                              int col
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public boolean isCellEditable(int row, int col)
     {
	  if (col == HOP || col == RTT)
	  {
	       return false;
	  }
	  else
	  {
	       return true;
	  }
     }	
	
/******************************************************************************
**
**     Method name:             getColumnName  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int col
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public String getColumnName(int col)
     {
	  return header[col];
     }


/******************************************************************************
**
**     Method name:             getColumnClass  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int col
**
**     Return Values:           Class
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public Class getColumnClass(int col)
     {
	  return getValueAt(0, col).getClass();
     }
	

/******************************************************************************
**
**     Method name:             setValueAt  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              Object value 
**                              int row 
**                              int col
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public void setValueAt(Object value, int row, int col) 
     {
	  Object[] elementsInRow = (Object[])data.elementAt(row);
	  JLabel temp = (JLabel)elementsInRow[4];
	  
	  if (!(temp.getText() == ""))
	  {   
	       elementsInRow[col] = value;
	  }

	  data.setElementAt(elementsInRow, row);
	  fireTableCellUpdated(row, col);	  
	  GTrace.redrawMap();
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/

    public void setCurrentHop(int hNum, String threadGroup)
    {
	if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	{
            currentHop = hNum;
	}
    }
    
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/

    public void addRow(int rowNum, Object[] row, Color time_c,
                       Color location_c,
                       LocationDetails lDetail, Integer step,
                       String threadGroup)
    {
        if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
        {
            data.addElement(row);
            time_color.addElement(time_c);
            location_color.addElement(location_c);
            location_details.addElement(lDetail);
            searchStep.addElement(step);
            fireTableRowsInserted(data.size(), data.size());
        }
    }

/******************************************************************************
**
**     Method name:             addRow  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              String[] row 
**                              int rowNum 
**                              Color t_color
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public synchronized void addRow(String[] row, int rowNum, Color t_color, 
				                          String threadGroup)
    {   
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		Object[] newRow = new Object[6];
		
		newRow[0] = new Boolean(false);
		newRow[1] = new JLabel(row[0]);
		newRow[2] = new JLabel(row[1]);
		newRow[3] = new JLabel(row[2]);
		newRow[4] = new JLabel(row[3]);
		newRow[5] = new JLabel(row[4]);
		
		if (rowNum < data.size())
		{
		    data.setElementAt(newRow, rowNum-1);		    
		    time_color.setElementAt(t_color, rowNum-1);
		    location_color.setElementAt(Color.gray, rowNum-1);
		    fireTableRowsUpdated(rowNum,rowNum);   
		}
		else
		{
		    while (data.size() < rowNum-1)
		    {   
			data.addElement(emptyRow);
			time_color.addElement(def_col);
			location_color.addElement(Color.gray);
			location_details.addElement(new LocationDetails());
			searchStep.addElement(new Integer(0));
			fireTableRowsInserted(data.size(), data.size());
		    }			
		    data.addElement(newRow);		
		    time_color.addElement(t_color);
		    location_color.addElement(Color.gray);
		    location_details.addElement(new LocationDetails());
		    searchStep.addElement(new Integer(0));
		    fireTableRowsInserted(data.size(), data.size());
		}
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }	
	

/******************************************************************************
**
**     Method name:             addLocation  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              String location 
**                              int rowNum 
**                              Integer type
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public synchronized void addLocation(String threadGroup)
    {	
        if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
        {
            numLocationsAdded++;
            location_details.addElement(new LocationDetails(
                                                     new Integer(TIMEOUT)));
            searchStep.addElement(new Integer(0));
        }
    }
    
/******************************************************************************
**
**     Method name:             addLocation  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              String location 
**                              int rowNum 
**                              Integer type
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public synchronized void addLocation(LocationDetails lDetail, 
					 int rowNum, Integer type,
                                         String threadGroup)
    {	
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		rowNum--;
		numLocationsAdded++;
		Object[] elementsInRow = (Object[])data.elementAt(rowNum);
		
		String city = lDetail.getFormattedCity();

		if (lDetail.getSearchStep().intValue() == PRIVATE)
		{
		    elementsInRow[4] = new JLabel("Private Internet Use");
		}
		else if (city.compareTo("UNKNOWN") == 0)
	 	{
		    elementsInRow[4] = new JLabel(lDetail.getLatLonString());
		}
		else
		{
		    elementsInRow[4] = new JLabel(city);
		}

		data.setElementAt(elementsInRow, rowNum);
		searchStep.setElementAt(type, rowNum);
		location_details.setElementAt(lDetail, rowNum);

		fireTableCellUpdated(rowNum, 4);
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }


/******************************************************************************
**
**     Method name:             addLocation  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              String location 
**                              int rowNum 
**                              Integer type
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public synchronized void changeRTT(float rtt, int rowNum,
                                       String threadGroup)
    {	
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		Object[] elementsInRow = (Object[])data.elementAt(rowNum);
		elementsInRow[5] = new JLabel(new Float(rtt).toString());
		data.setElementAt(elementsInRow, rowNum);
		time_color.setElementAt(Color.green, rowNum);
		fireTableCellUpdated(rowNum, 5);
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }

/******************************************************************************
**
**     Method name:             changeLocationColor  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              String location 
**                              int rowNum 
**                              Integer type
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public synchronized void changeLocationColor(Color color, int rowNum,
                                                 String threadGroup)
    {	
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		location_color.setElementAt(color, rowNum);
		fireTableCellUpdated(rowNum, 4);
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }
    
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    public synchronized void addCurrentSearchStep(int rowNum, Integer step,
                                                  String threadGroup)
    {	
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		rowNum--;
		Object[] elementsInRow = (Object[])data.elementAt(rowNum);
		searchStep.setElementAt(step, rowNum);
		fireTableCellUpdated(rowNum, 4);
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }


/******************************************************************************
**
**     Method name:             addLOCFound  (class GTraceTableModel)
**
**     Purpose:                 
**    
**     Parameters:              int rowNum
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public synchronized void addLOCFound(int rowNum, String threadGroup)
    {
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		rowNum--;
		Object[] elementsInRow = (Object[])data.elementAt(rowNum);
		elementsInRow[0] = new Boolean(true);
		data.setElementAt(elementsInRow, rowNum);
		fireTableCellUpdated(rowNum, 0);	  
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/

    public synchronized void updateTable(int modRowNum, int orgRowNum,
                                         String changes, String threadGroup)
    {
	try
	{
	    if (threadGroup.compareTo(GTrace.getCurrentGroupNumber()) == 0)
	    {
		Object[] elementsInModRow = (Object[])data.elementAt(
                                                                   modRowNum);
                LocationDetails lDetail =
                   (LocationDetails)location_details.elementAt(modRowNum);
                lDetail.setVerifierResults(changes);
		elementsInModRow[0] = new Boolean(false);
		data.setElementAt(elementsInModRow, modRowNum);
		fireTableCellUpdated(modRowNum, 0);	  
	    }
	}
	catch(ArrayIndexOutOfBoundsException aiobe) { }
    }
}
