/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    ParseTrace.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import java.io.*;
import javax.swing.*;
import java.awt.Color;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;


public class LocationVerifier implements Runnable
{
    private Thread t;
    private GTraceTableModel model;
    Object[] hopData = new Object[6];

    Vector data;
    Vector searchStep;
    Vector locationDetails;
    int dataSize;
    int searchStepSize;
    int locationDetailsSize;

    private String dest;
    private static int TimedOut = 0;
    float rtt[];
    float RTT[];
    int badRTTs[];
    Vector regions = new Vector();
    int aryRegions[];
    Hashtable trace = new Hashtable();
    Process p;
    boolean DEBUG = false;

    private ThreadGroup tg;
    private Timer timer;
    
    /* approx. radius of earth in meters. True radius varies from
       6357 km (polar) to 6378 km (equatorial)
    */
    public static final float earth_radius = 6367;
    public static final float speedoflightCU = 230;   //230 km/ms
    public static final float distanceOneDegree = 111;
    public static final int UNACPTRTT = 1;
    public static final int FUDGEFACTOR = 1;
    private final int IPADDRINDEX = 2;
    private final int RTTINDEX = 5;
    private final int LOCINDEX = 4;
    private final int PRIVATE = 80;
    private final int TIMEOUT = 90;
    private final int MAXNODES = 300;

    public static final String START = new String("Start");
    public static final String GOAL = new String("Goal");
    public static final String PING_ERROR = new String("vr hl tos");
    public String PING_OPT = new String();

    public static final int STEP2 = 2;
    public static final int STEP3 = 3;
    public static final int STEP4 = 4;
    public static final int STEP5 = 5;
    public static final int STEP6 = 6;
    public static final int STEP7 = 7;
    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------


/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:              GTraceTableModel model
**                              ThreadGroup tg
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     LocationVerifier(GTraceTableModel model, ThreadGroup tg)
     {
	  this.model = model;
	  this.tg = tg;

	  data = model.getData();
	  searchStep = model.getSearchStep();
	  locationDetails = model.getLocationDetails();
	  dataSize = data.size();
	  searchStepSize = searchStep.size();
	  locationDetailsSize = locationDetails.size();

          PING_OPT = GTrace.getGTraceResource("PING_OPT");

	  t = new Thread(tg, this, "Verifier");
	  t.start();			
     }


/******************************************************************************
**
**     Method name:             ping
**
**     Purpose:                 
**    
**     Parameters:              String ipaddr
**
**     Return Values:           float
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private float ping(String ipaddr)
    {
	try
	{
            int index = PING_OPT.indexOf("dest");
            
            String command = PING_OPT.substring(0,index);
            command = command.concat(ipaddr);
            command = command.concat(PING_OPT.substring(index+4));
            
	    command = GTrace.getGTraceResource("PING") + " " + command;
                             
	    GTrace.displayTextSBar(command);
	    p = Runtime.getRuntime().exec(command);			
	    
	    BufferedReader inStream = new BufferedReader(
                                  new InputStreamReader(p.getInputStream()));
	    String str;
	    boolean flag = true;				
	    int temp=99;	
	    String tokens[];

	    while(flag)
	    {	
		try
		{
		    if ((temp=p.exitValue()) == 0)
		    {
			flag = false;
		    }
		}
		catch(IllegalThreadStateException e)
		{
		    str=inStream.readLine();
		    
		    try
		    {
			if (str == null)
			{
			    return Float.MAX_VALUE;
			}
			else if (str.startsWith("round-trip"))
			{
			    GTrace.displayTextSBar(str);
			    tokens = Util.tokenizeToString(str,"=");
			    StringTokenizer tmp = new StringTokenizer(
                                                                tokens[1],"/");
			    Float minRTT = new Float(tmp.nextToken());
			    return minRTT.floatValue();
			}
			else if (str.toLowerCase().startsWith(PING_ERROR))
		        {
			    GTrace.displayTextSBar(str);
			    return Float.MAX_VALUE;
			}
		    }
		    catch(ArrayIndexOutOfBoundsException aieobe)
		    {
			return Float.MAX_VALUE;
		    }
		    catch(NoSuchElementException nsee)
		    {
			return Float.MAX_VALUE;
		    }
		}
	    }

	    boolean TRUE = true;
	    while(TRUE)
	    {
		str = inStream.readLine();
		    
		try
		{
		    if (str == null)
		    {
			return Float.MAX_VALUE;
		    }
		    else if (str.startsWith("round-trip"))
		    {
			GTrace.displayTextSBar(str);
			tokens = Util.tokenizeToString(str,"=");
			StringTokenizer tmp = new StringTokenizer(tokens[1]
                                                                  ,"/");
			Float minRTT = new Float(tmp.nextToken());
			return minRTT.floatValue();
		    }
		    else if (str.toLowerCase().startsWith(PING_ERROR))
		    {
			GTrace.displayTextSBar(str);
			return Float.MAX_VALUE;
		    }
		}
		catch(ArrayIndexOutOfBoundsException aieobe)
		{
		    return Float.MAX_VALUE;
		}
		catch(NoSuchElementException nsee)
		{
		    return Float.MAX_VALUE;
		}
	    }
	}
	catch(IOException e)
	{
	    GTrace.displayTextSBar("Command Failed" + e.getMessage());
	}

	return Float.MAX_VALUE;
    }


/******************************************************************************
**
**     Method name:             getStandardDeviation
**
**     Purpose:                 
**    
**     Parameters:              int bPtr
**                              int ePtr
**
**     Return Values:           float
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private float getStandardDeviation(int bPtr, int ePtr)
    {
	float mean;
	float variance=0;
	float sum=0;

	if (bPtr == ePtr)
	{
	    return 0;
	}

	for (int i=bPtr; i<=ePtr; i++)
	{
	    sum += RTT[i];
	}

	mean = sum / ((ePtr - bPtr) + 1);
	for (int i=bPtr; i<=ePtr; i++)
	{
	    variance += ((RTT[i] - mean) * (RTT[i] - mean));
	}

	if ((ePtr - bPtr) > 1 && bPtr != 0)
	{
	    variance /= (ePtr-bPtr);
	}
	else if ((ePtr - bPtr) > 1 && bPtr == 0)
	{
	    variance /= ePtr;
	}

	return new Float(Math.sqrt(new Float(variance).doubleValue()
                                   )).floatValue();
    }

/******************************************************************************
**
**     Method name:             getRTT
**
**     Purpose:                 
**    
**     Parameters:              int row
**
**     Return Values:           float
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public float getRTT(int row)
    {
	try
	{
	    hopData = (Object[])data.elementAt(row);
            String rttText = ((JLabel)hopData[RTTINDEX]).getText();
            if (rttText.length() > 0)
            {
                return Float.valueOf(rttText).floatValue();
            }
            else
            {
                return Float.POSITIVE_INFINITY;
            }
	}
	catch(NumberFormatException nfe)
	{
	    return Float.POSITIVE_INFINITY;
	}
    }

/******************************************************************************
**
**     Method name:             getIPADDR
**
**     Purpose:                 
**    
**     Parameters:              int row
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public String getIPADDR(int row)
    {
	hopData = (Object[])data.elementAt(row);
	return ((JLabel)hopData[IPADDRINDEX]).getText();
    }

/******************************************************************************
**
**     Method name:             getLocation
**
**     Purpose:                 
**    
**     Parameters:              int row
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public String getLocation(int row)
    {
	LocationDetails lDetail = (LocationDetails)locationDetails.elementAt(
                                                                          row);
	return lDetail.getLatLonString();
    }

/******************************************************************************
**
**     Method name:             run
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void run()
    {
	String ipaddr;
	float pingRTT;
	int numBadRTTs = 0;
	Stack hopRegions = new Stack();
	int locStep[];

	Integer region[] = new Integer[2];
	int bPtr;
	int ePtr;
	float max;
	float SD;
	int maxIndex;

	rtt = new float[dataSize];

        for (int i=0; i < dataSize; i++)
        {
            rtt[i] = getRTT(i);
        }
            
if (DEBUG)
{
    for (int i=0; i<rtt.length; i++)
    {
        System.out.println("rtt " + rtt[i]);
    }
}
 
	badRTTs = new int[rtt.length];
	for (int i=0; i<rtt.length; i++)
	{
	    badRTTs[i] = 0;
	}
	
	/* step 1 of verifier algorithm */

        int srchStep;
	if (rtt.length > 1)
	{
	    for(int i=0; i<rtt.length-1; i++)
	    {
		if (badRTTs[i] != UNACPTRTT && badRTTs[i+1] != UNACPTRTT)
		{
                    srchStep = ((LocationDetails)locationDetails.elementAt(i)
                                  ).getSearchStep().intValue();
		    if (rtt[i] > (rtt[i+1]+FUDGEFACTOR) &&
                        srchStep != PRIVATE && srchStep != TIMEOUT)
		    {
			ipaddr = Util.ipaddrToString(getIPADDR(i));
			if (ipaddr == null)
			{
			    badRTTs[i] = 1;
			    numBadRTTs++;
			}
			else
			{
			    pingRTT =  ping(ipaddr);

			    if (pingRTT == Float.MAX_VALUE)
			    {
				badRTTs[i] = 1;
				numBadRTTs++;
			    }
			    else
			    {
				if (pingRTT < rtt[i])
				{
				    rtt[i] = pingRTT;
				    model.changeRTT(rtt[i],i,tg.getName());
				}
			    
				if (rtt[i] > (rtt[i+1] + 1))
				{
				    badRTTs[i] = 1;
				    numBadRTTs++;
				}
			    }
			    i=-1;
			}
		    }
		}
		else if(badRTTs[i] != 1 && badRTTs[i+1] == 1)
		{
		    int k;
		    for (k=i+2; k<rtt.length-1; k++)
		    {
			if (badRTTs[k] != 1)
			{
			    break;
			}
		    }
		    
		    if (rtt[i] > (rtt[k] + 1))
		    {
			ipaddr = Util.ipaddrToString(getIPADDR(i));
			if (ipaddr == null)
			{
			    badRTTs[i] = 1;
			    numBadRTTs++;
			}
			else
			{
			    pingRTT =  ping(ipaddr);

			    if (pingRTT == Float.MAX_VALUE)
			    {
				badRTTs[i] = 1;
				numBadRTTs++;
			    }
			    else
			    {
				if (pingRTT < rtt[i])
				{
				    rtt[i] = pingRTT;
				    model.changeRTT(rtt[i],i,tg.getName());
				}
				
				if (rtt[i] > (rtt[k] + 1))
				{
				    badRTTs[i] = 1;
				    numBadRTTs++;
				}
			    }
			    i=-1;
			}
		    }
		}
	    }
	}

	for (int i=0; i < rtt.length; i++)
	{

if (DEBUG)
{
    System.out.println(badRTTs[i] + "\t" + rtt[i]);
}

            if (badRTTs[i] == 1)
	    {
		model.changeLocationColor(Color.red, i, tg.getName());
	    }
	}

        
	/* step 2 of RTT heuristics algorithm */


        RTT = new float[rtt.length-numBadRTTs];
	for (int i=0, z=0; i<rtt.length; i++)
	{
	    if (badRTTs[i] != 1)
	    {
		RTT[z++] = rtt[i];
	    }
	}


       hopRegions.push(new Integer[] {new Integer(0),new Integer(
                                                              RTT.length-1)});
	
       while(!hopRegions.empty())
       {   
           region = (Integer[])hopRegions.pop();
	   bPtr = region[0].intValue();
	   ePtr = region[1].intValue();

	   SD = getStandardDeviation(bPtr,ePtr);

	   if (SD > 2.5)  
	   {
	       float rttDiffs[] = new float[ePtr - bPtr];
	       max =0;
	       maxIndex = bPtr;

	       for (int i=bPtr, z=0; i<ePtr; i++, z++)
	       {
	           rttDiffs[z] = Math.abs(RTT[i]-RTT[i+1]);
	       
		   if (rttDiffs[z] > max)
		   {
		       max = rttDiffs[z];
		       maxIndex = i;
		   }
	       }
	       
	       hopRegions.push(new Integer[] {new Integer(bPtr),
                                                  new Integer(maxIndex)});
	       hopRegions.push(new Integer[] {new Integer(maxIndex+1),
                                                  new Integer(ePtr)});
	   }
	    else
	    {
		regions.addElement(region);
	    }
	}

	int indexBadRTTs[] = new int[numBadRTTs];
	int z=0;
	for (int i=0; i<badRTTs.length; i++)
	{
	    if (badRTTs[i] == 1)
	    {
		indexBadRTTs[z++] = i;
	    }
	}

	int rsize = regions.size();
	
	for (int n=0; n<indexBadRTTs.length; n++)
	{
	    for (int i=0; i<rsize; i++)
	    {
		region = (Integer[])regions.elementAt(i);
		if (region[0].intValue() >= indexBadRTTs[n])
		{
		    region[0] = new Integer(region[0].intValue()+1);
		}
		if (region[1].intValue() >= indexBadRTTs[n])
		{
		    region[1] = new Integer(region[1].intValue()+1);
		}
		regions.removeElementAt(i);
		regions.insertElementAt(region, i);
	    }
	}

	aryRegions = new int[rsize*2];
	int index = (rsize*2)-1;
	for (Enumeration e = regions.elements() ; e.hasMoreElements() ;) 
        {
	    region = (Integer[])e.nextElement();
	    aryRegions[index--] = region[1].intValue();
	    aryRegions[index--] = region[0].intValue();
	}

if (DEBUG)
{
    for (int i=0; i<aryRegions.length; i+=2)  /* xxx- check to make sure
                                                 all regions are in ascending
                                                 order
                                              */
    {
        System.out.println("Begin = "  + aryRegions[i] + " End = "
                           + aryRegions[i+1]);
    }
    System.out.println("---------------------------------------"
                       + "----------------");
}
 
	// build location step array
	locStep = new int[rtt.length];
	int step;
 	for (int i=0; i < rtt.length; i++)
	{
	    step = ((Integer)searchStep.elementAt(i)).intValue();
	    locStep[i] = (step - (step % 10)) / 10;
	}

	
	/* Step 3,4,5 of verifier algorithm */

	Hashtable vNodes = new Hashtable();
	Integer vNodeIdentifier = new Integer(1);

        String cityNames[] = new String[rtt.length];
        for (int i=0; i < rtt.length; i++)
        {
            cityNames[i] = model.getLocationCity(i);
        }
        
	verifierNode vnode = new verifierNode(vNodeIdentifier, rtt,
                                              locStep, cityNames);
	verifierNode children[] = new verifierNode[2];
	vNodes.put(vnode.getIdentifier(), vnode);

	Vector verifierTree = new Vector();

        verifierTree.addElement(vnode.getIdentifier());
	Integer nextVNode;
        boolean UNSOLVABLE = false;
        int numNodesVisited = 0;
        int goalStates=0;

        
	while(verifierTree.size() > 0)
	{
            if (numNodesVisited > MAXNODES) 
            {
                UNSOLVABLE = true;
                break;
            }
                 
	    nextVNode = (Integer)verifierTree.firstElement();
            verifierTree.removeElementAt(0);
	    vnode = (verifierNode)vNodes.get(nextVNode);

	    if (vnode != null)
	    {
		children = vnode.generateChildren();

		if (children == null)
                {
                    break;
                }
                else
		{
                    //children[0].print();                    
		    vNodeIdentifier = incInteger(vNodeIdentifier);
		    children[0].setIdentifier(vNodeIdentifier);
		    vnode.setLChild(vNodeIdentifier);
		    vNodes.put(vNodeIdentifier, children[0]);
		    verifierTree.addElement(vNodeIdentifier);
                    numNodesVisited++;
                    
                    if (children.length == 2)
		    {
                        //children[1].print();  
			vNodeIdentifier = incInteger(vNodeIdentifier);
			children[1].setIdentifier(vNodeIdentifier);
			vnode.setRChild(vNodeIdentifier);
			vNodes.put(vNodeIdentifier, children[1]);
			verifierTree.addElement(vNodeIdentifier);
                        numNodesVisited++;
		    }
		}
	    }
	}


	/* all nodes in vNodes that have their status as good are the
           final good configurations work the way up to their parents
           to see what changes were made to them
        */

        if (!UNSOLVABLE)
        {

if (DEBUG)
{
    System.out.println("----------------------------------"
                       + "---------------------");
    
    for (Enumeration e = vNodes.elements() ; e.hasMoreElements() ;) 
    {
        vnode = (verifierNode)e.nextElement();
        
        if (vnode.getStatus().compareTo(GOAL) == 0)
        {
            goalStates++;
        }
    }
    
    System.out.println("number of goal states = " + goalStates);
}
 
            for (Enumeration e = vNodes.elements() ; e.hasMoreElements() ;) 
            {
                vnode = (verifierNode)e.nextElement();

                if (vnode.getStatus().compareTo(GOAL) == 0)
                {
                    Vector changes = new Vector();

                    changes.addElement(vnode.getChanges());

if (DEBUG)
{
    System.out.println(vnode.getChngMade());
}
                    
                    Integer parent = vnode.getParent();

                    if (parent != null)
                    {
                        updateTable(vnode.getChngMade(),
                                    ((verifierNode)vNodes.get(parent)
                                     ).getChanges());
                    }
                    else
                    {
                        updateTable(vnode.getChngMade(), null);
                    }
                    
                    while(parent != null && parent.intValue() >= 1)
                    {
                        vnode = (verifierNode)vNodes.get(parent);
                        changes.addElement(vnode.getChanges());

if (DEBUG)
{
    System.out.println(vnode.getChngMade());
}

                        parent = vnode.getParent();
                        
                        if (parent != null)
                        {
                            updateTable(vnode.getChngMade(),
                                        ((verifierNode)vNodes.get(parent)
                                         ).getChanges());
                        }
                        else
                        {
                            updateTable(vnode.getChngMade(), null);
                        }
                    }

                    GTrace.redrawMap();
if (DEBUG)
{
    int numMessages = changes.size();
    for (int i=numMessages-1; i>=0; i--)
    {
        System.out.println((String)changes.elementAt(i));
    }
    System.out.println("--------------------------------"
                       + "-----------------------");
}

                    boolean badrtt = false;
                    for (int i=0; i < rtt.length; i++)
                    {
                        if (badRTTs[i] == 1)
                        {
                            badrtt = true;
                            break;
                        }
                    }

                    if (badrtt)
                    {
                        GTrace.displayTextSBar(
                                   "There were hops with bad RTTs, " +
                                   "the verification results might be wrong " +
                                   "as a result of this.");
                    }
                    else
                    {
                        GTrace.displayTextSBar(
                                      "Verification of locations completed, " +
                                      "click on locations for explanations");
                    }
                    break;
                }
            }
        }
        else
        {
            GTrace.displayTextSBar("The verifier could not arrive "
                                   + "at a suitable result");
        }

    }

/******************************************************************************
**
**     Method name:             updateTable
**
**     Purpose:                 
**    
**     Parameters:              String hops
**                              String changes
**
**     Return Values:           void 
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/

    private void updateTable(String hops, String changes)
    {
        if (hops.length() > 0 && hops.indexOf('-') > 0)
        {
            int rows[] = Util.tokenizeToint(hops,"-");
            model.updateTable(rows[0], rows[1], changes, tg.getName());
        }
   }

/******************************************************************************
**
**     Method name:             incInteger
**
**     Purpose:                 
**    
**     Parameters:              Integer num
**
**     Return Values:           Integer
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/
    
    private Integer incInteger(Integer num)
    {
	int tmp = num.intValue();
	return new Integer(++tmp);
    }


//---------------------------------------------------------------------------
//                             inner classes
//---------------------------------------------------------------------------

class verifierNode
{
    int interRegion[];
    int intraRegion[];

    float rtt[];
    float hopDist[];
    int locStep[];
    float latLoc[];
    float lonLoc[];
    String cityNames[];

    Integer identifier;
    Integer parent;
    Integer lChild;
    Integer rChild;

    String chngDescription;
    String status;
    String chngMade;

/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public verifierNode(Integer parent, float rtt[], int interRegion[],
                        int intraRegion[], float latLoc[], float lonLoc[],
                        int locStep[], int srcIndex,
                        int chgIndex, String cityNames[])
    {
	this.parent = parent;

	this.rtt = rtt;
 	this.interRegion = interRegion;
	this.intraRegion = intraRegion;
	this.locStep = locStep;
        this.cityNames = cityNames;

	this.latLoc = latLoc;
	this.lonLoc = lonLoc;
	createHopDist();
	updateInterRegion();
	updateIntraRegion();

        chngMade = new String(chgIndex + "-" + srcIndex);
        status = new String("");
	rChild = null;
	lChild = null;
    }

/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:              
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public verifierNode(Integer identifier, float rtt[], int locStep[],
                        String cityNames[])
    {
	this.identifier = identifier;
	this.rtt = (float[])rtt.clone();
	this.locStep = (int[])locStep.clone();
        this.cityNames = (String[])cityNames.clone();
	createLatLon();
	createHopDist();
	createInterRegion();
	createIntraRegion();

        status = START;
        chngMade = new String("");
	parent = null;
	rChild = null;
	lChild = null;
    }

/******************************************************************************
**
**     Method name:             getCity
**
**     Purpose:                 
**    
**     Parameters:              int hop
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 26th, 1999
**
******************************************************************************/

    private String getCity(int hop)
    {
        if (cityNames == null)
        {
            return "where node ? is";
        }
        else if (cityNames[hop].startsWith("UNKNOWN"))
        {
            return "where node " + (hop+1) + " is";
        }
        else
        {
            return cityNames[hop];
        }
    }

/******************************************************************************
**
**     Method name:             generateChildren
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           verifierNode[]
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public verifierNode[] generateChildren()
    {
	int bLocStep;
	int eLocStep;
	int region[];

	// check interRegions first

        for (int i=1; i<rtt.length; i++)
        {
            if (interRegion[i] == 1  && badRTTs[i] != 1 && badRTTs[i-1] != 1)
            {
                chngDescription = new String(
                          "Found location discrepancy between hops " + i
                          + " and " + (i+1) + "\n");
                chngDescription = chngDescription.concat(
                      "Its not possible to travel a round trip distance of "
                      + Util.roundNum(hopDist[i]) +
                      " km (" + getCity(i-1) + " to " + getCity(i) + ") in "
                      + Util.roundNum(Math.abs(rtt[i-1] - rtt[i])) + "ms\n"); 
                                                         
                bLocStep = locStep[i-1];
                eLocStep = locStep[i];

                // rules
                // here you should always be using i and i-1 for the arrays
                // only i and i+1 when debugging as they refer to the actual
                // hop numbers

                if (bLocStep == STEP2 || bLocStep == STEP3 && eLocStep > STEP2)
                {
                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i-1, i);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                  "Changed hop " + (i+1)
                                  + "'s location to that of hop " + i + "\n");
                    }

                    chngDescription = chngDescription.concat(
                               "Its probably the case that node " + (i+1)
                               + " is located around " + getCity(i-1) + "\n");
                    return tmp;
                }
                else if (eLocStep == STEP2 || eLocStep == STEP3
                         && bLocStep > STEP2)
                {
                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i, i-1);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                               "Changed hop " + i
                               + "'s location to that of hop " + (i+1) + "\n");
                    }

                    chngDescription = chngDescription.concat(
                                  "Its probably the case that node " + i
                               + " is located around " + getCity(i) + "\n");
                    
                    return tmp;
                }
                else if (bLocStep == STEP4 && eLocStep >= STEP4)
                {
                    if (eLocStep == STEP4 || eLocStep == STEP5)
                    {
                        verifierNode tmp[] = new verifierNode[2];
                        tmp[0] = newNode(i-1, i);
                        tmp[1] = newNode(i, i-1);
                        chngDescription = chngDescription.concat(
                          "Interchanging and trying both locations as one "
                          + "cannot be definetly eliminated\n");
                        return tmp;
                    }

                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i-1, i);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                         "Changed hop " + (i+1)
                                         + "'s location to that of hop " + i);
                    }

                    chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + (i+1)
                               + " is located around " + getCity(i-1) + "\n");
                    
                    return tmp;
                }
                else if (eLocStep == STEP4 && bLocStep >= STEP4)
                {
                    if (bLocStep == STEP4 || bLocStep == STEP5)
                    {
                        verifierNode tmp[] = new verifierNode[2];
                        tmp[0] = newNode(i-1, i);
                        tmp[1] = newNode(i, i-1);
                        chngDescription = chngDescription.concat(
                          "Interchanging and trying both locations as one "
                          + "cannot be definetly eliminated\n");
                        return tmp;
                    }

                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i, i-1);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                    "Changed hop " + i
                                    + "'s location to that of hop " + (i+1));
                    }

                    chngDescription = chngDescription.concat(
                                    "Its probably the case that node " + i
                                 + " is located around " + getCity(i) + "\n");
                    return tmp;

                }
                else if (bLocStep == STEP5 && eLocStep >= STEP5)
                {
                    if (eLocStep == STEP5)
                    {
                        verifierNode tmp[] = new verifierNode[2];
                        tmp[0] = newNode(i-1, i);
                        tmp[1] = newNode(i, i-1);
                        chngDescription = chngDescription.concat(
                           "Interchanging and trying both locations as "
                           + "one cannot be definetly eliminated\n");
                        return tmp;
                    }

                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i-1, i);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                      "Changed hop " + (i+1)
                                      + "'s location to that of hop " + i);
                    }

                    chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + (i+1)
                                + " is located around " + getCity(i-1) + "\n");
                    return tmp;
                }
                else if (eLocStep == STEP5 && bLocStep >= STEP5)
                {
                    if (bLocStep == STEP5)
                    {
                        verifierNode tmp[] = new verifierNode[2];
                        tmp[0] = newNode(i-1, i);
                        tmp[1] = newNode(i, i-1);
                        chngDescription = chngDescription.concat(
                           "Interchanging and trying both locations as "
                           + "one cannot be definetly eliminated\n");
                        return tmp;
                    }

                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i, i-1);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                          "Changed hop " + i
                                     + "'s location to that of hop " + (i+1));
                    }

                    chngDescription = chngDescription.concat(
                                  "Its probably the case that node " + i
                              + " is located around " + getCity(i) + "\n");
                    return tmp;
                }
                else if (bLocStep == STEP6 && eLocStep >= STEP6)
                {
                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i-1, i);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                        "Changed hop " + (i+1)
                                         + "'s location to that of hop " + i);
                    }

                    chngDescription = chngDescription.concat(
                                 "Its probably the case that node " + (i+1)
                                + " is located around " + getCity(i-1) + "\n");
                    return tmp;
                }
                else if (eLocStep == STEP6 && bLocStep >= STEP6)
                {
                    verifierNode tmp[] = new verifierNode[1];
                    tmp[0] = newNode(i, i-1);

                    if (DEBUG)
                    {
                        chngDescription = chngDescription.concat(
                                       "Changed hop " + i
                                    + "'s location to that of hop " + (i+1));
                    }

                    chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + i
                                + " is located around " + getCity(i) + "\n");
                    return tmp;
                }
            }
        }


        // next work on the intraRegions

        for (int i=1; i<rtt.length; i++)
        {
            if (intraRegion[i] == 1 && badRTTs[i] != 1 && badRTTs[i-1] != 1)
            {
                chngDescription = new String(
                             "Found location discrepancy between hops " + i
                             + " and " + (i+1) + "\n");
                chngDescription = chngDescription.concat(
                       "Its not possible to travel a round trip distance of "
                       + Util.roundNum(hopDist[i]) +
                       " km (" + getCity(i-1) + " to " + getCity(i) + ") in "
                       + Util.roundNum(Math.abs(rtt[i-1] - rtt[i])) + "ms\n"); 

                if (DEBUG)
                {
                    chngDescription = chngDescription.concat(
                           "Hop "+i+"'s location was obtained from step "
                           + locStep[i-1] + " and hop " + (i+1)
                           +"'s location was obtained from step "
                           + locStep[i] + "\n");
                }
                
                bLocStep = locStep[i-1];
                eLocStep = locStep[i];
                region = getRegion(i);

                if (region != null)
                {
                    chngDescription = chngDescription.concat(
                         "These hops fall within the geographical cluster "
                         + "(based on the RTT's) beginning at hop " +
                         (region[0]+1) + " and ending at hop " + (region[1]+1)
                         + "\n");

                    if (bLocStep == STEP2 || bLocStep == STEP3
                        && eLocStep > STEP2)
                    {
                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i-1, i);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                      "Changed hop " + (i+1)
                                   + "'s location to that of hop " + i + "\n");
                        }

                        chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + (i+1)
                                + " is located around " + getCity(i-1) + "\n");
                        return tmp;
                    }
                    else if (eLocStep == STEP2 || eLocStep == STEP3
                             && bLocStep > STEP2)
                    {
                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i, i-1);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                   "Changed hop " + (i)
                               + "'s location to that of hop " + (i+1) + "\n");
                        }

                        chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + i
                                 + " is located around " + getCity(i) + "\n");
                        
                        return tmp;
                    }
                    else if (bLocStep == STEP4 && eLocStep >= STEP4)
                    {
                        if (interRegion[region[0]] == 2)
                        {
                            verifierNode tmp[] = new verifierNode[1];
                            tmp[0] = newNode(i, i-1);

                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                       "Changed hop " + (i)
                                        + "'s location to that of hop "
                                       + (i+1) + "\n");
                            }

                            chngDescription = chngDescription.concat(
                                    "Its probably the case that node " + i
                                    + " is located around " + getCity(i)
                                    + "\n");
                            return tmp;
                        }
                        else if (eLocStep == STEP4 || eLocStep == STEP5)
                        {
                            verifierNode tmp[] = new verifierNode[2];
                            tmp[0] = newNode(i-1, i);
                            tmp[1] = newNode(i, i-1);

                            chngDescription = chngDescription.concat(
                               "Interchanging and trying both locations "
                               + "as one cannot be definetly eliminated\n");

                            return tmp;
                        }
			
                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i-1, i);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                          "Changed hop " + (i+1)
                                   + "'s location to that of hop " + i + "\n");
                        }

                        chngDescription = chngDescription.concat(
                              "Its probably the case that node " + (i+1)
                             + " is located around " + getCity(i-1) + "\n");
                        return tmp;
                    }
                    else if (eLocStep == STEP4 && bLocStep >= STEP4)
                    {
                        if (interRegion[region[0]] == 2)
                        {
                            verifierNode tmp[] = new verifierNode[1];
                            tmp[0] = newNode(i-1, i);

                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                      "Changed hop " + (i+1)
                                   + "'s location to that of hop " + i + "\n");
                            }
                            
                            chngDescription = chngDescription.concat(
                                "Its probably the case that node " + (i+1)
                                + " is located around " + getCity(i-1) + "\n");
                            return tmp;
                        }
                        else if (bLocStep == STEP4 || bLocStep == STEP5)
                        {
                            verifierNode tmp[] = new verifierNode[2];
                            tmp[0] = newNode(i-1, i);
                            tmp[1] = newNode(i, i-1);
                            chngDescription = chngDescription.concat(
                                "Interchanging and trying both locations "
                                + "as one cannot be definetly eliminated\n");
                            return tmp;
                        }
			
                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i, i-1);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                   "Changed hop " + (i)
                               + "'s location to that of hop " + (i+1) + "\n");
                        }
                        
                        chngDescription = chngDescription.concat(
                                    "Its probably the case that node " + i
                                  + " is located around " + getCity(i) + "\n");
                        return tmp;
                    }
                    else if (bLocStep == STEP5 && eLocStep >= STEP5)
                    {
                        if (eLocStep == STEP5  || interRegion[region[0]] == 2)
                        {
                            verifierNode tmp[] = new verifierNode[2];
                            tmp[0] = newNode(i-1, i);
                            tmp[1] = newNode(i, i-1);
                            chngDescription = chngDescription.concat(
                                  "Interchanging and trying both locations "
                                  + "as one cannot be definetly eliminated\n");
                            return tmp;
                        }

                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i-1, i);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                     "Changed hop " + (i+1)
                                  + "'s location to that of hop " + i + "\n");
                        }

                        chngDescription = chngDescription.concat(
                              "Its probably the case that node " + (i+1)
                              + " is located around " + getCity(i-1) + "\n");
                        
                        return tmp;
                    }
                    else if (eLocStep == STEP5 && bLocStep >= STEP5)
                    {
                        if (bLocStep == STEP5 || interRegion[region[0]] == 2)
                        {
                            verifierNode tmp[] = new verifierNode[2];
                            tmp[0] = newNode(i-1, i);
                            tmp[1] = newNode(i, i-1);
                            chngDescription = chngDescription.concat(
                                "Interchanging and trying both locations "
                                + "as one cannot be definetly eliminated\n");
                            return tmp;
                        }

                        verifierNode tmp[] = new verifierNode[1];
                        tmp[0] = newNode(i, i-1);

                        if (DEBUG)
                        {
                            chngDescription = chngDescription.concat(
                                       "Changed hop " + (i)
                                       + "'s location to that of hop "
                                       + (i+1) + "\n");
                        }

                        chngDescription = chngDescription.concat(
                                    "Its probably the case that node " + i
                                    + " is located around " + getCity(i)
                                    + "\n");
                        return tmp;
                    }
                    else if (bLocStep == STEP6 && eLocStep >= STEP6)
                    {
                        verifierNode tmp[] = new verifierNode[1];

                        if (interRegion[region[0]] == 2)
                        {
                            tmp[0] = newNode(i, i-1);

                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                           "Changed hop " + i
                                    + "'s location to that of hop " + (i+1)
                                           + "\n");
                            }

                            chngDescription = chngDescription.concat(
                                     "Its probably the case that node " + i
                                  + " is located around " + getCity(i) + "\n");
                        }
                        else
                        {
                            tmp[0] = newNode(i-1, i);

                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                          "Changed hop " + (i+1)
                                           + "'s location to that of hop "
                                          + i + "\n");
                            }

                            chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + (i+1)
                                   + " is located around " + getCity(i-1)
                                   + "\n");
                        }
                        return tmp;
                    }
                    else if (eLocStep == STEP6 && bLocStep >= STEP6)
                    {
                        verifierNode tmp[] = new verifierNode[1];

                        if (interRegion[region[0]] == 2)
                        {
                            tmp[0] = newNode(i, i-1);
                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                       "Changed hop " + (i)
                                       + "'s location to that of hop "
                                       + (i+1) + "\n");
                            }

                            chngDescription = chngDescription.concat(
                                    "Its probably the case that node " + i
                                    + " is located around " + getCity(i)
                                    + "\n");
                        }
                        else
                        {
                            tmp[0] = newNode(i-1, i);

                            if (DEBUG)
                            {
                                chngDescription = chngDescription.concat(
                                      "Changed hop " + (i+1)
                                     + "'s location to that of hop "
                                      + i + "\n");
                            }

                            chngDescription = chngDescription.concat(
                                   "Its probably the case that node " + (i+1)
                                   + " is located around " + getCity(i-1)
                                   + "\n");
                        }                        
                        return tmp;
                    }
                }
            }
        }

        chngDescription = new String("A valid state\n");
        status = GOAL;
        return null; 
    }	

/******************************************************************************
**
**     Method name:             newNode
**
**     Purpose:                 
**    
**     Parameters:              int srcIndex
**                              int chgIndex
**
**     Return Values:           verifierNode
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private verifierNode newNode(int srcIndex, int chgIndex)
    {
	float tmpLatLoc[] = (float[])latLoc.clone();
	float tmpLonLoc[] = (float[])lonLoc.clone();
        String tmpCityNames[] = (String[])cityNames.clone();
	tmpLatLoc[chgIndex] = latLoc[srcIndex];
	tmpLonLoc[chgIndex] = lonLoc[srcIndex];
        tmpCityNames[chgIndex] = cityNames[srcIndex];
        
	int tmpLocStep[] = (int[])locStep.clone();
	tmpLocStep[chgIndex] = locStep[srcIndex];

	return new verifierNode(identifier, (float[])rtt.clone(), 
                                (int[])interRegion.clone(),
                                (int[])intraRegion.clone(), 
                                tmpLatLoc, tmpLonLoc, tmpLocStep,
                                srcIndex, chgIndex, tmpCityNames);
    }

/******************************************************************************
**
**     Method name:             getRegion
**
**     Purpose:                 
**    
**     Parameters:              int hop
**
**     Return Values:           int[]
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private int[] getRegion(int hop)
    {
 	int bPtr;
	int ePtr;

	for (int i=0; i<aryRegions.length-1; i+=2)
	{
	    if (hop >= aryRegions[i] && hop <= aryRegions[i+1])
	    {
		return (new int[] {aryRegions[i],aryRegions[i+1]});
	    }
	}

	return null;
    }

/******************************************************************************
**
**     Method name:             setIdentifier
**
**     Purpose:                 
**    
**     Parameters:              Integer indentifier
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void setIdentifier(Integer indentifier)
    {
	this.identifier = indentifier;
    }

/******************************************************************************
**
**     Method name:             setParent
**
**     Purpose:                 
**    
**     Parameters:              Integer parent
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void setParent(Integer parent)
    {
	this.parent = parent;
    }

/******************************************************************************
**
**     Method name:             setLChild
**
**     Purpose:                 
**    
**     Parameters:              Integer lChild
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void setLChild(Integer lChild)
    {
	this.lChild = lChild;
    }

/******************************************************************************
**
**     Method name:             setRChild
**
**     Purpose:                 
**    
**     Parameters:              Integer rChild
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void setRChild(Integer rChild)
    {
	this.rChild = rChild;
    }

    
/******************************************************************************
**
**     Method name:             getIdentifier
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           Integer
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Integer getIdentifier()
    {
	return identifier;
    }

/******************************************************************************
**
**     Method name:             getParent
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           Integer
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Integer getParent()
    {
	return parent;
    }

/******************************************************************************
**
**     Method name:             getLChild
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           Integer 
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Integer getLChild()
    {
	return lChild;
    }

/******************************************************************************
**
**     Method name:             getRChild
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           Integer 
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public Integer getRChild()
    {
	return rChild;
    }

/******************************************************************************
**
**     Method name:             isValid
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           boolean
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public boolean isValid()
    {
	for (int i=0; i<rtt.length; i++)
	{
	    if (interRegion[i] != 0 && intraRegion[i] != 0)
	    {
		return false;
	    }
	}
	
	return true;
    }

/******************************************************************************
**
**     Method name:             getChanges
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/
    
    public String getChanges()
    {
        return chngDescription;
    }

/******************************************************************************
**
**     Method name:             getChngMade
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/
    
    public String getChngMade()
    {
        return chngMade;
    }

/******************************************************************************
**
**     Method name:             getStatus 
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/
    
    public String getStatus()
    {
        return status;
    }
    
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void createLatLon()
    {
	latLoc = new float[rtt.length];
	lonLoc = new float[rtt.length];

	String location[] = new String[2];

 	for (int i=0; i < rtt.length; i++)
	{
	    location = Util.tokenizeToString(getLocation(i),",");

	    try
	    {
		if (location != null
                    && !location[0].startsWith("Private Internet Use"))
		{
		    latLoc[i] = Util.stringToLat(location[0]);
		    lonLoc[i] = Util.stringToLon(location[1]);
		}
                else
                {
		    latLoc[i] = Float.NaN;
		    lonLoc[i] = Float.NaN;
                }
	    }
	    catch(ArrayIndexOutOfBoundsException aiofbe)
	    {
		latLoc[i] = Float.POSITIVE_INFINITY;
		lonLoc[i] = Float.POSITIVE_INFINITY;
	    }
	}

    }

/******************************************************************************
**
**     Method name:             createHopDist
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void createHopDist()
    {
	hopDist = new float[rtt.length];
	hopDist[0] = 0;

	for (int i=0; i < rtt.length-1; i++)
	{
            if (latLoc[i] == Float.NaN || lonLoc[i] == Float.NaN
                || latLoc[i+1] == Float.NaN || lonLoc[i+1] == Float.NaN) 
            {
                hopDist[i+1] = 0;
                continue;
            }
            
	    hopDist[i+1]  = 2*getDistance(latLoc[i],
					lonLoc[i],
					latLoc[i+1],
					lonLoc[i+1]);
if (DEBUG)
{
    System.out.println("hopdist["+(i+1)+"]= " + hopDist[i+1]);
}
	}

     }

/******************************************************************************
**
**     Method name:             createInterRegion
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void createInterRegion()
    {
        interRegion = new int[rtt.length];
 	int bPtr;
	int ePtr;

	float rttDiff;

	for (int i=0; i<rtt.length; i++)
	{
	    interRegion[i] = 0;
	}

	for (int i=1; i<aryRegions.length-1; i+=2)
	{
	    bPtr = aryRegions[i];  /* xxx - check to make sure that bPtr
                                      and ePtr are correct
                                   */
	    ePtr = aryRegions[i+1];

	    rttDiff = Math.abs(rtt[bPtr] - rtt[ePtr]);

	    if (hopDist[ePtr] > rttDiff*speedoflightCU)
	    {
		interRegion[ePtr] = 1;
	    }
	    else if (hopDist[ePtr] < distanceOneDegree)
	    {
		interRegion[ePtr] = 2;
	    }
	}

    }
    
/******************************************************************************
**
**     Method name:             createIntraRegion
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void createIntraRegion()
    {
	intraRegion = new int[rtt.length];
	int bPtr;
	int ePtr;
	float rttDiff;

	for (int i=0; i<rtt.length; i++)
	{
	    intraRegion[i] = 0;
	}

	for (int i=0; i<aryRegions.length; i+= 2)
	{
	    bPtr = aryRegions[i];
	    ePtr = aryRegions[i+1];
	    
  	    for (int j=bPtr;j<ePtr; j++)
	    {
		rttDiff = Math.abs(rtt[j+1] - rtt[j]);
		
		if (rttDiff < 1 && hopDist[j+1] < speedoflightCU)
		{
		    continue;
		}

		if (hopDist[j+1] > rttDiff*speedoflightCU)
		{
		    intraRegion[j+1] = 1;
		}
  
	    }
	}
    }


/******************************************************************************
**
**     Method name:             updateInterRegion
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void updateInterRegion()
    {
	int bPtr;
	int ePtr;

	float rttDiff;

	for (int i=1; i<aryRegions.length-1; i+=2)
	{
	    bPtr = aryRegions[i];  /* xxx - check to make sure that bPtr
                                      and ePtr are correct
                                   */
	    ePtr = aryRegions[i+1];

	    rttDiff = Math.abs(rtt[bPtr] - rtt[ePtr]);
	    if (hopDist[ePtr] > rttDiff*speedoflightCU)
	    {
 		interRegion[ePtr] = 1;
	    }
	    else if (hopDist[ePtr] < distanceOneDegree)
	    {
		interRegion[ePtr] = 2;
	    }
	    else
	    {
		interRegion[ePtr] = 0;
	    }
	}

    }
    
/******************************************************************************
**
**     Method name:             updateIntraRegion
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    private void updateIntraRegion()
    {
	int bPtr;
	int ePtr;
	float rttDiff;

	for (int i=0; i<aryRegions.length; i+= 2)
	{
	    bPtr = aryRegions[i];
	    ePtr = aryRegions[i+1];
	    
  	    for (int j=bPtr;j<ePtr; j++)
	    {
		rttDiff = Math.abs(rtt[j+1] - rtt[j]);
		
		if (rttDiff < 1 && hopDist[j+1] < speedoflightCU)
		{
		    intraRegion[j+1] = 0;
		    continue;
		}

		if (hopDist[j+1] > rttDiff*speedoflightCU)
		{
		    intraRegion[j+1] = 1;
		}
		else
		{
		    intraRegion[j+1] = 0;
		}
	    }
	}

    }

/******************************************************************************
**
**     Method name:             getDistance
**
**     Purpose:                 
**    
**     Parameters:              float lat1
**                              float lon1
**                              float lat2
**                              float lon2
**
**     Return Values:           float
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public float getDistance(float lat1, float lon1, float lat2, float lon2)
    {
        double lat1rad = (lat1 * Math.PI)/180; 
        double lon1rad = (lon1 * Math.PI)/180; 
        double lat2rad =  (lat2 * Math.PI)/180; 
        double lon2rad = (lon2 * Math.PI)/180; 

	double dlon = (double)(lon2rad - lon1rad);
	double dlat = (double)(lat2rad - lat1rad);
 

	double a = Math.pow((Math.sin(dlat / 2)),2.0) + Math.cos(lat1rad) 
	          * Math.cos(lat2rad) * Math.pow(Math.sin(dlon/2),2.0);

	float d = new Float(2 * Math.atan2(Math.sqrt(a),
                                           Math.sqrt(1 - a))).floatValue();

	return (earth_radius * d);
    }

/******************************************************************************
**
**     Method name:             print
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public void print()
    {
	System.out.println("------------------------------"
                           + "-------------------------");
	if (parent != null)
	{
	    System.out.println("Parent = " + parent.toString());
	}

	System.out.println("IntraRegion");
	for (int i=0; i<rtt.length; i++)
	{
	    System.out.println((i+1) + " --> " + intraRegion[i]);
	}

	System.out.println("InterRegion");
	for (int i=0; i<rtt.length; i++)
	{
	    System.out.println((i+1) + " --> " + interRegion[i]);
	}

    }
}

}
