/******************************************************************************
**
**     File:                    LookupClient.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept 5th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

package caida.GTrace;

import java.net.Socket;
import java.net.UnknownHostException;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.FileOutputStream;

public class LookupClient
{
    private String server=null;
    private int port=55555;
    private Socket echoSocket = null;
    private PrintWriter out = null;
    private BufferedReader in = null;


//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 18th, 1999
**
******************************************************************************/

    public LookupClient(String server, int port)
    {
	this.server = server;
	this.port = port;
	
	try 
	{
	    if (server != null) 
	    {
		echoSocket = new Socket(server, port);
		out = new PrintWriter(echoSocket.getOutputStream(), true);
		in = new BufferedReader(new InputStreamReader(
                                                echoSocket.getInputStream()));
	    }
	} 
	catch (UnknownHostException e) 
	{
	    System.err.println("Don't know about host: " + server);
	    System.exit(1);
	} 
	catch (IOException e) 
	{
	    System.err.println("Couldn't get I/O for "
                               + "the connection to: " + server);
	    System.exit(1);
	}

    }
    
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 18th, 1999
**
******************************************************************************/

    private static void usage()
    {
        System.err.println("usage: LookupClient -s Server -f "
                           + "inputFile [-p portnumber]\n");
        System.exit(1);
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 18th, 1999
**
******************************************************************************/

    public void readData(String fname)
    {
	RandomAccessFile fptr;

 	try
	{
	    fptr = new RandomAccessFile(fname, "r");
	    FileOutputStream fout = new FileOutputStream("noloc");

	    String line;
	    String inLine;

	    while ((line = fptr.readLine()) != null)
	    {
		out.println(line);
		
		if (line.compareTo("Done") != 0)
		{
		    try
		    {
			inLine = in.readLine();
			if (!inLine.endsWith("null"))
			{
			    System.out.println(line + "--" + inLine);
			}
			else
			{
			    fout.write(line.getBytes());
			    fout.write('\n');
			    fout.flush();
			}
		    }
		    catch(IOException ioe) 
		    {
			System.out.println("Error reading from socket");
		    }
		}
	    }
	    
	    fptr.close();
	    fout.close();
	    out.close();
	    in.close();
	    echoSocket.close();
	}
 	catch(IOException ioe) 
	{
	    System.out.println("Couldn't open file : " + fname);
	    System.exit(-1);
	}

    }


    /*
	BufferedReader stdIn = new BufferedReader(
                                             new InputStreamReader(System.in));
	String userInput;

	while ((userInput = stdIn.readLine()) != null) 
	{
	    out.println(userInput);
	    System.out.println("echo: " + in.readLine());
	}

	stdIn.close();
    */


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 18th, 1999
**
******************************************************************************/


    public static void main(String argv[]) throws IOException 
    {

	String param = new String();
	String fname = new String();
	int port=55555;
	String server = new String();

        for (int i = 0; i < argv.length; i++)
        {
            if (argv[i].equals("-p"))
            {
		param = argv[++i];
		try
		{
                    if (param != null)
		    {
			port = Integer.valueOf(param).intValue();
                    }
                    else
                    {
			usage();
                    }
		}
		catch(NumberFormatException e)
		{
		    System.err.println("Port number must be an integer "
                                       + "less than 65535");
		    System.exit(-1);
		}
            }      
            else if (argv[i].equals("-s"))         
	    {
		server = argv[++i];
	    }
            else if (argv[i].equals("-f"))         
	    {
		fname = argv[++i];
	    }
	    else
	    {
                usage();
            }
	}
	
	LookupClient client = new LookupClient(server, port);
	client.readData(fname);

    }
}
