/******************************************************************************
**
**     File:                    LookupServer.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept 5th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

package caida.GTrace;

import java.net.ServerSocket;
import java.io.File;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.RandomAccessFile;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Date;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.Timer;

public class LookupServer
{ 
    ServerSocket serverSocket = null;
    public static Cache cache;

    public static String cacheFile;
    private static String LOGDIR = new String("log");
    private String serverConnectionLog = null;
    public static String serverDataLog = null;
    private String serverErrorLog = null;

    private FileOutputStream outConnection;
    private FileOutputStream outData;
    private FileOutputStream outError;

    public static Hashtable dbtxts = new Hashtable();
    public static Hashtable dpfs = new Hashtable();

    private static ResourceBundle gtraceResource;    
    public static char separator = File.separatorChar;
    private String HOMEDIR;
    private String DPFSDIR;
    private static boolean skitter = false;

    public static String DbDir;
    public static String AIRPORT;
    public static String CITIES;

    //Country center latlons
    public static Hashtable country_latlon = new Hashtable();

    //Number of cache entries
    private final int CACHESIZE = 250;

    private static String fileLine = 
	new String("--------------------------------------------------------------------------------\n");

    private Timer timer;
    Date date = new Date();
    private int connectionCount = 0;


//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 18th, 1999
**
******************************************************************************/

    public LookupServer(int port)
    {
	boolean listening = true;

	initServer();
	
	
	try 
	{
	    serverSocket = new ServerSocket(port);
	    System.out.println("Starting LookupServer on port : " + port);
	} 
	catch (IOException e) 
	{
	    System.err.println("Could not listen on port: " + port);
	    System.exit(-1);
	}
	
	while (listening)
	{
	    try
	    {
		new LookupServerThread(serverSocket.accept(), skitter).start();
		if (outConnection != null)
		{
		    date = new Date();
		    outConnection.write(new String("Connection " + (++connectionCount) + " at " + date.toString() + "\n").getBytes());
		}
	    }
	    catch(IOException ioe) 
	    {
		System.err.println("Could not accept connection on port: " + port);
		System.exit(-1);
	    }
	}
	
	try
	{
	    serverSocket.close();
	}
	catch(IOException ioe) 
	{
	    System.err.println("Could not close socket");
	}
    }



/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**
**     Parameters:              NONE
**   
**     Return Values:           void
**   
**     Author/Last Modified:    Ram Periakaruppan, August 26th, 1999 
**   
******************************************************************************/

    private static void usage()
    {
        System.err.println("usage: LookupServer [-p portnumber]\n");
        System.exit(1);
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**
**     Parameters:              NONE
**   
**     Return Values:           void
**   
**     Author/Last Modified:    Ram Periakaruppan, August 26th, 1999 
**   
******************************************************************************/

    public static void main(String[] argv) throws IOException 
    {
        int port = 55555;
	String param = new String();

        for (int i = 0; i < argv.length; i++)
        {
            if (argv[i].equals("-p"))
            {
		param = argv[++i];
		try
		{
                    if (param != null)
		    {
			port = Integer.valueOf(param).intValue();
                    }
                    else
                    {
			usage();
                    }
		}
		catch(NumberFormatException e)
		{
		    System.err.println("Port number must be an integer less than 65535");
		    System.exit(-1);
		}
            }      
            else if (argv[i].equals("-s"))         
	    {
		skitter = true;
	    }
	    else
	    {
                usage();
            }
        }
    
	new LookupServer(port);
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 18th, 1999
**
******************************************************************************/

    public void initServer()
    {

/*---------------------------------------------------------------------------
 *  Check for the GTrace configuration file which is required for the 
 *  successful initialization of server, exit if not found
 *---------------------------------------------------------------------------*/
	
	try 
	{
            gtraceResource = ResourceBundle.getBundle("GTrace");
        } 
	catch (MissingResourceException mre) 
	{
            System.err.println("GTrace.properties not found");
            System.exit(0);
        }

/*---------------------------------------------------------------------------
 *  Initialize all directories and filenames
 *---------------------------------------------------------------------------*/

	HOMEDIR = gtraceResource.getString("HOMEDIR");
	DbDir = new String(HOMEDIR + gtraceResource.getString("DATABASES") + separator);
	cacheFile = new String(HOMEDIR + separator + LOGDIR + separator + "GTraceServerCache");
	serverConnectionLog = new String(HOMEDIR + separator + LOGDIR + separator + "GTraceServerConnec");
	serverDataLog = new String(HOMEDIR + separator + LOGDIR + separator + "GTraceServerData");
	serverErrorLog = new String(HOMEDIR + separator + LOGDIR + separator + "GTraceServerError");

/*---------------------------------------------------------------------------
 *  Open error log and write today's date
 *---------------------------------------------------------------------------*/

	System.out.println("Date = " + date.toString());
	try
	{
	    outError = new FileOutputStream(serverErrorLog,true);
	    outError.write(fileLine.getBytes());
	    outError.write(new String("Error Log starting date : " + date.toString() + "\n").getBytes());
	    outError.write(fileLine.getBytes());
	    outError.flush();
	}
	catch(IOException ioe)
	{
	    System.err.println("LookupServer : Could not open " + serverErrorLog);
	    System.err.println("Error messages will not be archived");
	}


/*---------------------------------------------------------------------------
 *  Open connection log and write today's date
 *---------------------------------------------------------------------------*/


	try
	{
	    outConnection = new FileOutputStream(serverConnectionLog,true);
	    outConnection.write(fileLine.getBytes());
	    outConnection.write(new String("Connection Log starting date : " + date.toString() + "\n").getBytes());
	    outConnection.write(fileLine.getBytes());
	    outConnection.flush();
	}
	catch(IOException ioe)
	{
	    System.err.println("LookupServer : Could not open " + serverConnectionLog);
	    System.err.println("Connections will not be archived");
	}

/*---------------------------------------------------------------------------
 *  Open data log and write today's date
 *---------------------------------------------------------------------------*/

	try
	{
	    outData = new FileOutputStream(serverDataLog,true);
	    outData.write(fileLine.getBytes());
	    outData.write(new String("Data Log starting date : " + date.toString() + "\n").getBytes());
	    outData.write(fileLine.getBytes());
	    outData.flush();
	    outData.close();
	}
	catch(IOException ioe)
	{
	    System.err.println("LookupServer : Could not open " + serverDataLog);
	    System.err.println("Data requests will not be archived");
	}

/*---------------------------------------------------------------------------
 *  Check if there is an existing cache object from a previous invocation of
 *  the server and initialize it to the stored object. If one does not exist
 *  create a new cache of CACHESIZE size.
 *---------------------------------------------------------------------------*/

	try
	{
	    FileInputStream in = new FileInputStream(cacheFile);
	    ObjectInputStream s = new ObjectInputStream(in);
	    cache = (Cache)s.readObject();
	    cache.cache = (Hashtable)s.readObject();
	}
	catch(Exception e) 
	{ 
	    cache = new Cache(CACHESIZE);
	}

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/

	DPFSDIR = new String(gtraceResource.getString("DOMAINFILES"));
	File f = new File(HOMEDIR + DPFSDIR);
	
	String files[] = f.list();

	for (int i=0; i<files.length; i++)
	{
	    dpfs.put(files[i],new DomainParsingFile("LookupServer",HOMEDIR+DPFSDIR+separator+files[i]));
	}

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/

	AIRPORT = gtraceResource.getString("AIRPORTS");
	CITIES = gtraceResource.getString("CITIES");
	dbtxts.put(CITIES, new DbTxt(DbDir, gtraceResource.getString("CITIES")));
	dbtxts.put(AIRPORT, new DbTxt(DbDir, AIRPORT));

	String dbs[] = Util.tokenizeToString(gtraceResource.getString("DBS"),";");
	parseDbTxts(dbs);
	String txts[] = Util.tokenizeToString(gtraceResource.getString("TEXTFILES"),";");
	parseDbTxts(txts);

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/

	try
	{
	    String line;
	    String linetokens[] = new String[3];
	    RandomAccessFile fptr = new RandomAccessFile(DbDir+"country.txt","r");
	    while((line = fptr.readLine()) != null)
	    {
		linetokens = Util.tokenizeToString(line);
		country_latlon.put(linetokens[0],new LatLon(linetokens[1],linetokens[2]));
	    }
	}
	catch(Exception e)
	{

	}

/*---------------------------------------------------------------------------
 *  Initialize the timer that will write the contents of the cache to the disk
 *  every five minutes.
 *---------------------------------------------------------------------------*/

	timer = new Timer(300000,new cacheTimer());
	timer.start();
    }

/******************************************************************************
**
**     Method name:             parseDbTxts
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
**
******************************************************************************/

    private void parseDbTxts(String dbs[])
    {
	String item[] = new String[2];

	for (int i=0; i<dbs.length; i++)
	{
	    item = Util.tokenizeToString(dbs[i],",");
	    
	    if (item[1].compareTo("hostnames/ipaddr") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], DbTxt.HOSTNAMEIPADDR));
	    }
	    else if (item[1].compareTo("hostnames") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], DbTxt.HOSTNAME));
	    }
	    else  if (item[1].compareTo("ipaddr") == 0)
	    {
		dbtxts.put(item[0], new DbTxt(DbDir, item[0], DbTxt.IPADDR));
	    }
	}
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
**
******************************************************************************/

    public static LatLon getCountryLatLon(String countryCode)
    {
	return (LatLon)country_latlon.get(countryCode.trim().toLowerCase());  
    }


/******************************************************************************
**
**     Purpose:                 InnerClass 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 21th, 1999
**
******************************************************************************/


    class cacheTimer implements ActionListener
    {
	public void actionPerformed(ActionEvent  e)
	{
	    try
	    {
		FileOutputStream out = new FileOutputStream(cacheFile);
		ObjectOutputStream s = new ObjectOutputStream(out);
		s.writeObject(cache);
		s.writeObject(cache.cache);
		s.flush();
		s.close();
		out.close();
	    }
	    catch(Exception ep) 
	    { 
		System.out.println("Did not write Cache");
	    }
	}
    }
}
