/******************************************************************************
**
**     File:                    LookupServerThread.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept 5th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/

package caida.GTrace;

import java.net.Socket;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.io.FileOutputStream;
import com.sleepycat.db.*;
import java.util.Enumeration;
import java.util.Hashtable;

public class LookupServerThread extends Thread 
{
    private Socket socket = null;
    private boolean skitter = false;
    
    public static final Integer STEP1 = new Integer(1);
    public static final Integer STEP2 = new Integer(2);
    public static final Integer STEP3 = new Integer(3);
    public static final Integer STEP4 = new Integer(4);
    public static final Integer STEP5 = new Integer(5);
    public static final Integer STEP6 = new Integer(6);
    public static final Integer STEP7 = new Integer(7);

    FileOutputStream outData = null;

    NetGeoClient netgeo = new NetGeoClient("GTLS");

    public LookupServerThread(Socket socket, boolean skitter) 
    {
        super("LookupServerThread");
        this.socket = socket;
	this.skitter = skitter;
	try
	{
	    outData = new FileOutputStream(LookupServer.serverDataLog,true);
	}
	catch(IOException ioe) { } 
    }

/******************************************************************************
**
**     Method name:             run
**
**     Purpose:                 
**
**     Parameters:              NONE
**   
**     Return Values:           void
**   
**     Author/Last Modified:    Ram Periakaruppan, August 26th, 1999 
**   
******************************************************************************/

    public void run() 
    {
	boolean moreData = true;

        try 
	{
            PrintWriter out = new PrintWriter(socket.getOutputStream(), true);
            BufferedReader in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	    
            String inputLine = new String();
	    String outputLine = new String();

	    while (moreData)
	    {
		inputLine = in.readLine(); 
		if (inputLine == null || inputLine.compareTo("Done") == 0)
		{
		    break;
		}
                outputLine = findLocation(inputLine);
		out.println(outputLine);
		writeDataLog(inputLine + " --- " + outputLine + "\n");
	    }
	    
            out.close();
            in.close();
	    outData.close();
            socket.close();
        } 
	catch (IOException e) 
	{
            e.printStackTrace();
        }
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 21th, 1999
**
******************************************************************************/

    public void writeDataLog(String data)
    {
/*---------------------------------------------------------------------------
 *  Open data log and write today's date
 *---------------------------------------------------------------------------*/
	try
	{
	    outData.write(data.getBytes());
	    outData.flush();
	}
	catch(IOException ioe) { }
    }


/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 18th, 1999
**
******************************************************************************/

    private String locToString(float lat, float lon, Integer step)
    {
	String locString = new String();
	locString = String.valueOf(lat) + ":" +
	            String.valueOf(lon) + ":" +
	            step.toString();
	return locString;

    }

/******************************************************************************
**
**     Method name:             findLocation  
**
**     Purpose:
**
**     Parameters:              String inputLine
**   
**     Return Values:           String
**   
**     Author/Last Modified:    Ram Periakaruppan, August 26th, 1999 
**   
******************************************************************************/

    public String findLocation(String inputLine)
    {
	try
	{
	    String inputTokens[] = Util.tokenizeToString(inputLine,":");

	    if (inputTokens.length >= 2)
	    {
		String ipaddr = Util.ipaddrToString(inputTokens[0]);
		String hostname = inputTokens[1].toLowerCase();

/*---------------------------------------------------------------------------
 *  Check if the IP address fall in the ranges 
 *
 *     10.0.0.0 - 10.255.255.255
 *     172.16.0.0.- 172.31.255.255
 *     192.168.0.0 - 192.168.255.255
 *
 *  These ranges are reserved for private internet use
 *---------------------------------------------------------------------------*/
		
		try
		{
		    /* check to make sure that all versions of traceroutes ouput the ip address in brackets 
		       otherwise test first character to check if its a number and proceed
		    */
		    int octets[] = Util.tokenizeToint(ipaddr.substring(1, ipaddr.length()-1),".");

		    if (octets[0] == 10
			|| (octets[0] == 172 && octets[1] >= 16 && octets[1] <= 31) 
			|| (octets[0] == 192 && octets[1] == 168))
			{
			    return "Private Internet Use";
			}
		}
		catch(NumberFormatException nfe) {  /* should never happen */  }

/*---------------------------------------------------------------------------
 *  Step 1 - Check if the location for the IP Address exists in the cache
 *---------------------------------------------------------------------------*/
		
		LocationDetails cacheEntry = LookupServer.cache.lookup(ipaddr);
		if (cacheEntry != null)
		{
		    return locToString(cacheEntry.getLat(), cacheEntry.getLon(), cacheEntry.getSearchStep());
		} 
		
/*---------------------------------------------------------------------------
 *  Step 2 - Check for a LOC Record
 *---------------------------------------------------------------------------*/
	
		/*
	        String lookupName;
		String hstTokens[] = Util.tokenizeToString(hostname,".");
		if (hstTokens.length > 1)
	        {
		    String hostnames[];
		    String lastDomain = Util.getDomain(hostname, 1);
		    int end=0;
		    
		    if (lastDomain != null && lastDomain.length() == 2)
		    {
			end = hstTokens.length-2;
			hostnames = new String[end];
		    }
		    else
		    {
			end = hstTokens.length-1;
			hostnames = new String[end];
		    }

		    for(int i=0 ; i<end ; i++)
		    {
			lookupName = "";
			for (int j=i; j<hstTokens.length; j++)
			{
			    if (j != hstTokens.length-1)
			    {
				lookupName = lookupName + hstTokens[j].toLowerCase() + ".";
			    }
			    else
			    {
				lookupName = lookupName + hstTokens[j].toLowerCase();
			    }
			}
			hostnames[i] = lookupName;
		    }

		    Process p;
		    BufferedReader inStream;

		    for (int i=0; i<hostnames.length; i++)
		    {
			try
			{    
			    p = Runtime.getRuntime().exec(LookupServer.getGTraceResource("NSLOOKUP") + " -type=LOC " + hostnames[i]);
			    try
			    {
				p.waitFor();
			    }
			    catch(InterruptedException ie) { continue; }
			    
			    inStream = new BufferedReader(new InputStreamReader(p.getInputStream()));
			    
			    String str; 	
			    while( (str=inStream.readLine()) != null)
			    {
				if (str.endsWith("record type LOC, interpreted as:"))
				{ 
				    str=inStream.readLine();
				    int index = str.indexOf("LOC");
				    if (index > 0)
				    {
					str = str.substring(index+3).trim();
					String[] loctokens = Util.tokenizeToString(str);
					
					model.addCurrentSearchStep(hNum, STEP2, tg.getName());
					float lat = Util.degminsecTofloat(loctokens[0],loctokens[1],
									  loctokens[2],loctokens[3]);
					float lon = Util.degminsecTofloat(loctokens[4],loctokens[5],
								  loctokens[6],loctokens[7]);
					model.addLocation(Util.latlonToString(lat,lon),
							  hNum, STEP2, tg.getName());
					model.addLOCFound(hNum, tg.getName());
					LookupServer.cache.add(ipaddr,new LocationDetails(lat,lon,STEP2));
					return;
				    }
				}
			    }
			}
			catch(IOException ioe)
			{
			    //System.out.println("Caused an IO exception:" + ioe.getMessage());
			}
		    }
		}
		*/

/*---------------------------------------------------------------------------
 *  Step 3 - Search for a complete match in the databases and files 
 *           present in the GTrace Configuration File.
 *---------------------------------------------------------------------------*/
		/*
	        DbRecord dbRecord = null;
		for (Enumeration e = LookupServer.dbtxts.keys() ; e.hasMoreElements() ;) 
		{
		    String dbtxtname = (String)e.nextElement();
		    if (dbtxtname.compareTo(LookupServer.CITIES) != 0 && dbtxtname.compareTo(LookupServer.AIRPORT) != 0)
		    {
			try
			{
			    dbRecord = ((DbTxt)LookupServer.dbtxts.get(dbtxtname)).lookup(hostname,ipaddr);
			    if (dbRecord != null)
			    {
				break;
			    }
			}
			catch(DbException dbe) { }
		    }
		}
		
		if (dbRecord != null)
		{
		    String dbFields[] = Util.tokenizeToString(dbRecord);
		    LookupServer.cache.add(ipaddr,
					   new LocationDetails(
							 Util.stringTofloat(dbFields[0]),
							 Util.stringTofloat(dbFields[1]),
							 STEP3)
					   );
		    return locToString(Util.stringTofloat(dbFields[0]),
				       Util.stringTofloat(dbFields[1]), 
				       STEP3);
		 
		}
		*/
/*---------------------------------------------------------------------------
 *  Step 4 - Check for a Domain Parsing File
 *---------------------------------------------------------------------------*/
		/*
		String firstDomain = Util.getDomain(hostname,1);
		String secondDomain = Util.getDomain(hostname,2);
		DomainParsingFile dpf;
		
		if (firstDomain != null && secondDomain != null)
		{
		    dpf = (DomainParsingFile)LookupServer.dpfs.get(
						   secondDomain + firstDomain);

		    if (dpf != null)
		    {
			dbRecord = dpf.lookup(hostname);
			
			if (dbRecord != null)
			{
			    if (loc.startsWith("L:") || loc.startsWith("l:"))
			    {
				//xxx fill in this
			    }
			    else if (dbRecord.startsWith("country:"))
			    {
				LatLon country = LookupServer.getCountryLatLon(
				       dbRecord.substring(dbRecord.indexOf(":")+1));
				if (country != null)
				{
				    LookupServer.cache.add(ipaddr,
							   new LocationDetails(
								  country.getLat(), 
								  country.getLon(),
								  STEP4)
							   );
				    return locToString(country.getLat(), 
						       country.getLon(), 
						       STEP4);
				}
			    }
			    else
			    {
				String dbFields[] = Util.tokenizeToString(dbRecord);
				LookupServer.cache.add(ipaddr,
						       new LocationDetails(
							   Util.stringTofloat(dbFields[0]),
							   Util.stringTofloat(dbFields[1]),
							   STEP4)
						       );
				return locToString(Util.stringTofloat(dbFields[0]),
						   Util.stringTofloat(dbFields[1]), 
						   STEP4);
			    }
			}
		    }
		}

		*/
/*---------------------------------------------------------------------------
 *  Step 5 - Repeat step 3 with reduction technique
 *---------------------------------------------------------------------------*/
		/*
		if (!Character.isDigit(hostname.charAt(hostname.length()-1)))
		{
		    for (Enumeration e = LookupServer.dbtxts.keys() ; 
			 e.hasMoreElements() ;) 
		    {
			String dbtxtname = (String)e.nextElement();
			if (dbtxtname.compareTo("CITIES") != 0 
			    && dbtxtname.compareTo("AIRPORTS") != 0)
			{
			    try
			    {
				dbRecord = ((DbTxt)LookupServer.dbtxts.get(
						   dbtxtname)).rlookup(hostname);
				if (dbRecord != null)
				{
				    break;
				}
			    }
			    catch(DbException dbe) { }
			}
		    }
		    
      
		    if (dbRecord != null)
		    {
			String dbFields[] = Util.tokenizeToString(dbRecord);
			LookupServer.cache.add(ipaddr,
					       new LocationDetails(
							     Util.stringTofloat(dbFields[0]),
							     Util.stringTofloat(dbFields[1]),
							     STEP5)
				       );
			return locToString(Util.stringTofloat(dbFields[0]),
					   Util.stringTofloat(dbFields[1]), 
					   STEP5);
		    }
		}
		*/
/*---------------------------------------------------------------------------
 *  Step 6 - Query NetGeo server with whois server with IP Address
 *---------------------------------------------------------------------------*/
		/*
		if (ipaddr != null)
		{
		    if (!hostname.endsWith(".net") && !hostname.endsWith(".NET"))
		    {
			Hashtable latLongHash = netgeo.getLatLong(ipaddr);
			
			float lat = NetGeoClient.getLat( latLongHash );
			float lon = NetGeoClient.getLong( latLongHash );
			
			LookupServer.cache.add(ipaddr,new LocationDetails(lat,lon,STEP6));
			return locToString(lat, lon, STEP6);
		    }
		}
		*/
/*---------------------------------------------------------------------------
 *  Step 7 - Check if last two characters end in a country code
 *---------------------------------------------------------------------------*/
		/*
		String lastDomain = Util.getDomain(hostname, 1);
		if (lastDomain != null)
		{
		    LatLon country = LookupServer.getCountryLatLon(lastDomain);
		    if (country != null)
		    {
			LookupServer.cache.add(ipaddr,
					       new LocationDetails(
							     country.getLat(), 
							     country.getLon(),
							     STEP7)
					       );
			return locToString(country.getLat(), 
					   country.getLon(), 
					   STEP7);
		    }
		}
		*/
	    }
	    return null;
	}
	catch(NullPointerException npe)
	{
	    return null;
	}
    }
}
