/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    MapAdd.java
**
**     Purpose:                 Constructs and displays the addition of maps
**                              dialog that modifies and adds a map of to the
**                              GTraceMaps.properties configuration file.
**
**     Author/Date:             Ram Periakaruppan, Sept. 13th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;


public class MapAdd extends JDialog implements ActionListener
{
    private JDialog mapAddDialog;   // the dialog 
    JFrame frame;                   /* the parent frame on which the dialog is
                                       to be displayed
                                    */

    // Labels
    JLabel fileNameLabel;           // label asking for name of map
    JLabel tlhcLatLabel;            // top left hand corner latitude label
    JLabel brhcLatLabel;            // bottom right hand corner latitude label
    JLabel tlhcLonLabel;            // top left hand corner longitude label
    JLabel brhcLonLabel;            /* bottom right hand corner longitude
                                       label
                                    */
    JLabel warningLabel1;           // first line of warning message
    JLabel warningLabel2;           // second line of warning message
    JLabel fgLabel;                 // foreground label
    JLabel bgLabel;                 // background label
    JLabel parentLabel;             // parent label
    JLabel projLabel;               // projection label
    JLabel childrenLabel;           // children label

    JLabel projConstantsLabel;      // projection constants label
    JLabel scaleLabel;              // scale label
    JLabel stndprllLabel;           // standard parallel label
    JLabel originLatLonLabel;       // origin lat/long label
    JLabel falseENLabel;            // false east/north label


    // TextFields
    JTextField fileNameTextField;   // the name of the map
    JTextField tlhcLatTextField;    // top left hand corner latitude value
    JTextField brhcLatTextField;    // bottom right hand corner latitude value
    JTextField tlhcLonTextField;    // top left hand corner longitude value
    JTextField brhcLonTextField;    /* bottom right hand corner longitude
                                       value
                                    */

    // Alber and Lambert Constants
    JTextField scaleTextField;         // the radius of map
    JTextField stndprll1TextField;     // standard parallel 1
    JTextField stndprll2TextField;     // standard parallel 2
    JTextField originLatTextField;     // point of latitude origin
    JTextField originLonTextField;     // point of longitude origin
    JTextField falseEastingTextField;  // false easting
    JTextField falseNorthingTextField; // false northing

    // Buttons
    JButton chooseFileButton;     // choose file button
    JButton cancelButton;         // cancel button
    JButton addButton;            // add button
    JButton addLabelsButton;      // add labels button

    // Combo boxes
    JComboBox fgCombo;        // foreground combo
    JComboBox bgCombo;        // background combo
    JComboBox parentCombo;    // parent combo
    JComboBox projCombo;      // projection combo

    // Text areas 
    JTextArea childrenTArea;  // children Textarea

    // The width and height of the dialog
    public static int WIDTH = 580;
    public static int HEIGHT = 500;
    
    // Constants
    private static final String fileNameString = "Filename";
    private static final String tlhcString = "Top left corner";
    private static final String brhcString = "Bottom right corner";
    private static final String fgString = "Foreground";
    private static final String bgString = "Background";
    private static final String parentString = "Parent";
    private static final String projString = "Projection";
    private static final String childrenString  = "Children";
    private static final String projConstantsString = "Projection Constants";
    private static final String scaleString = "Scale R";
    private static final String stndprllString = "Standard Parallels";
    private static final String stndprll1String = "Standard Parallel 1";
    private static final String stndprll2String = "Standard Parallel 2";
    private static final String originLatString = "Origin Latitude";
    private static final String originLonString = "Origin Longitude";
    private static final String originLatLonString = "Origin Lat/Lon";
    private static final String falseENString = "False E/N";
    private static final String falseEastingString = "False Easting";
    private static final String falseNorthingString = "False Northing";

    private static final String[] colors = {"white",
					    "red",
					    "blue",
					    "cyan",
					    "darkGray",
					    "gray",
					    "green",
					    "lightGray",
					    "magenta",
					    "orange",
					    "pink",
					    "black",
					    "yellow"
    };

    private final int WIDTH15 = 15;
    private final int WIDTH5 = 5;
    private final int WIDTH3 = 3;
    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Purpose:                 Constructs the map add dialog and adds the 
 **                              components to the dialog
 **    
 **     Parameters:              JFrame f - the parent frame on which the
 **                                         dialog is displayed on.
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 10th, 1999
 **
 *****************************************************************************/
    
    public MapAdd(JFrame f) 
    {
        this.frame = f;

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
 	mapAddDialog = new JDialog(frame, "Map Addition", false);
	mapAddDialog.setSize(WIDTH, HEIGHT);
	mapAddDialog.setResizable(false);

/*---------------------------------------------------------------------------
 *  Assign the string constants to the labels
 *---------------------------------------------------------------------------*/

	fileNameLabel = new JLabel(fileNameString);
	tlhcLatLabel = new JLabel(tlhcString);
	brhcLatLabel = new JLabel(brhcString);
	tlhcLonLabel = new JLabel(tlhcString);
	brhcLonLabel = new JLabel(brhcString);
	fgLabel = new JLabel(fgString);
	bgLabel = new JLabel(bgString);
	parentLabel = new JLabel(parentString);
	projLabel = new JLabel(projString);
	childrenLabel = new JLabel(childrenString);

	projConstantsLabel = new JLabel(projConstantsString); 
	scaleLabel = new JLabel(scaleString);
	stndprllLabel = new JLabel(stndprllString);
	originLatLonLabel = new JLabel(originLatLonString);
	falseENLabel = new JLabel(falseENString);

	warningLabel1 = new JLabel("* File must be located in `maps' " +
                                   "subdirectory and only gif and jpg");
	warningLabel2 = new JLabel("formats are supported.");
	warningLabel1.setBorder(BorderFactory.createEmptyBorder(10,0,0,0));

/*---------------------------------------------------------------------------
 *  Initialize the textfields to their width and set their names to the
 *  string constant describing the text field
 *---------------------------------------------------------------------------*/
        
	fileNameTextField = new JTextField(WIDTH15);
        fileNameTextField.setActionCommand(fileNameString);
	tlhcLatTextField = new JTextField(WIDTH5);
	tlhcLatTextField.setActionCommand(tlhcString);
	brhcLatTextField = new JTextField(WIDTH5);
	brhcLatTextField.setActionCommand(brhcString);
	tlhcLonTextField = new JTextField(WIDTH5);
	tlhcLonTextField.setActionCommand(tlhcString);
	brhcLonTextField = new JTextField(WIDTH5);
	brhcLonTextField.setActionCommand(brhcString);

	scaleTextField = new JTextField(WIDTH3);
	scaleTextField.setActionCommand(scaleString);
	scaleTextField.setEnabled(false);
	stndprll1TextField = new JTextField(WIDTH3);
	stndprll1TextField.setActionCommand(stndprll1String);
	stndprll1TextField.setEnabled(false);
	stndprll2TextField = new JTextField(WIDTH3);
	stndprll2TextField.setActionCommand(stndprll2String);
	stndprll2TextField.setEnabled(false);
	originLatTextField = new JTextField(WIDTH3);
	originLatTextField.setActionCommand(originLatString);
	originLatTextField.setEnabled(false);
	originLonTextField = new JTextField(WIDTH3);
	originLonTextField.setActionCommand(originLonString);
	originLonTextField.setEnabled(false);
	falseEastingTextField = new JTextField(WIDTH3);
	falseEastingTextField.setActionCommand(falseEastingString);
	falseEastingTextField.setEnabled(false);
	falseNorthingTextField = new JTextField(WIDTH3);    
	falseNorthingTextField.setActionCommand(falseNorthingString);
	falseNorthingTextField.setEnabled(false);
	

/*---------------------------------------------------------------------------
 *  Initialize the foreground combobox with the colors stored in the
 *  colors[] constant array.
 *---------------------------------------------------------------------------*/
        
	fgCombo = new JComboBox();
	fgCombo.setEditable(false);

	for (int i=0; i< colors.length; i++)
	{
	    fgCombo.addItem(colors[i]);
	}

/*---------------------------------------------------------------------------
 *  Initialize the background combobox with the colors stored in the
 *  colors[] constant array.
 *---------------------------------------------------------------------------*/
        
	bgCombo = new JComboBox();
	bgCombo.setEditable(false);

	for (int i=0; i< colors.length; i++)
	{
	    bgCombo.addItem(colors[i]);
	}
	bgCombo.setSelectedItem(colors[5]);

/*---------------------------------------------------------------------------
 *  Initialize the parent combobox with the maps currently present in the 
 *  application
 *---------------------------------------------------------------------------*/
        
	parentCombo = new JComboBox();
	parentCombo.setEditable(false);
	parentCombo.setPreferredSize(new Dimension(140,25));
	parentCombo.setMaximumSize(new Dimension(140,25));

	String maps[] = GTrace.getSortedMapKeys();
	for (int i=0; i < maps.length; i++)
	{
	    parentCombo.addItem(maps[i]);
	}


/*---------------------------------------------------------------------------
 *  Initialize the projection combo box with the projections supported by
 *  GTrace, and add a action listener that will activate the alber/lambert
 *  constant text boxes if that value is chosen.
 *---------------------------------------------------------------------------*/
        
	projCombo = new JComboBox();
	projCombo.setEditable(false);
	projCombo.setPreferredSize(new Dimension(140,25));
	projCombo.setMaximumSize(new Dimension(140,25));
	projCombo.addItem("Linear/Mercator");
	projCombo.addItem("Alber");
	projCombo.addItem("Lambert");
	projCombo.addItem("XY-Coordinates");
	projCombo.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    String item = (String)projCombo.getSelectedItem();
                    if (item.compareTo("Alber's") == 0
                        || item.compareTo("Lambert") == 0)
                    {
                        scaleTextField.setEnabled(true);
                        stndprll1TextField.setEnabled(true);
                        stndprll2TextField.setEnabled(true);
                        originLatTextField.setEnabled(true);
                        originLonTextField.setEnabled(true);
                        falseEastingTextField.setEnabled(true);
                        falseNorthingTextField.setEnabled(true);
                    }
                    else
                    {
                        scaleTextField.setEnabled(false);
                        stndprll1TextField.setEnabled(false);
                        stndprll2TextField.setEnabled(false);
                        originLatTextField.setEnabled(false);
                        originLonTextField.setEnabled(false);
                        falseEastingTextField.setEnabled(false);
                        falseNorthingTextField.setEnabled(false);
                    }
                }
	});


/*---------------------------------------------------------------------------
 *  create the choose file button and add a action listener that will popup
 *  the open file dialog setting the file filters to jpg and gif files.
 *---------------------------------------------------------------------------*/
        
	chooseFileButton = new JButton("Choose File...");
	chooseFileButton.setMnemonic('F');
	chooseFileButton.setBorder(BorderFactory.createRaisedBevelBorder());
	chooseFileButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	chooseFileButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    JFileChooser chooser = new JFileChooser(GTrace.HOMEDIR
                                                            + GTrace.separator
                                                            + "maps");
                    
                    chooser.setMultiSelectionEnabled(false);
                    chooser.addChoosableFileFilter(
                              new GTraceFileFilter(
                              new String[] {"jpg"}, "(*.jpg)"));
                    chooser.addChoosableFileFilter(
                              new GTraceFileFilter(
                              new String[] {"gif"}, "(*.gif)"));
	    
                    int option = chooser.showOpenDialog(frame);
                    if (option == JFileChooser.APPROVE_OPTION)
                    {
                        if (chooser.getSelectedFile() != null)
                        {
                            String OPENDIR =
                               chooser.getCurrentDirectory().getAbsolutePath();
                            String fname =
                               chooser.getCurrentDirectory().getAbsolutePath()
                               + System.getProperty("file.separator") 
                               + chooser.getSelectedFile().getName();
                            fileNameTextField.setText(
                                        chooser.getSelectedFile().getName() ); 
                        }
                    }
                }
	});

/*---------------------------------------------------------------------------
 *  create the cancel button and add a action listener that will dispose the
 *  dialog if clicked on.
 *---------------------------------------------------------------------------*/

	cancelButton = new JButton("Cancel");
	cancelButton.setMnemonic('C');
	cancelButton.setBorder(BorderFactory.createRaisedBevelBorder());
	cancelButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	cancelButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    mapAddDialog.setVisible(false);
                    mapAddDialog.dispose();
                }
	});

/*---------------------------------------------------------------------------
 *  create the add labels button and add a action listener that will popup
 *  an information dialog indicating that this feature is presentln not
 *  available.
 *---------------------------------------------------------------------------*/
        
	JButton addLabelsButton = new JButton("Add Labels");
	addLabelsButton.setMnemonic('L');
	addLabelsButton.setBorder(BorderFactory.createRaisedBevelBorder());
	addLabelsButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	addLabelsButton.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    JOptionPane.showMessageDialog(frame,
                              "This feature not available as of this release",
                              "Option unavailable",
                              JOptionPane.INFORMATION_MESSAGE);
                }
	});

/*---------------------------------------------------------------------------
 *  create the add button and set the actionlistener so that the current
 *  instances action performed method is called when clicked on
 *---------------------------------------------------------------------------*/
        
	addButton = new JButton("Add");
	addButton.setMnemonic('A');
	addButton.setBorder(BorderFactory.createRaisedBevelBorder());
	addButton.setAlignmentX(Component.CENTER_ALIGNMENT);    
	addButton.addActionListener(this);


/*---------------------------------------------------------------------------
 *  define the children text area and its attributes
 *---------------------------------------------------------------------------*/
        
        childrenTArea= new JTextArea();
	childrenTArea.setEditable(true);
        childrenTArea.setLineWrap(true);
        childrenTArea.setWrapStyleWord(true);
        JScrollPane childrenTAreaScrollPane = new JScrollPane(childrenTArea);
        childrenTAreaScrollPane.setVerticalScrollBarPolicy(
                                   JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        childrenTAreaScrollPane.setPreferredSize(new Dimension(250, 100));



	
	//Panels

/*---------------------------------------------------------------------------
 *  construct the latitude panel add the components to its two rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagLatitudePane = new GridBagLayout();
	GridBagConstraints clp = new GridBagConstraints();

	JPanel latitudePane = new JPanel();
        latitudePane.setBorder(BorderFactory.createCompoundBorder(
                                 BorderFactory.createTitledBorder(
                                                                  "Latitude"),
                                 BorderFactory.createEmptyBorder(3,3,3,3)));
        latitudePane.setLayout(gridbagLatitudePane);


	// Row 1	
	clp.gridx = 0;
	clp.gridy = 0;
	clp.anchor = GridBagConstraints.WEST;
	clp.insets = new Insets(0,0,0,5);  // top, bottom, left, rigth
	gridbagLatitudePane.setConstraints(tlhcLatLabel, clp);
	latitudePane.add(tlhcLatLabel);
	
	clp.gridx = 1;
	clp.insets = new Insets(0,0,0,0);  // top, bottom, left, rigth
	gridbagLatitudePane.setConstraints(tlhcLatTextField, clp);
	latitudePane.add(tlhcLatTextField);

	// Row 2	
	clp.gridx = 0;
	clp.gridy = 1;
	clp.anchor = GridBagConstraints.WEST;
	clp.insets = new Insets(2,0,0,5);  // top, bottom, left, rigth
	gridbagLatitudePane.setConstraints(brhcLatLabel, clp);
	latitudePane.add(brhcLatLabel);

	clp.gridx = 1;
	clp.insets = new Insets(2,0,0,0);  // top, bottom, left, rigth
	gridbagLatitudePane.setConstraints(brhcLatTextField, clp);
	latitudePane.add(brhcLatTextField);
	
/*---------------------------------------------------------------------------
 *  construct the longitude panel and add the components to its two rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagLongitudePane = new GridBagLayout();
	GridBagConstraints clnp = new GridBagConstraints();

	JPanel longitudePane = new JPanel();
        longitudePane.setBorder(BorderFactory.createCompoundBorder(
                                  BorderFactory.createTitledBorder(
                                                               "Longitude"),
                                  BorderFactory.createEmptyBorder(3,3,3,3)));

        longitudePane.setLayout(gridbagLongitudePane);


	// Row 1	
	clnp.gridx = 0;
	clnp.gridy = 0;
	clnp.anchor = GridBagConstraints.WEST;
	clnp.insets = new Insets(0,1,0,5);  // top, bottom, left, right
	gridbagLongitudePane.setConstraints(tlhcLonLabel, clnp);
	longitudePane.add(tlhcLonLabel);
	
	clnp.gridx = 1;
	clnp.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbagLongitudePane.setConstraints(tlhcLonTextField, clnp);
	longitudePane.add(tlhcLonTextField);

	// Row 2	
	clnp.gridx = 0;
	clnp.gridy = 1;
	clnp.anchor = GridBagConstraints.WEST;
	clnp.insets = new Insets(2,1,0,5);  // top, bottom, left, right
	gridbagLongitudePane.setConstraints(brhcLonLabel, clnp);
	longitudePane.add(brhcLonLabel);

	clnp.gridx = 1;
	clnp.insets = new Insets(2,0,0,0);  // top, bottom, left, right
	gridbagLongitudePane.setConstraints(brhcLonTextField, clnp);
	longitudePane.add(brhcLonTextField);


/*---------------------------------------------------------------------------
 *  construct the Parent Projection panel and add the components to its two
 *  rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagParProjPane = new GridBagLayout();
	GridBagConstraints clpp = new GridBagConstraints();

	JPanel ParProjPane = new JPanel();
        ParProjPane.setLayout(gridbagParProjPane);


	// Row 1	
	clpp.gridx = 0;
	clpp.gridy = 0;
	clpp.anchor = GridBagConstraints.WEST;
	clpp.insets = new Insets(10,0,0,10); // top, bottom, left, right
	gridbagParProjPane.setConstraints(parentLabel, clpp);
	ParProjPane.add(parentLabel);

	clpp.gridx = 1;
	clpp.ipadx = WIDTH15;  
	clpp.insets = new Insets(10,0,0,0);  // top, bottom, left, right
	gridbagParProjPane.setConstraints(parentCombo, clpp);
	ParProjPane.add(parentCombo);

	// Row 2	
	clpp.gridx = 0;
	clpp.gridy = 1;
	clpp.ipadx = 0;  
	clpp.anchor = GridBagConstraints.WEST;
	clpp.insets = new Insets(15,0,0,10);  // top, bottom, left, right
	gridbagParProjPane.setConstraints(projLabel, clpp);
	ParProjPane.add(projLabel);

	clpp.gridx = 1;
	clpp.ipadx = WIDTH15;  
	clpp.insets = new Insets(15,0,0,0);  // top, bottom, left, right
	gridbagParProjPane.setConstraints(projCombo, clpp);
	ParProjPane.add(projCombo);


/*---------------------------------------------------------------------------
 *  construct the Projection Details panel and add the components to its
 *  five rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagProjDtlPane = new GridBagLayout();
	GridBagConstraints cldtl = new GridBagConstraints();

	JPanel ProjDtlPane = new JPanel();
        ProjDtlPane.setLayout(gridbagProjDtlPane);

	// Row 1	
	cldtl.gridx = 0;
	cldtl.gridy = 0;
	cldtl.anchor = GridBagConstraints.WEST;
	cldtl.insets = new Insets(5,0,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(projConstantsLabel, cldtl);
	ProjDtlPane.add(projConstantsLabel);

	// Row 2	
	cldtl.gridx = 0;
	cldtl.gridy = 1;
	cldtl.anchor = GridBagConstraints.WEST;
	cldtl.insets = new Insets(10,0,0,5);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(scaleLabel, cldtl);
	ProjDtlPane.add(scaleLabel);

	cldtl.gridx = 1;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(10,0,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(scaleTextField, cldtl);
	ProjDtlPane.add(scaleTextField);

	// Row 3	
	cldtl.gridx = 0;
	cldtl.gridy = 2;
	cldtl.ipadx = 0;  
	cldtl.anchor = GridBagConstraints.WEST;
	cldtl.insets = new Insets(3,0,0,5);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(stndprllLabel, cldtl);
	ProjDtlPane.add(stndprllLabel);

	cldtl.gridx = 1;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(stndprll1TextField, cldtl);
	ProjDtlPane.add(stndprll1TextField);

	cldtl.gridx = 2;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,3,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(stndprll2TextField, cldtl);
	ProjDtlPane.add(stndprll2TextField);

	// Row 4	
	cldtl.gridx = 0;
	cldtl.gridy = 3;
	cldtl.ipadx = 0;  
	cldtl.anchor = GridBagConstraints.WEST;
	cldtl.insets = new Insets(3,0,0,5);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(originLatLonLabel, cldtl);
	ProjDtlPane.add(originLatLonLabel);
	
	cldtl.gridx = 1;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(originLatTextField, cldtl);
	ProjDtlPane.add(originLatTextField);

	cldtl.gridx = 2;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,3,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(originLonTextField, cldtl);
	ProjDtlPane.add(originLonTextField);
	
	// Row 5	
	cldtl.gridx = 0;
	cldtl.gridy = 4;
	cldtl.ipadx = 0;  
	cldtl.anchor = GridBagConstraints.WEST;
	cldtl.insets = new Insets(3,0,0,5);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(falseENLabel, cldtl);
	ProjDtlPane.add(falseENLabel);

	cldtl.gridx = 1;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(falseEastingTextField, cldtl);
	ProjDtlPane.add(falseEastingTextField);

	cldtl.gridx = 2;
	cldtl.ipadx = WIDTH15;  
	cldtl.insets = new Insets(0,3,0,0);  // top, bottom, left, right
	gridbagProjDtlPane.setConstraints(falseNorthingTextField, cldtl);
	ProjDtlPane.add(falseNorthingTextField);


/*---------------------------------------------------------------------------
 *  construct the Children panel and add the components to its two rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagChildrenPane = new GridBagLayout();
	GridBagConstraints clchld = new GridBagConstraints();

	JPanel childrenPane = new JPanel();
        childrenPane.setLayout(gridbagChildrenPane);


	// Row 1	
	clchld.gridx = 0;
	clchld.gridy = 0;
	clchld.anchor = GridBagConstraints.WEST;
	clchld.insets = new Insets(5,0,0,0);  // top, bottom, left, right
	gridbagChildrenPane.setConstraints(childrenLabel, clchld);
	childrenPane.add(childrenLabel);

	// Row 2	
	clchld.gridx = 0;
	clchld.gridy = 1;
	clchld.ipadx = 0;  
	clchld.anchor = GridBagConstraints.WEST;
	clchld.insets = new Insets(3,0,0,0);  // top, bottom, left, right
	gridbagChildrenPane.setConstraints(childrenTAreaScrollPane, clchld);
	childrenPane.add(childrenTAreaScrollPane);

/*---------------------------------------------------------------------------
 *  construct the Color panel and add the components to its two rows 
 *---------------------------------------------------------------------------*/

	GridBagLayout gridbagColorPane = new GridBagLayout();
	GridBagConstraints cl = new GridBagConstraints();

	JPanel colorPane = new JPanel();
        colorPane.setBorder(BorderFactory.createCompoundBorder(
                              BorderFactory.createTitledBorder(
                                                               "Color"),
                              BorderFactory.createEmptyBorder(2,2,2,2)));
        colorPane.setLayout(gridbagColorPane);

	// Row 1	
	cl.gridx = 0;
	cl.gridy = 0;
	cl.anchor = GridBagConstraints.WEST;
	cl.insets = new Insets(0,1,0,5);  // top, bottom, left, right
	gridbagColorPane.setConstraints(fgLabel, cl);
	colorPane.add(fgLabel);
	
	cl.gridx = 1;
	cl.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbagColorPane.setConstraints(fgCombo, cl);
	colorPane.add(fgCombo);

	// Row 2	
	cl.gridx = 0;
	cl.gridy = 1;
	cl.anchor = GridBagConstraints.WEST;
	cl.insets = new Insets(2,1,0,5);  // top, bottom, left, right
	gridbagColorPane.setConstraints(bgLabel, cl);
	colorPane.add(bgLabel);

	cl.gridx = 1;
	cl.insets = new Insets(2,0,0,0);  // top, bottom, left, right
	gridbagColorPane.setConstraints(bgCombo, cl);
	colorPane.add(bgCombo);

/*---------------------------------------------------------------------------
 *  construct the main add dialog panel and add the components/panels to its
 *  seven rows 
 *---------------------------------------------------------------------------*/
        
        JPanel addPane = new JPanel();
	GridBagLayout gridbag = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
        addPane.setLayout(gridbag);

	// ROW 1
	c.gridx = 0;
	c.gridy = 0;
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,20,0,10);  // top, bottom, left, right
	gridbag.setConstraints(fileNameLabel, c);
	addPane.add(fileNameLabel);

	c.gridx = 1;
	c.ipady = 8;  
	c.insets = new Insets(20,0,0,0);  // top, bottom, left, right
	gridbag.setConstraints(fileNameTextField, c);
	addPane.add(fileNameTextField);

	c.gridx = 2;
	c.ipadx = 20;  
	c.insets = new Insets(20,15,0,0);  // top, bottom, left, right
	c.anchor = GridBagConstraints.WEST;
	gridbag.setConstraints(chooseFileButton, c);
	addPane.add(chooseFileButton);

	// ROW 2
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridx = 0;
	c.gridy = 1;
	c.insets = new Insets(0,45,0,0);  // top, bottom, left, right
	gridbag.setConstraints(warningLabel1, c);
	addPane.add(warningLabel1);

	// ROW 3
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridx = 0;
	c.gridy = 2;
	c.anchor = GridBagConstraints.NORTHWEST;
	c.insets = new Insets(0,55,0,0);  // top, bottom, left, right
	gridbag.setConstraints(warningLabel2, c);
	addPane.add(warningLabel2);

	// ROW 4
	c.gridx = 0;
	c.gridy = 3;
	//c.gridwidth = 1;
	c.insets = new Insets(5,20,0,5);  // top, bottom, left, right
	c.anchor = GridBagConstraints.WEST;
	gridbag.setConstraints(latitudePane, c);
	addPane.add(latitudePane);
	
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridx = 2;
	c.insets = new Insets(5,0,0,0);  // top, bottom, left, right
	gridbag.setConstraints(longitudePane, c);
	addPane.add(longitudePane);
	
	// ROW 5
	c.gridx = 0;
	c.gridy = 4;
	c.insets = new Insets(5,10,0,5);  // top, bottom, left, right
	c.anchor = GridBagConstraints.WEST;
	gridbag.setConstraints(ParProjPane, c);
	addPane.add(ParProjPane);

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.gridx = 2;
	c.insets = new Insets(5,0,0,0);  // top, bottom, left, right
	gridbag.setConstraints(colorPane, c);
	addPane.add(colorPane);

	// Row 6	
	c.gridx = 0;
	c.gridy = 5;
	c.insets = new Insets(5,10,0,5);  // top, bottom, left, right
	c.anchor = GridBagConstraints.WEST;
	c.gridwidth = 0;
	gridbag.setConstraints(ProjDtlPane, c);
	addPane.add(ProjDtlPane);
	
	c.gridx = 2;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.insets = new Insets(0,0,0,0);  // top, bottom, left, right
	gridbag.setConstraints(childrenPane, c);
	addPane.add(childrenPane);
	
	
	// Row 7
	c.gridx = 1;
	c.gridy = 6;
	c.ipadx = 30;  
	c.anchor = GridBagConstraints.WEST;
	c.insets = new Insets(20,40,0,10);  // top, bottom, left, right
	gridbag.setConstraints(addButton, c);
	addPane.add(addButton);
	
	c.insets = new Insets(20,115,0,10);  // top, bottom, left, right
	gridbag.setConstraints(addLabelsButton, c);
	addPane.add(addLabelsButton);

	c.insets = new Insets(20,240,0,0);  // top, bottom, left, right
	gridbag.setConstraints(cancelButton, c);
	addPane.add(cancelButton);

/*---------------------------------------------------------------------------
 *  add the add panel to the content pane of the map dialog and set its
 *  postion relative to the parent frame and display the dialog.
 *---------------------------------------------------------------------------*/
        
	mapAddDialog.getContentPane().add(addPane);
	mapAddDialog.setLocationRelativeTo(frame);
	mapAddDialog.setVisible(true);
    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 10th, 1999
 **
 *****************************************************************************/
    
    public void actionPerformed(ActionEvent e)
    {
	boolean parentAdded = false;  /* flag indicating if parent was added
                                         to configuration file
                                      */
	String newMapConf = new String(); /* string that will hold the
                                             contents of modified config file
                                          */
	String str;          /* string holding contents of each line read
                                from file
                             */
	File mapFile = null; // file object 
	RandomAccessFile fptr = null; /* random access file pointer to file
                                         file object
                                      */

/*---------------------------------------------------------------------------
 *  make sure the bare minimum essential fields have been specified for a
 *  map to be added, if not display an error dialog.
 *---------------------------------------------------------------------------*/
        
	if (fileNameTextField.getText().length() > 0 &&
	    tlhcLonTextField.getText().length() > 0 &&
	    tlhcLatTextField.getText().length() > 0 &&
	    brhcLonTextField.getText().length() > 0 &&
	    brhcLatTextField.getText().length() > 0)
	{
	    try
	    {

/*---------------------------------------------------------------------------
 *  open the existing config file, read each line, append the new map to the
 *  the MAPS line, append the list of children to the new map's parent and
 *  copy other lines into new buffer.
 *---------------------------------------------------------------------------*/
                
		mapFile = new File(GTrace.HOMEDIR + GTrace.separator 
				   + "maps" + GTrace.separator 
				   + "GTraceMaps.properties");

		fptr = new RandomAccessFile(mapFile, "r");

		while((str = fptr.readLine()) != null)
		{
		    if (str.startsWith("MAPS"))
		    {
			newMapConf = newMapConf.concat(str 
                                  + "," + fileNameTextField.getText() + "\n");
		    }
		    else if (str.startsWith(
                                     (String)parentCombo.getSelectedItem() 
				     + ".CHILDREN"))
		    {
			newMapConf = newMapConf.concat(str 
                                   + "," + fileNameTextField.getText() + "\n");
			parentAdded = true;
		    }
		    else
		    {
			newMapConf = newMapConf.concat(str + "\n");
		    }
		}

/*---------------------------------------------------------------------------
 *  if a parent map;s .CHILREN does not exist, create one and add the current
 *  map name to the list.
 *---------------------------------------------------------------------------*/
                
		if (!parentAdded)
		{
		    int lastIndex = newMapConf.lastIndexOf(
                                        (String)parentCombo.getSelectedItem());
		    int length = newMapConf.length();
		    for (int i=lastIndex; i<length; i++)
		    {
			if (newMapConf.charAt(i) == '\n' || i == length-1)
			{
			    lastIndex = i;
			    break;
			}
		    }
		
		    newMapConf = newMapConf.substring(0,lastIndex) +
			"\n" + (String)parentCombo.getSelectedItem()
                        + ".CHILDREN=" +
			fileNameTextField.getText()
                        + newMapConf.substring(lastIndex);
		}

		fptr.close();
	    }
	    catch(IOException ioe) 
	    {
		JOptionPane.showMessageDialog(frame,
                   "Could not add " + fileNameTextField.getText()
                   + " to the\n" +
                   "GTraceMaps.properties configuration file. \n" +
                   "Try manually adding the map using the instructions\n" +
                   "found on the GTrace web page www.caida.org/Tools/GTrace\n",
                   "Map error",
                   JOptionPane.ERROR_MESSAGE);
		return;
	    }

/*---------------------------------------------------------------------------
 *  delete exiting file, and write contents present in newMapsConf buffer
 *  back to the file, append the currently added new map and its attributes
 *  at the end.
 *---------------------------------------------------------------------------*/
            
	    try
	    {
		if (mapFile != null)
		{
		    String comboText;
		    mapFile.delete();
		    fptr = new RandomAccessFile(mapFile, "rw");
		    fptr.writeBytes(newMapConf);

		    fptr.writeBytes("\n" + fileNameTextField.getText()
                                    + ".BOUNDARY="
                                    + tlhcLonTextField.getText() + ","
                                    + tlhcLatTextField.getText() + ","
                                    + brhcLonTextField.getText() + ","
                                    + brhcLatTextField.getText() + "\n");

		    fptr.writeBytes(fileNameTextField.getText() + ".PARENT=" 
                                    + (String)parentCombo.getSelectedItem()
                                    + "\n");
		
		    String childrenText = childrenTArea.getText();
		    String childTokens[] = Util.tokenizeToString(childrenText,
                                                                 "\n");

		    if (childTokens.length > 0)
		    {
			fptr.writeBytes(fileNameTextField.getText()
                                        + ".CHILDREN=");
		    
			for (int i=0; i<childTokens.length-1; i++)
			{
			    fptr.writeBytes(childTokens[i] + ",");
			}

			fptr.writeBytes(childTokens[childTokens.length-1]
                                        + "\n");
		    }

		    comboText = (String)fgCombo.getSelectedItem();
		    if (comboText.compareTo("white") != 0)
		    {
			fptr.writeBytes(fileNameTextField.getText()
                                        + ".FGCOLOR="
					+ comboText + "\n");
		    }
		    
		    comboText = (String)bgCombo.getSelectedItem();
		    if (comboText.compareTo("gray") != 0)
		    {
			fptr.writeBytes(fileNameTextField.getText()
                                        + ".BGCOLOR="
					+ comboText + "\n");
		    }

		    comboText = (String)projCombo.getSelectedItem();
		    if (comboText.compareTo("Alber") == 0
                        || comboText.compareTo("Lambert") == 0)
		    {
			fptr.writeBytes(fileNameTextField.getText()
                                        + ".PROJECTION="
                                        + comboText.toUpperCase() + ","	
                                        + scaleTextField.getText() + ","
                                        + stndprll1TextField.getText() + ","
                                        + stndprll2TextField.getText() + ","
                                        + originLatTextField.getText() + ","
                                        + originLonTextField.getText() + ","
                                        + falseEastingTextField.getText() + ","
                                        + falseNorthingTextField.getText()
                                        + "\n");    
		    }
		    else if (comboText.compareTo("XY-Coordinates") == 0)
		    {
			fptr.writeBytes(fileNameTextField.getText()
                                        + ".PROJECTION="
                                        + comboText + "\n");
		    }

		    fptr.close();
		}
	    }
	    catch(IOException ioe) 
	    {
		JOptionPane.showMessageDialog(frame,
                  "Error writing Map configuration file, file might be corrupt"
                  + "\nrestore orginal on next invocation of GTrace.",
                  "Map error",
                  JOptionPane.ERROR_MESSAGE);
		return;
	    }

            mapAddDialog.setVisible(false);
	    mapAddDialog.dispose();
	    JOptionPane.showMessageDialog(frame,
                "You must restart GTrace for changes to take effect",  
                "Map Added",
                JOptionPane.INFORMATION_MESSAGE);
	}
	else
	{
	    JOptionPane.showMessageDialog(frame,
                 "You at least have to provide the following details\n" + 
                 "to add a map:\n" +
                 "\n\t1. File Name" + 
                 "\n\t2. Boundaries" +
                 "\n\t3. Parent",  
                 "Map error",
                 JOptionPane.ERROR_MESSAGE);
	}
    }
}
