/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    MapDelete.java
**
**     Purpose:                 Constructs and displays the deletion of maps
**                              dialog that deletes a map from the
**                              GTraceMaps.properties configuration file.
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept. 13th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

public class MapDelete extends JDialog implements ActionListener
{
    JFrame frame;            // the parent gtrace frame
    JDialog mapDeleteDialog; // the dialog
    JLabel selectMap;        // select map label
    JButton delete;          // delete button
    JButton cancel;          // cancel button
    JComboBox mapCombo;      // combo box showing list of maps

    // The width and height of the dialog
    public static int WIDTH = 350;
    public static int HEIGHT = 205;
    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Purpose:                 Constructs the map add dialog and adds the 
 **                              components to the dialog
 **    
 **     Parameters:              JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
 **
 *****************************************************************************/
    
    public MapDelete(JFrame frame) 
    {
	this.frame = frame;

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
	mapDeleteDialog = new JDialog(frame, "Delete Map", true);
	mapDeleteDialog.setSize(WIDTH, HEIGHT);
	mapDeleteDialog.setResizable(false);

	JPanel mapDelete = new JPanel();
	mapDelete.setLayout(new BorderLayout());
	mapDelete.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	setMPMSize(mapDelete,new Dimension(410,460));

	selectMap = new JLabel("Select Map");

        mapCombo = new JComboBox();
	mapCombo.setEditable(false);
	setMPMSize(mapCombo,new Dimension(200,25));

	String maps[] = GTrace.getSortedMapKeys();
	for (int i=0; i < maps.length; i++)
	{
	    mapCombo.addItem(maps[i]);
	}

	JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        topPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	topPanel.add(selectMap);
	topPanel.add(Box.createRigidArea(new Dimension(10,0)));
	topPanel.add(mapCombo);

	delete = new JButton("Delete");
	delete.setMnemonic('D');
	delete.setBorder(BorderFactory.createRaisedBevelBorder());
	setMPMSize(delete,new Dimension(100,30));
	delete.addActionListener(this);
	delete.setAlignmentX(Component.CENTER_ALIGNMENT);    

	cancel = new JButton("Cancel");
	cancel.setMnemonic('C');
	cancel.setBorder(BorderFactory.createRaisedBevelBorder());
	setMPMSize(cancel,new Dimension(100,30));
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    mapDeleteDialog.setVisible(false);
                    mapDeleteDialog.dispose();
                }
	});
	cancel.setAlignmentX(Component.CENTER_ALIGNMENT);    

	JPanel bottomPanel = new JPanel();
	bottomPanel.setLayout(new FlowLayout());
        bottomPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	bottomPanel.add(delete);
	bottomPanel.add(Box.createRigidArea(new Dimension(10,0)));
	bottomPanel.add(cancel);

        mapDelete.add(topPanel,BorderLayout.NORTH);
        mapDelete.add(Box.createRigidArea(new Dimension(0,70)));
        mapDelete.add(bottomPanel,BorderLayout.SOUTH);

/*---------------------------------------------------------------------------
 *  add the delete panel to the content pane of the map dialog and set its
 *  postion relative to the parent frame and display the dialog.
 *---------------------------------------------------------------------------*/
        
	mapDeleteDialog.getContentPane().add(mapDelete);
	mapDeleteDialog.setLocationRelativeTo(frame);
	mapDeleteDialog.setVisible(true);

    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 10th, 1999
 **
 *****************************************************************************/
    
    public void actionPerformed(ActionEvent e)
    {
	String mapName = (String)mapCombo.getSelectedItem(); /* name of map
                                                                to delete
                                                             */
	String newMapConf = new String(); /* string that will hold the
                                             contents of modified config file
                                          */

	String str;                   /* string holding contents of each line
                                         read from file
                                      */
	int index;                    /* point in a string where mapName
                                         occurs
                                      */
	File mapFile = null;          // file object 
	RandomAccessFile fptr = null; /* random access file pointer to file
                                         file object
                                      */
        
/*---------------------------------------------------------------------------
 *  open the config file, read each line, delete any line that starts with
 *  mapName, if a line contains mapName somewhere in it, remove it.
 *---------------------------------------------------------------------------*/

	try
	{
	    mapFile = new File(GTrace.HOMEDIR + GTrace.separator 
			       + "maps" + GTrace.separator 
			       + "GTraceMaps.properties");

	    fptr = new RandomAccessFile(mapFile, "r");

	    while((str = fptr.readLine()) != null)
	    {
		if (str.startsWith(mapName))
		{
		    // delete the entire line by not adding it to newMapConf
		}
		else if ((index = str.indexOf(mapName)) > 0)
	        {
                    if ((index+mapName.length()+1) < str.length())
		    {
			str = str.substring(0,index) + str.substring(index
                                                          +mapName.length()+1);
		    }
		    else
		    {
			if (str.charAt(index-1) == ',')
			{
			    str = str.substring(0,index-1);
			}
			else
			{
			    str = str.substring(0,index);
			}
		    }
		    newMapConf = newMapConf.concat(str + "\n");
		}
		else
	        {
		    newMapConf = newMapConf.concat(str + "\n");
		}
	    }

	    fptr.close();
	}
	catch(IOException ioe) 
	{ 
	    JOptionPane.showMessageDialog(frame,
                              "Could not remove " + mapName + " from the\n" +
                              "GTraceMaps.properties configuration file ",  
                              "Delete Map error",
                              JOptionPane.ERROR_MESSAGE);
	    return;
	}

/*---------------------------------------------------------------------------
 *  delete exiting file, and write contents present in newMapConf buffer
 *  back to the file.
 *---------------------------------------------------------------------------*/

	try
	{
	    if (mapFile != null)
	    {
		mapFile.delete();
		fptr = new RandomAccessFile(mapFile, "rw");
		fptr.writeBytes(newMapConf);
		fptr.close();
	    }
	}
	catch(IOException ioe) 
	{
	    JOptionPane.showMessageDialog(frame,
                "Error writing Map configuration file, file might be corrupt" +
                "\nrestore orginal on next invocation of GTrace.",  
                "Map error",
                 JOptionPane.ERROR_MESSAGE);
	    return;
	}

	mapDeleteDialog.setVisible(false);
	mapDeleteDialog.dispose();
	JOptionPane.showMessageDialog(frame,
                "You must restart GTrace for changes to take effect",  
                "Map Removed",
                JOptionPane.INFORMATION_MESSAGE);
    }

/******************************************************************************
 **
 **     Method name:             setMPMSize
 **
 **     Purpose:                 Sets the Minimum, Preferred and Maximimum
 **                              sizes for a component
 **    
 **     Parameters:              JComponent component - the component whose
 **                                       size attributes are to be set.
 **                              Dimension d - the dimension to which they are
 **                                       to be set.
 **                              
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/
    
    public void setMPMSize(JComponent component, Dimension d)
    {
	component.setMinimumSize(d);
	component.setPreferredSize(d);
	component.setMaximumSize(d);
    }	
}
