/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    MapImage.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.MissingResourceException;
import java.util.Enumeration;

public class MapImage extends Maps
{
    private static final String path = GTrace.HOMEDIR + GTrace.separator
                                       + "maps" + GTrace.separator;	
    static Vector latlonValues = new Vector();
    
    double offset_x;
    double offset_y;
    
    public static final int TYPE1 = 1;
    public static final int TYPE2 = 2;
    public static final int TYPE3 = 3;
    public static final int NODESIZE = 4;   
    public static final int LONGITUDECENTER = 180;

    public static final int STEP2 = 20;
    public static final int STEP3 = 30;
    public static final int STEP4 = 40;
    public static final int STEP4STATE = 41;
    public static final int STEP4COUNTRY = 42;
    public static final int STEP5 = 50;
    public static final int STEP6 = 60;
    public static final int STEP7 = 70;

    public static int numberOfPlottedPoints = 0;

    /* approx. radius of earth in meters. True radius varies from
       6357 km (polar) to 6378 km (equatorial)
    */
    public static final float earth_radius = 6367;
    public static final int DISTANCEFONTSIZE = 8;

    /* the following constants define the various quadrant combinations */ 
    public final static int quadone = 0;
    public final static int quadtwo = 1;
    public final static int quadthree = 2;
    public final static int quadfour = 3;

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 3th, 1999
**
******************************************************************************/

     public MapImage(String mapName,float mapBoundary[],String mapParent,
		     String mapChildren[],String mapFGColor,String mapBGColor, 
		     String mapProjection)
     {
	 super( mapName,mapBoundary,mapParent,mapChildren,mapFGColor,
                mapBGColor,
		mapProjection);
     }
	


//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
//                             public utility Methods
//---------------------------------------------------------------------------
     
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
**
******************************************************************************/

    public void loadMap()
    {
	if (getImage() == null)
	{
	    Image image = Toolkit.getDefaultToolkit().getImage(MapsPath
                                                               +getMapName());
	    loadImage(image);
	    setImage(image);
	    setDescription(getMapName());
	    readLabels();
	    computeLatLonParameters(getIconWidth(),getIconHeight());
	}
    }

/******************************************************************************
**
**     Method name:             latLonToRealPoint  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              double lat, double lon 
**
**     Return Values:           Point
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public Point latLonToRealPoint(float lat, float lon)
     {
	  Point p = latLonToPoint(lat,lon);
	  p = GTrace.getRealPoint(p);
	  return p;
     }
	

/******************************************************************************
**
**     Method name:             plotpoints  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              Vector points, Component c
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
    
     public void plotpoints(Vector points, Component c)
     {
	  latlonValues = (Vector)points.clone();
          c.repaint();
     }
    
	
/******************************************************************************
**
**     Method name:             toString  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public String toString()
     {
	  return (getMapName() +":" + " Height = " 
		  + this.getIconHeight() 
		  + " Width = " + this.getIconWidth() ); 
     }

/******************************************************************************
**
**     Method name:             withinRegion  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              int x, int y
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public boolean withinRegion(int x, int y)
    {
	 return true;
    }
	
/******************************************************************************
**
**     Method name:             pointWithinComponent  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              Point p,Component c  
**
**     Return Values:           boolean
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     private boolean pointWithinComponent(Point p,Component c)
     {
	 if ( p.x < 0 || p.x > c.getSize().width) 
	 {
	     return false;
	 }
	 else if (p.y < 0 || p.y > c.getSize().height)	 
	 {
	     return false;
	 }

	 return true;
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 1st, 1999
**
******************************************************************************/

    private int getType(int type)
    {
	switch (type)
	{
	    case STEP2:
	    case STEP3:
		        return TYPE1;
	    case STEP4:
	    case STEP5:
		        return TYPE2;
	    case STEP4STATE:
	    case STEP4COUNTRY:
	    case STEP6:
	    case STEP7:
		        return TYPE3;
	}
	return TYPE3;
    }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
**
******************************************************************************/
    
    private void drawLineTitle(Graphics g, Point p1, Point p2,
                               String lineTitle)
    {
        int lineSize = 1;
        String tmp = new String();
        g.setFont(new Font("monospaced", Font.PLAIN,DISTANCEFONTSIZE));
        FontMetrics fm = g.getFontMetrics();
        int position=1;

        Point p = new Point();

        p.x = Math.min(p1.x,p2.x) + Math.abs(p1.x - p2.x)/2;
        p.y = Math.min(p1.y,p2.y) + Math.abs(p1.y - p2.y)/2;

        int strlen = fm.stringWidth(lineTitle);

        if ( !(p1.x == p2.x || p1.y == p2.y) )
        {
            switch(getQuadP1(p1,p2))
            {
            case quadone:
            case quadthree:
                switch(position)
                {
                case 1:
                    p.x = p.x - (lineSize+1)/2 - 
                        strlen - fm.getMaxAdvance();
                    p.y = p.y - fm.getLeading() - 
                        fm.getDescent();
                    break;
                case 2:
                    p.x = p.x - (lineSize+1)/2 - 
                        strlen/2;
                    break;
                case 3:
                    p.x = p.x + (lineSize+1)/2 + 
                        fm.getMaxAdvance();
                    p.y = p.y + fm.getLeading() + 
                        fm.getAscent();
                }
                break;
            case quadtwo:
            case quadfour:
                switch(position)
                {
                case 3:
                    p.x = p.x - (lineSize+1)/2 - 
                        strlen - fm.getMaxAdvance();
                    p.y = p.y + fm.getLeading() + 
                        fm.getAscent();
                    break;
                case 2:
                    p.x = p.x - (lineSize+1)/2 - 
                        strlen/2;
                    p.y += fm.getAscent();
                    break;
                case 1:
                    p.x = p.x + (lineSize+1)/2 + 
                        fm.getMaxAdvance();
                    p.y = p.y - fm.getLeading() - 
                        fm.getDescent();
                }
            }
        }
        else
        {
            if (p1.x == p2.x)
            {
                switch(position)
                {
                case 1:
                    p.x = p.x - (lineSize+1)/2 - strlen - 
                        fm.getMaxAdvance();
                    break;
                case 2:
                    p.x = p.x - (lineSize+1)/2 - strlen/2;
                    break;
                case 3:
                    p.x = p.x + (lineSize+1)/2 + 
                        fm.getMaxAdvance();
                }
            }
            else
            {
                p.x = p.x - strlen/2;
                switch(position)
                {
                case 1:
                    p.y = p.y - fm.getLeading() - (lineSize+1)/2;
                    break;
                case 3:
                    p.y += fm.getAscent() + (lineSize+1)/2;
                    break;
                case 2:
                    p.y = p.y + fm.getLeading() + fm.getDescent();
                }
            }
        }

        // Handle case when p.x and p.y become invalid values
        g.drawString(lineTitle,p.x, p.y);			 
    }

/******************************************************************************
**
**     Method name:             getQuadP1 (class GeoPlot)
**
**     Purpose:                 
**    
**     Parameters:              Point p1 - start point of the line
**                              Point p2 - end point of the line
**
**     Return Values:           int
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, June 18th, 1999
**
******************************************************************************/

     public static int getQuadP1(Point p1, Point p2)
     {
	  if (p1.x <= p2.x)
	  {
	       if (p1.y >= p2.y)
	       {
		    return quadone;
	       }
	       else
	       {
		    return quadfour;
	       }
	  }
	  else
	  {
	       if (p1.y > p2.y)
	       {
		    return quadtwo;
	       }
	       else
	       {
		    return quadthree;
	       }
	  }
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public float getPointDistance(Point p1, Point p2)
    {

        return new Float(Math.pow(Math.pow((double)(p1.x - p2.x),2.0)
                                  + Math.pow((double)(p1.y - p2.y),2.0)
                                  ,0.5)).floatValue(); 
    }

    
/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Oct. 6th, 1999
**
******************************************************************************/

    public float getEarthDistance(float lat1, float lon1, float lat2,
                                  float lon2)
    {
        double lat1rad = (lat1 * Math.PI)/180; 
        double lon1rad = (lon1 * Math.PI)/180; 
        double lat2rad =  (lat2 * Math.PI)/180; 
        double lon2rad = (lon2 * Math.PI)/180; 

	double dlon = (double)(lon2rad - lon1rad);
	double dlat = (double)(lat2rad - lat1rad);
 

	double a = Math.pow((Math.sin(dlat / 2)),2.0) + Math.cos(lat1rad) 
	          * Math.cos(lat2rad) * Math.pow(Math.sin(dlon/2),2.0);

	float d = new Float(2 * Math.atan2(Math.sqrt(a),
                                           Math.sqrt(1 - a))).floatValue();

	return (earth_radius * d);
    }


    
/******************************************************************************
**
**     Method name:             paintIcon  (class MapImage)  
**
**     Purpose:                 
**    
**     Parameters:              Component c, Graphics g, int x, int y
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     public synchronized void paintIcon(Component c, Graphics g, int x, int y) 
     {	    
	  super.paintIcon(c, g, x, y);

	  String tokens[];
	  g.setColor(Color.black);
	  FontMetrics fm;              /* fontmetrics object used to find
                                          out attributes about current font
                                       */
	  Vector mapLabels = getMapLabels();
          MapLabel label;
	  String labelName;
	  for (Enumeration e = mapLabels.elements() ; e.hasMoreElements() ;) 
	  {
	      label = (MapLabel)e.nextElement();
	      g.setFont(new Font("monospaced",
                                 Font.PLAIN,label.getFontSize()));
	      fm = g.getFontMetrics();

	      Point p = latLonToRealPoint(label.getLat(), label.getLon());
	      labelName = label.getName();
	      if (labelName.indexOf("$") == -1)
	      {
		  g.drawString(labelName,
                               p.x-fm.stringWidth(labelName)/2, p.y);
	      }
	      else
	      {
		  int chHgt = 0;
		  tokens = Util.tokenizeToString(labelName,"$");
	
		  for (int i=0; i<tokens.length; i++)
		  {
		      g.drawString(tokens[i],
                                   p.x-fm.stringWidth(tokens[i])/2, p.y+chHgt);
		      chHgt += fm.getHeight();
		  }
	      }
          }

	  
 	  if (latlonValues.size() > 0)
  	  {
  	       int i=0;
 	       int j=0;
  	       int length = latlonValues.size();
	       JLabel labelPoint;
	       JLabel labelPoint1;
	       JLabel labelPoint2;
	       Integer typeP;
	       Integer typeP1;
	       Integer typeP2;
	       int typ1;
	       int typ2;
	       String labelP1[] = new String[2];
	       String labelP2[] = new String[2];
	       Point P[] = new Point[2];
		
  while ( i < length)
  {
      labelPoint1 = (JLabel)latlonValues.elementAt(i);
      if (labelPoint1.getText() == ""
          || labelPoint1.getText().compareTo(
                                             "Private Internet Use") == 0 
          || !GTraceTableModel.getCheckBoxValueAt(i))
      {
          i++;
          continue;
      }
      else
      {
          labelP1 = Util.tokenizeToString(
                                          labelPoint1.getText(), ",");
          Point p1 = latLonToRealPoint(
                                       Util.stringToLat(labelP1[0]),
                                       Util.stringToLon(labelP1[1]));
		       
          typeP1 = GTraceTableModel.getTypeofLOC(i);
			 
          for (j=i+1; j<length; j++)
          {
              labelPoint2 = (JLabel)latlonValues.elementAt(j);
              if(labelPoint1.getText() != labelPoint2.getText() 
                 && labelPoint2.getText().compareTo(
                                                  "Private Internet Use") != 0 
                 && labelPoint2.getText() != "" 
                 && GTraceTableModel.getCheckBoxValueAt(j))
              {
                  typeP2 = GTraceTableModel.getTypeofLOC(j);

                  for (int k=j+1; k<length; k++)
                  {
                      labelPoint = (JLabel)latlonValues.elementAt(k);
                          
                      if(labelPoint2.getText() != labelPoint.getText())
                      {
                          break;
                      }

                      if (GTraceTableModel.getCheckBoxValueAt(k))
                      {
                          typeP = GTraceTableModel.getTypeofLOC(k);
				       
                          if (typeP.intValue() < typeP2.intValue())
                          {
                              labelPoint2 = labelPoint;
                              typeP2 = typeP;
                          }
                      }
                  }

                  labelP2 = Util.tokenizeToString(labelPoint2.getText(), ",");
                  Point p2 = latLonToRealPoint(Util.stringToLat(labelP2[0]),
                                               Util.stringToLon(labelP2[1])); 
				   
                  g.setColor(Color.black);
                  //g.setColor(Color.red);
                  g.fillOval(p1.x-NODESIZE/2,p1.y-NODESIZE/2,NODESIZE,
                                                                     NODESIZE);
                  g.fillOval(p2.x-NODESIZE/2,p2.y-NODESIZE/2,NODESIZE,
                                                                     NODESIZE);
				   
                  typ1 = getType(typeP1.intValue());
                  typ2 = getType(typeP2.intValue());

                  if (typ1 == TYPE1 && typ2 == TYPE1)
                  {
                      g.setColor(Color.green);
                      //g.setColor(Color.black);
                  }
                  else if (typ1 == TYPE2 && typ2 == TYPE2)
                  {
                      g.setColor(Color.blue);
                      //g.setColor(Color.black);
                  }
                  else if (typ1 == TYPE3 || typ2 == TYPE3)
                  {
                      g.setColor(Color.red);
                      //g.setColor(Color.black);
                  }
                  else
                  {
                      g.setColor(Color.cyan);
                      //g.setColor(Color.black);
                  }

                  float earthDist = getEarthDistance(
                                                Util.stringToLat(labelP1[0]),
                                                Util.stringToLon(labelP1[1]),
                                                Util.stringToLat(labelP2[0]),
                                                Util.stringToLon(labelP2[1]));

                               

   if (Util.comparelon(labelP1[1],labelP2[1]) < LONGITUDECENTER)
   {
       if (pointWithinComponent(p1,c)
           && pointWithinComponent(p2,c))
       {
           g.drawLine(p1.x,p1.y,p2.x,p2.y);
       }
       else
       {
           if (getProjection() == LINEAR)
           {
               g.drawLine(p1.x,p1.y,p2.x,p2.y);
           }
       }
                                   
       float mapDist = getPointDistance(p1,p2);
       if (mapDist > 30 && GTrace.showDistances)
       {
           g.setColor(Color.black);
           drawLineTitle(g, p1, p2, new Integer(
                         new Float(earthDist).intValue() ).toString() + " km");
       }
   } 
   else 
   {
       if (pointWithinComponent(p1,c)
           && pointWithinComponent(p2,c)) 
       { 
           if (p1.x < p2.x)
           {
               Point pTemp = p2;
               p2 = p1;
               p1 = pTemp;
           }
		    
           P[0] = new Point(p1.x - c.getSize().width,p1.y);
           P[1] = p2;
				       
           float mdash = new Float(P[0].y - P[1].y).floatValue()/
               new Float(P[0].x - P[1].x).floatValue();
           float cdash = new Integer(P[0].y).floatValue()
               - (new Float(mdash).floatValue()
                  * new Integer(P[0].x).floatValue());
           int ydash = Math.round(cdash);
				       
           g.drawLine(p1.x,p1.y,c.getSize().width,ydash);
           g.drawLine(0,ydash,p2.x,p2.y);

           float mapDistP1 = getPointDistance(p1,
                                          new Point(c.getSize().width,ydash));
           float mapDistP2 = getPointDistance(new Point(0,ydash),p2);

           if (mapDistP1 > 30 && mapDistP1 > mapDistP2
                                                       && GTrace.showDistances)
           {
               g.setColor(Color.black);
               drawLineTitle(g, p1, new Point(c.getSize().width,ydash),
                    new Integer( new Float(earthDist).intValue() ).toString()
                             + " km");
           }
           else if (mapDistP2 > 30 && mapDistP2 > mapDistP1
                    && GTrace.showDistances)
           {
               g.setColor(Color.black);
               drawLineTitle(g, new Point(0,ydash), p2,
                  new Integer( new Float(earthDist).intValue() ).toString()
                             + " km");
           }
                                           
       }      
       else if (!pointWithinComponent(p1,c) && getProjection() == LINEAR)
       {
           // From P2's perspective of things  
           float latp1 = Util.stringToLat(labelP1[0]);
           float lonp1 = Util.stringToLon(labelP1[1]);
           float latp2 = Util.stringToLat(labelP2[0]);
           float lonp2 = Util.stringToLon(labelP2[1]);
           Point orgp2 = latLonToPoint(latp2,lonp2); 
           float x1 = getIconWidth() - orgp2.x; 
           float y1; 
           float laty ;
           if (latp2 < latp1) 
           { 
               y1 = orgp2.y; 
               laty = getmapBoundaryTopBound() - latp2; 
           } 
           else 
           { 
               y1 = getIconHeight() - orgp2.y; 
               laty = Math.abs(getmapBoundaryBottomBound() - latp2); 
           } 

           float latx = getmapBoundaryRightBound() - lonp2; 

           float realx = Util.difflon(labelP1[1],labelP2[1])*x1 /latx;
           float realy = Util.difflat(labelP1[0],labelP2[0])*y1 /laty;
		    
           float slope = realx / realy;
           int y1dash = new Float((c.getSize().width - p2.x)
                                  / slope).intValue();
				       
           if (latp2 < latp1)
           {
               g.drawLine(c.getSize().width,p2.y-y1dash,p2.x,p2.y);
                                           
               float mapDistP1 = getPointDistance(
                                 new Point(c.getSize().width,p2.y-y1dash),p2);
               if (mapDistP1 > 30  && GTrace.showDistances)
               {
                   g.setColor(Color.black);
                   drawLineTitle(g, new Point(c.getSize().width,p2.y-y1dash),
                                 p2,
                   new Integer( new Float(earthDist).intValue() ).toString()
                                 + " km");
               }

           }
           else
           {
               g.drawLine(c.getSize().width,p2.y+y1dash,p2.x,p2.y);

               float mapDistP1 = getPointDistance(new Point(c.getSize().width,
                                                            p2.y+y1dash),p2);
               if (mapDistP1 > 30  && GTrace.showDistances)
               {
                   g.setColor(Color.black);
                   drawLineTitle(g, new Point(c.getSize().width,p2.y+y1dash),
                                 p2,
                    new Integer( new Float(earthDist).intValue() ).toString()
                                 + " km");
               }
           }
       }
       else if (getProjection() == LINEAR)
       {
           // From P1's perspective of things 
           float latp1 = Util.stringToLat(labelP1[0]);
           float lonp1 = Util.stringToLon(labelP1[1]);
           float latp2 = Util.stringToLat(labelP2[0]);
           float lonp2 = Util.stringToLon(labelP2[1]);
           Point orgp1 = latLonToPoint(latp1,lonp1); 

           float x1 = orgp1.x;
           float y1;
           float laty;

           if (latp1 > latp2)
           {
               y1 = getIconHeight() - orgp1.y;
               laty = Math.abs(getmapBoundaryBottomBound() - latp1);
           }
           else
           {
               y1 = orgp1.y;
               laty = latp1 - getmapBoundaryTopBound();
           }
				       
           float latx = lonp1 - getmapBoundaryTopBound();
				       
           float realx = Util.difflon(labelP1[1],labelP2[1])*x1 /latx;
           float realy = Util.difflat(labelP1[0],labelP2[0])*y1 /laty;
				       
           float slope = realx / realy;
           int y1dash = new Float(p1.x / slope).intValue();
				       
           if (latp1 > latp2)
           {
               g.drawLine(p1.x,p1.y,1,p1.y+y1dash);

               float mapDistP1 = getPointDistance(p1,new Point(1,p1.y+y1dash));
               if (mapDistP1 > 30  && GTrace.showDistances)
               {
                   g.setColor(Color.black);
                   drawLineTitle(g, p1, new Point(1,p1.y+y1dash),
                    new Integer( new Float(earthDist).intValue() ).toString()
                                 + " km");
               }
           }
           else
           {
               g.drawLine(p1.x,p1.y,1,p1.y-y1dash);
                                           
               float mapDistP1 = getPointDistance(p1,new Point(1,p1.y-y1dash));
               if (mapDistP1 > 30  && GTrace.showDistances)
               {
                   g.setColor(Color.black);
                   drawLineTitle(g, p1, new Point(1,p1.y-y1dash),
                    new Integer( new Float(earthDist).intValue() ).toString()
                                 + " km");
               }

           }
       }
   }
   break;
              }
          }
          i = j;
      }
  }
	  }
     }
}
