/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    Maps.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.io.File;
import java.io.RandomAccessFile;
import java.io.IOException;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import javax.swing.*;
import java.awt.*;


public class Maps extends ImageIcon
{
    private String mapName;                // the map name
    private String mapChildren[];          // children of this map
    private Vector mapLabels = new Vector(); // labels on map
    private String mapParent;                // parent of this map
    private String mapFGColor;  // xxx - change to type Color
    private String mapBGColor;  // xxx - change to type Color
    private String mapLabelFileName = new String(); // the actual file name
 
    protected String MapsPath = GTrace.HOMEDIR + File.separator + "maps"
                                + File.separator;

/*---------------------------------------------------------------------------
 *  format of mapBoundary 
 *  {left_bound, top_bound, right_bound, bottom_bound}
 *  [0,1,2,3] {top_lon, top_lat, bot_lon, bot_lat}
 *---------------------------------------------------------------------------*/
    private float mapBoundary[];

    //parameters for linear/mercator/XY projection
    public float bLat;	
    public float bLon;	
    public float aLat;
    public float aLon;

    //parameters for alber/lambert projection
    public double R;        // Radius of sphere
    public double phione;   // standard parallel
    public double phitwo;   // standard parallel
    public double phinought;    // orgin latitude
    public double lambdanought; // orgin longitude
    public int falseEasting;    // the false easting amount
    public int falseNorthing;   // the false northing amount

    // variables calculated when determing alber/lambert mappings
    public static double PHIONE;
    public static double PHITWO;
    public static double PHINOUGHT;
    public static double LAMBDANOUGHT;
    public static double n;
    public static double C;

    // Constants
    public final int LINEAR = 1;
    public final int MERCATOR = 2;
    public final int LAMBERT = 3;
    public final int ALBER = 4;
    public final int XYCOORD = 5;

    private int mapProjection = LINEAR;   /* mapprojection is set to linear
                                             by default
                                          */

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Purpose:                 
 **    
 **     Parameters:             
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 3th, 1999
 **
 *****************************************************************************/

    public Maps(String mapName,float mapBoundary[],String mapParent,
                String mapChildren[],String mapFGColor,String mapBGColor, 
                String mapProjection)
    {
	this.mapName = mapName;
	this.mapBoundary = mapBoundary;
	this.mapParent = mapParent;
	this.mapChildren = mapChildren;
	this.mapFGColor = mapFGColor;
	this.mapFGColor = mapBGColor;

	if (mapProjection != null)
        {
	    if (mapProjection.compareTo("LINEAR") == 0)
	    {
		this.mapProjection = LINEAR;
	    }
	    else if (mapProjection.startsWith("LAMBERT"))
	    {
		this.mapProjection = LAMBERT;
		String tokens[] = Util.tokenizeToString(mapProjection,",");
		try
		{
		    R = new Double(tokens[1]).doubleValue();
		    phione = new Double(tokens[2]).doubleValue();
		    phitwo = new Double(tokens[3]).doubleValue();
		    phinought = new Double(tokens[4]).doubleValue();
		    lambdanought = new Double(tokens[5]).doubleValue();
		    falseEasting = new Integer(tokens[6]).intValue(); 
		    falseNorthing = new Integer(tokens[7]).intValue();

		    PHIONE = (phione * Math.PI)/180;
		    PHITWO = (phitwo * Math.PI)/180;
		    PHINOUGHT = (phinought * Math.PI)/180;
		    LAMBDANOUGHT = (lambdanought * Math.PI)/180;

		    n = (Math.sin(PHIONE) + Math.sin(PHITWO))/2;
		    C = (Math.pow(Math.cos(PHIONE),2)
                         + (2*n*Math.sin(PHIONE)));
		}
		catch(NumberFormatException nfe)
		{
		    // invalid map attributes
		}
	    }
	    else if (mapProjection.startsWith("ALBER"))
	    {
		this.mapProjection = ALBER;
		String tokens[] = Util.tokenizeToString(mapProjection,",");
		try
		{
		    R = new Double(tokens[1]).doubleValue();
		    phione = new Double(tokens[2]).doubleValue();
		    phitwo = new Double(tokens[3]).doubleValue();
		    phinought = new Double(tokens[4]).doubleValue();
		    lambdanought = new Double(tokens[5]).doubleValue();
		    falseEasting = new Integer(tokens[6]).intValue(); 
		    falseNorthing = new Integer(tokens[7]).intValue();

		    PHIONE = (phione * Math.PI)/180;
		    PHITWO = (phitwo * Math.PI)/180;
		    PHINOUGHT = (phinought * Math.PI)/180;
		    LAMBDANOUGHT = (lambdanought * Math.PI)/180;

		    n = (Math.sin(PHIONE) + Math.sin(PHITWO))/2;
		    C = (Math.pow(Math.cos(PHIONE),2)
                         + (2*n*Math.sin(PHIONE)));
		}
		catch(NumberFormatException nfe)
		{
		    // invalid map attributes
		}
	    }
	    else if (mapProjection.startsWith("XYCOORD"))
	    {
		this.mapProjection = XYCOORD;
	    }
	}

	mapLabelFileName = mapName.substring(0,mapName.lastIndexOf('.'));
    }


//---------------------------------------------------------------------------
//                             initMethods
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//                             getMethods
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Method name:             getLat    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int y
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public float getLat(Point p)
    {
	return getLat(p.x, p.y);
    }
    
/******************************************************************************
 **
 **     Method name:             getLat    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int y
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/
    
    public float getLat(int x, int y)
    {
 	if (mapProjection == LINEAR)
	{
	    return ( aLat*y ) + bLat;
	}
	else if (mapProjection == ALBER || mapProjection == LAMBERT)
	{
	    double X = new Double(x-falseEasting).doubleValue();
	    double Y = new Double(falseNorthing-y).doubleValue();

	    double pnought = (R * Math.sqrt(C - (2*n*Math.sin(PHINOUGHT))))/n;

	    double p = Math.sqrt(Math.pow(X,2) + Math.pow((pnought-Y),2));

	    return new Float(((Math.asin((
                 C - Math.pow((p*n)/R, 2))/(2*n)))*180)/Math.PI).floatValue();
	}
	
	return Float.POSITIVE_INFINITY;
    }

/******************************************************************************
 **
 **     Method name:             getLon    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int x
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public float getLon(Point p)
    {
	return getLon(p.x,p.y);
    }


/******************************************************************************
 **
 **     Method name:             getLon    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int x
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/
    
    public float getLon(int x, int y)
    {
 	if (mapProjection == LINEAR)
	{
	    return (aLon*x ) + bLon;
	}
	else if (mapProjection == ALBER || mapProjection == LAMBERT)
	{
	    double X = new Double(x-falseEasting).doubleValue();
	    double Y = new Double(falseNorthing-y).doubleValue();

	    double pnought = (R * Math.sqrt(C - (2*n*Math.sin(PHINOUGHT))))/n;

	    double theta = Math.atan(X/(pnought-Y));

	    return new Float(((LAMBDANOUGHT
                               + theta/n)*180)/Math.PI).floatValue();
	}

	return Float.POSITIVE_INFINITY;
    }

/******************************************************************************
 **
 **     Method name:             
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/


    public String getMapName()
    {
	return mapName;
    }

/******************************************************************************
 **
 **     Method name:             getMapLabelFileName    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int x
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public String getMapLabelFileName()
    {
	return mapLabelFileName;
    }

/******************************************************************************
 **
 **     Method name:             getParentMap    
 **
 **     Purpose:                 
 **    
 **     Parameters:              int x
 **
 **     Return Values:           double
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public String getParentMap()
    {
	return mapParent;
    }

/******************************************************************************
 **
 **     Method name:             
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 21st, 1999
 **
 *****************************************************************************/

    public int getProjection()
    {
        return mapProjection;
    }
    
/******************************************************************************
 **
 **     Method name:             getChildMap  (MltClkMapImage)
 **
 **     Purpose:                 
 **    
 **     Parameters:              int x
 **                              int y
 **
 **     Return Values:           String[]
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/
        	
    public String[] getChildMap(int x, int y)
    {	
        float lat = getLat(x,y);
        float lon = getLon(x,y);

        Maps tmp;
        Vector matches = new Vector();

        if (mapChildren == null)
        {
            String chldName[] = new String[1];
            chldName[0] = null;
            return chldName;
        }

        for (int index = 0 ; index < mapChildren.length; index++)
        {
            tmp = (Maps)GTrace.getMap(mapChildren[index]);

            if ( lat < tmp.mapBoundary[1] && lat > tmp.mapBoundary[3]  
                 && lon > tmp.mapBoundary[0] && lon < tmp.mapBoundary[2] )
            {
                matches.addElement(tmp.mapName);
            }
        }

        if (matches.size() > 0)
        {
            String chldNames[] = new String[matches.size()];
            for (int i=0 ; i < chldNames.length ; i++)
            {
                chldNames[i] = (String)matches.elementAt(i);
            }
            return chldNames;
        }
        else
        {
            String chldName[] = new String[1];
            chldName[0] = null;
            return chldName;
        }
    }
    
/******************************************************************************
 **
 **     Method name:             getmapBoundary
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public float[] getmapBoundary()
    {
	return mapBoundary;
    }

/******************************************************************************
 **
 **     Method name:             getmapBoundaryTopBound
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public float getmapBoundaryTopBound()
    {
	return mapBoundary[1];
    }

/******************************************************************************
 **
 **     Method name:             getmapBoundaryBottomBound
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public float getmapBoundaryBottomBound()
    {
	return mapBoundary[3];
    }

/******************************************************************************
 **
 **     Method name:             getmapBoundaryLeftBound
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public float getmapBoundaryLeftBound()
    {
	return mapBoundary[0];
    }

/******************************************************************************
 **
 **     Method name:             getmapBoundaryRightBound
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public float getmapBoundaryRightBound()
    {
	return mapBoundary[2];
    }


/******************************************************************************
 **
 **     Method name:             getMapLabels
 **
 **     Purpose:                 
 **    
 **     Parameters:              
 **
 **     Return Values:           
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 1th, 1999
 **
 *****************************************************************************/

    public Vector getMapLabels()
    {
	return mapLabels;
    }


//---------------------------------------------------------------------------
//                             public utility Methods
//---------------------------------------------------------------------------


/******************************************************************************
 **
 **     Method name:             pointToLatLon      
 **
 **     Purpose:                 
 **    
 **     Parameters:              Point p
 **
 **     Return Values:           String
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public String pointToLatLon(Point p)
    {
        float lat = getLat(p.x,p.y);		
        float lon = getLon(p.x,p.y);
	     
        if (lon > 180)
        {
            lon = lon - 360;
        }

        return Util.roundNum(lat) + ((lat < 0.0) ? "S" :"N") + ", " 
            + Util.roundNum(lon) + ((lon < 0.0) ? "W" :"E");
    }


/******************************************************************************
 **
 **     Method name:             computeLatLonParameters    
 **
 **     Purpose:                 
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public void computeLatLonParameters(int width, int height)
    {
 	if (mapProjection == LINEAR)
	{
	    bLon = mapBoundary[0];
	    bLat = mapBoundary[1];
	    aLon = (mapBoundary[2]-mapBoundary[0])/width;
	    aLat = (mapBoundary[3]-mapBoundary[1])/height;
	}
    }


/******************************************************************************
 **
 **     Method name:             latLonToPoint    
 **
 **     Purpose:                 
 **    
 **     Parameters:              double lat, double lon
 **
 **     Return Values:           Point
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public Point latLonToPoint(float lat, float lon)
    {
        if (mapProjection == LINEAR)
        {
            Point p = new Point();
            p.y = (int)( ( lat-bLat )/aLat + 0.5 );
            p.x = (int)( ( lon-bLon )/aLon + 0.5 );
            return p;
        }
        else if (mapProjection == ALBER || mapProjection == LAMBERT)
        {   
            double LAT = new Double(lat).doubleValue();
            double LON = new Double(lon).doubleValue();
	     
            double phi = (LAT * Math.PI)/180;
            double lambda = (LON * Math.PI)/180;
	     
            double p = (R * Math.sqrt(C - (2*n*Math.sin(phi))))/n;
            double pnought = (R * Math.sqrt(C - (2*n*Math.sin(PHINOUGHT))))/n;
	     
            double theta = n*(lambda - LAMBDANOUGHT);
	     
            Point pt = new Point();
            pt.x = falseEasting + (int)(p * Math.sin(theta));
            pt.y = falseNorthing + (-1*(int)(pnought - p * Math.cos(theta)));
	     
	    return pt;
        }

        return null;
    }	


/******************************************************************************
 **
 **     Method name:             readLabels    
 **
 **     Purpose:                 
 **    
 **     Parameters:              None
 **
 **     Return Values:           Point
 **                                           
 **     Side effects:            NONE
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/


    public void readLabels()
    {
	String fline = new String();
	RandomAccessFile fptr=null;
	
	String fname = MapsPath+mapLabelFileName;

	try
	{
	    fptr = new RandomAccessFile(fname,"r"); 

	    for(;;)
	    {
		fline = fptr.readLine();

		if (fline == null)
		{
		    break;
		}

		if (!fline.startsWith("#") && fline.trim().length() > 0)
		{	
		    try
		    {
			mapLabels.addElement(new MapLabel(fline));
		    }
		    catch(MapException me)
		    {
			/* xxx - write filename and that had a malformed label
                           to GTrace.log
                        */
			System.err.println(fname + " malformed Label");
		    }
		}
	    }
	}
	catch(IOException fnfe) 
	{ 
	    //System.err.println(fname + " not found");
	}
    }

/******************************************************************************
 **
 **     Method name:             paintIcon 
 **
 **     Purpose:                 calls the base class paintIcon method
 **    
 **     Parameters:              Component c - 
 **                              Graphics g
 **                              int x
 **                              int y
 **
 **     Return Values:           void
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public synchronized void paintIcon(Component c, Graphics g, int x, int y) 
    {	    
        super.paintIcon(c, g, x, y);
    }

//---------------------------------------------------------------------------
//                             inner classes
//---------------------------------------------------------------------------
    
    class MapLabel
    {
        String labelName = new String();
        float latLoc;
        float lonLoc;
        int fontSize;

/******************************************************************************
 **
 **     Purpose:                 Parses a labledata value and stores the
 **                              relevant attributes of that label
 **    
 **     Parameters:              String labelData - the label
 **
 **     Side Effects:            throws MapException
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 22nd, 1999
 **
 *****************************************************************************/
    
        public MapLabel(String labelData) throws MapException
        {
            try
            {			
                StringTokenizer tokenizer_a = new StringTokenizer(
                                                               labelData,"=");
                labelName = tokenizer_a.nextToken();

                float tokens[] = Util.tokenizeTofloat(tokenizer_a.nextToken()
                                                                         ,",");
                latLoc = tokens[0];
                lonLoc = tokens[1];
                fontSize = new Float(tokens[2]).intValue();
            }
            catch (NoSuchElementException nsee)
            {
                System.err.println("nsee");
                throw new MapException(MapException.MalFormedLabel);
            }
            catch (NumberFormatException e)
            {
                System.err.println("nfe");
                throw new MapException(MapException.MalFormedLabel);
            }
        }

/******************************************************************************
 **
 **     Method name:             getName
 **
 **     Purpose:                 returns the name of the label
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 22nd, 1999
 **
 *****************************************************************************/
    
        public String getName()
        {
            return labelName;
        }

/******************************************************************************
 **
 **     Method name:             getLat
 **
 **     Purpose:                 return the latitude
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           float
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 22nd, 1999
 **
 *****************************************************************************/
    
        public float getLat()
        {
            return latLoc;
        }

/******************************************************************************
 **
 **     Method name:             getLon
 **
 **     Purpose:                 return the longitude
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           float
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 22nd, 1999
 **
 *****************************************************************************/
    
        public float getLon()
        {
            return lonLoc;
        }

/******************************************************************************
 **
 **     Method name:             getFontSize
 **
 **     Purpose:                 returns the font size
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           int
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 22nd, 1999
 **
 *****************************************************************************/
    
        public int getFontSize()
        {
            return fontSize;
        }
    }

}
