/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    MenuBar.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept. 4th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.border.BevelBorder;
import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;

public class MenuBar extends JMenuBar implements ActionListener
{
    JFrame frame;
    GTrace gtrace;
    String[] traceItems = new String[] { "New", "Open", "Save",
                                         "Redo", "Stop", "Exit" };
    String[] mapItems = new String[] { "Add", "Delete" };
    String[] toolItems = new String[] { "whois", "ping", "nslookup",
                                            "Delete Cache" };
    String[] helpItems = new String[] { "Contents", "Location Legend",
                                            "Map Line Legend", "About GTrace" };
    String[] toolDatabaseItems = new String[] { "Create", "Record"};
    char[] traceShortcuts = {'N', 'O', 'S', 'R', 'T', 'X' };
    char[] toolShortcuts = { 'W', 'P', 'L' ,'D'};
    char[] helpShortcuts = { 'H', 'A' };
    char[] toolDatabaseShortcuts = { 'E', 'R' };

    static final String ABOUTMSG =
        "                       GTrace\n" + 
        "      A Graphical Traceroute Tool \n\n" + 
        "                     Written by\n" + 
        "     Ram Periakaruppan (11/01/99)\n" +
        "                 ram@caida.org\n\n" +
        "   Maps courtesy of visualroute.com\n" +
        " Send bugs to gtrace-bugs@caida.org\n\n" +
        "Development of this tool was partially\n" +
        "   funded by NSF under ANI-9996248\n\n" +
        "Cooperative Association for Internet\n" + 
        "            Data Analysis (CAIDA)";


    public static String OPENDIR = System.getProperty("user.dir");
    public static String SAVEDIR = System.getProperty("user.dir");

    boolean show_distances=false;
    boolean show_toolTips=true;
    boolean toolbar = false;

    ToolBar toolBar;

    public static final String newAction  = "New";
    public static final String openAction = "Open";
    public static final String saveAction = "Save";
    public static final String redoAction = "Redo";
    public static final String stopAction = "Stop";
    public static final String exitAction = "Exit";

    public static final String addAction = "Add";
    public static final String deleteAction = "Delete";

    public static final String whoisAction = "whois";
    public static final String nslookupAction = "nslookup";
    public static final String pingAction = "ping";
    public static final String dCacheAction = "Delete Cache";
    public static final String dbNewAction = "Create";
    public static final String dbQAMDAction = "Record";

    public static final String helpAction = "Help";
    public static final String ballLegendAction = "BallLegend";
    public static final String lineLegendAction = "LineLegend";
    public static final String aboutAction = "About";

    public static final String NEW_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "New.gif";
    public static final String OPEN_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "Open.gif";
    public static final String SAVE_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "Save.gif";
    public static final String REDO_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "Redo.gif";
    public static final String STOP_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "Stop.gif";
    public static final String REDDOT_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "redDot.gif";
    public static final String NODOT_PATH = GTrace.HOMEDIR + GTrace.separator
                                          + "images" + GTrace.separator
                                          + "noDot.gif";
                                          
    private Hashtable commands;

    GTraceFileFilter gtr = new GTraceFileFilter(new String[] {"gtr"},
                                                   "Graphical Traces (*.gtr)");
    GTraceFileFilter trd = new GTraceFileFilter(new String[] {"trd"},
                                                    "traceroute Dump (*.trd)");

    /**
     * Actions defined by the Menubar class
     */
    
    private Action[] menuActions = {
        new NewAction(),
        new OpenAction(),
        new SaveAction(),
        new RedoAction(),
        new StopAction(),
        new ExitAction(),
        new AddAction(),
        new DeleteAction(),
        new WhoisAction(),
        new NslookupAction(),	
        new PingAction(),
        new DCacheAction(),       
        new DbNewAction(),
        new DbQAMDAction(),
        new HelpAction(),
        new BallLegendAction(),
        new LineLegendAction(),
        new AboutAction()
    };

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

/******************************************************************************
 **
 **     Purpose:                 
 **    
 **     Parameters:             
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 3th, 1999
 **
 *****************************************************************************/

    public MenuBar(GTrace gtrace, JFrame frame)
    {
	this.frame = frame;
	this.gtrace = gtrace;
	JMenuItem item;
	JMenu traceMenu = new JMenu("Trace");
	JMenu mapMenu = new JMenu("Maps");
	JMenu mapGoMenu = new JMenu("Go");
	JMenu toolMenu = new JMenu("Tools");
	JMenu toolDatabaseMenu = new JMenu("Database");
	JMenu optionMenu = new JMenu("Options");
	JMenu helpMenu = new JMenu("Help");

	traceMenu.setMnemonic('T');
	mapMenu.setMnemonic('M');
	mapGoMenu.setMnemonic('G');
	toolMenu.setMnemonic('L');
	toolDatabaseMenu.setMnemonic('B');
	optionMenu.setMnemonic('O');
	helpMenu.setMnemonic('H');


	commands = new Hashtable();
	for (int i = 0; i < menuActions.length; i++) 
	{
	    Action a = menuActions[i];
	    commands.put(a.getValue(Action.NAME), a);
	}
	
	
	//Create a desktop for internal frames

	
	// Assemble the Trace Menu Items with mnumonics

	item = new JMenuItem(traceItems[0],new ImageIcon(NEW_PATH,
                                                         "New"));
	item.addActionListener((Action)commands.get(traceItems[0]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[0],
						   java.awt.Event.CTRL_MASK, 
						   false) );

	traceMenu.add(item);


	item = new JMenuItem(traceItems[1],new ImageIcon(OPEN_PATH,
                                                         "Open"));
	item.addActionListener((Action)commands.get(traceItems[1]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[1],
						   java.awt.Event.CTRL_MASK, 
						   false) );
	traceMenu.add(item);
	
	item = new JMenuItem(traceItems[2],new ImageIcon(SAVE_PATH,
                                                         "Save"));
	item.addActionListener((Action)commands.get(traceItems[2]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[2],
						   java.awt.Event.CTRL_MASK, 
						   false) );
	traceMenu.add(item);


	item = new JMenuItem(traceItems[3],new ImageIcon(REDO_PATH,
                                                         "Redo"));
	item.addActionListener((Action)commands.get(traceItems[3]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[3],
						   java.awt.Event.CTRL_MASK, 
						   false) );
	traceMenu.add(item);


	item = new JMenuItem(traceItems[4],new ImageIcon(STOP_PATH,
                                                         "Stop"));
	item.addActionListener((Action)commands.get(traceItems[4]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[4],
						   java.awt.Event.CTRL_MASK, 
						   false) );
	traceMenu.add(item);
	
	item = new JMenuItem(traceItems[5]);
	item.addActionListener((Action)commands.get(traceItems[5]));
	item.setAccelerator(KeyStroke.getKeyStroke(traceShortcuts[5],
						   java.awt.Event.CTRL_MASK, 
						   false) );
	traceMenu.add(item);
	    

	traceMenu.insertSeparator(3);
	traceMenu.insertSeparator(6);
	

	// Assemble the MapsGo Menu Items
	String maps[] = GTrace.getSortedMapKeys();
	for (int i=0; i < maps.length; i++)
	{
	    maps[i] = maps[i].substring(0,maps[i].lastIndexOf('.'));
	    String startUpMap = GTrace.getStartUpMap().getMapLabelFileName();

	    if (maps[i].compareTo(startUpMap) == 0)
	    {
		item = new JMenuItem(maps[i],new ImageIcon(REDDOT_PATH
                                                           ,"A red bullet"));
		item.setToolTipText("StartUp Map");
	    }
	    else
	    {
		item = new JMenuItem(maps[i],new ImageIcon(NODOT_PATH
                                                           ,"No bullet"));
	    }

	    item.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ev) 
                    { 
                        GTrace.setMap(ev.getActionCommand());
                    }
	    });
	    mapGoMenu.add(item);
	}
	
	// Assemble the Maps Menu Items with mnumonics
	for (int i=0; i < mapItems.length; i++)
	{
	    item = new JMenuItem(mapItems[i]);
	    item.addActionListener((Action)commands.get(mapItems[i]));
	    mapMenu.add(item);
	}
	mapMenu.add(mapGoMenu);

	// Assemble the toolDatabase Menu Items with mnumonics
	for (int i=0; i < toolDatabaseItems.length; i++)
	{
	    item = new JMenuItem(toolDatabaseItems[i]);
	    item.addActionListener((Action)commands.get(toolDatabaseItems[i]));
	    toolDatabaseMenu.add(item);
	}

	// Assemble the Tools Menu Items with mnumonics
	for (int i=0; i < toolItems.length; i++)
	{
	    item = new JMenuItem(toolItems[i],toolShortcuts[i]);
	    item.addActionListener((Action)commands.get(toolItems[i]));
	    toolMenu.add(item);
	}
	toolMenu.add(toolDatabaseMenu);
	toolMenu.insertSeparator(3);
	toolMenu.insertSeparator(5);

	// Assemble the Options Menu Items with mnumonics
	JMenuItem chkBoxItem;
	optionMenu.add(chkBoxItem = new JCheckBoxMenuItem("Show Distances"));
	chkBoxItem.setSelected(false);
	chkBoxItem.addActionListener(this);
	optionMenu.add(chkBoxItem = new JCheckBoxMenuItem("Show ToolTips"));
	chkBoxItem.setSelected(true);
	chkBoxItem.addActionListener(this);
	optionMenu.add(chkBoxItem = new JCheckBoxMenuItem("Disable Verifier"));
	chkBoxItem.setSelected(false);
	chkBoxItem.addActionListener(this);
        optionMenu.add(chkBoxItem = new JCheckBoxMenuItem("Toolbar"));
	chkBoxItem.setSelected(false);
	chkBoxItem.addActionListener(this);
        optionMenu.add(chkBoxItem = new JCheckBoxMenuItem("Status Bar"));
	chkBoxItem.setSelected(true);
	chkBoxItem.addActionListener(this);
	optionMenu.insertSeparator(2);
	optionMenu.insertSeparator(4);


	// Assemble the Help Menu Items with mnumonics

	item = new JMenuItem(helpItems[0],helpShortcuts[0]);
	item.addActionListener((Action)commands.get("Help"));
	helpMenu.add(item);

        item = new JMenuItem(helpItems[1]);
	item.addActionListener((Action)commands.get("BallLegend"));
	helpMenu.add(item);

        item = new JMenuItem(helpItems[2]);
	item.addActionListener((Action)commands.get("LineLegend"));
	helpMenu.add(item);

	item = new JMenuItem(helpItems[3],helpShortcuts[1]);
	item.addActionListener((Action)commands.get("About")); 

	helpMenu.add(item);
	helpMenu.insertSeparator(1);
        helpMenu.insertSeparator(4);

	add(traceMenu);
	add(mapMenu);
	add(toolMenu);
	add(optionMenu);
	add(helpMenu);

	toolBar = new ToolBar();
    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 4th, 1999
 **
 *****************************************************************************/
    
    public void actionPerformed(ActionEvent e)
    {
	String event = e.getActionCommand();

	if (event.compareTo("Toolbar") == 0)
	{
	    if (toolbar)
	    {
		toolbar = false;
		frame.remove(toolBar);
	    }
	    else
	    {
		toolbar = true;
		frame.getContentPane().add(toolBar, BorderLayout.EAST);
	    }

	    frame.getContentPane().validate();
	}
	else if (event.compareTo("Status Bar") == 0)
	{
	    gtrace.statusBarUpdate();
	}
	else if (event.compareTo("Show Distances") == 0)
	{
 	    if (gtrace.showDistances)
	    {
		gtrace.showDistances = false;
	    }
	    else
	    {
		gtrace.showDistances = true;
	    }
            gtrace.mapScrollPane.getViewport().getView().repaint();
	}
	else if (event.compareTo("Disable Verifier") == 0)
	{
 	    if (gtrace.disableVerifier)
	    {
		gtrace.disableVerifier = false;
	    }
	    else
	    {
		gtrace.disableVerifier = true;
	    }
	}
	else if (event.compareTo("Show ToolTips") == 0)
	{
 	    if (gtrace.showToolTips)
	    {
		gtrace.showToolTips = false;
	    }
	    else
	    {
		gtrace.showToolTips = true;
	    }
	}
    }

//---------------------------------------------------------------------------
//                             inner classes
//---------------------------------------------------------------------------

    
/*---------------------------------------------------------------------------
 *  Source O'rielly Java Swing
 *---------------------------------------------------------------------------*/
    
    public class GTraceFileFilter extends FileFilter
    {
	String[] extensions;
	String description;

	public GTraceFileFilter(String ext)
	{
	    this (new String[] {ext}, null);
	}

	public GTraceFileFilter(String[] exts, String descr)
	{
	    //clone and lowercase the extensions
	    extensions = new String[exts.length];
	    for(int i = exts.length - 1; i>=0; i--)
	    {
		extensions[i] = exts[i].toLowerCase();
	    }

	    //make sure we have a valid description

	    description = (descr == null ? exts[0] + " files" : descr);
	}

	public boolean accept(File f)
	{
	    //allow directories regardless of their extensions
	    if (f.isDirectory())
	    {
		return true;
	    }

	    // check the extension of regular files
	    String name = f.getName().toLowerCase();
	    for (int i = extensions.length - 1; i>=0; i--)
	    {
		if (name.endsWith(extensions[i]))
		{
		    return true;
		}
	    }
	    return false;
	}


	public String getDescription()
	{
	    return description;
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/

    public class ToolBar extends JPanel
    {
	JToolBar toolBar;
	String toolKeys[] =  new String[] { "New", "Open", "Save",
                                                "Redo", "Stop", 
                                                "whois", "ping", "nslookup",
                                                "Help", "About"};
	
	public final Insets insets0 = new Insets(0,0,0,0);
	public final Dimension D33 = new Dimension(3,3);
	public final Dimension D55 = new Dimension(5,5);

	public ToolBar()
	{
	    toolBar = new JToolBar(SwingConstants.VERTICAL);
	    addTool(toolBar, toolKeys[0]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[1]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[2]);
	    toolBar.addSeparator( D55 );
	    toolBar.addSeparator( D55 );
	    addTool(toolBar, toolKeys[3]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[4]);
	    toolBar.addSeparator( D55 );
	    toolBar.addSeparator( D55 );
	    addTool(toolBar, toolKeys[5]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[6]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[7]);
	    toolBar.addSeparator( D55 );
	    toolBar.addSeparator( D55 );
	    addTool(toolBar, toolKeys[8]);
	    toolBar.addSeparator( D33 );
	    addTool(toolBar, toolKeys[9]);
	    toolBar.putClientProperty( "JToolBar.isRollover", Boolean.FALSE );
	    toolBar.setFloatable(false);
	    add(toolBar);
	}
	

	protected void addTool(JToolBar toolBar, String name) 
	{
	    JButton b = (JButton) toolBar.add(  
			new JButton(new ImageIcon(GTrace.HOMEDIR +
                                                  GTrace.separator +
                                                  "images" +
                                                  GTrace.separator +
                                                  name
                                                  + ".gif",name)));
	    b.setToolTipText(name);
	    b.setMargin(insets0);
	    b.addActionListener((Action)commands.get(name)); 
	}   
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/

    class NewAction extends AbstractAction 
    {
	
	NewAction() 
	{
	    super(newAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    GTrace.resetButton.doClick();
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class OpenAction extends AbstractAction 
    {
	
	OpenAction() 
	{
	    super(openAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    JFileChooser chooser = new JFileChooser(OPENDIR);
	    chooser.setMultiSelectionEnabled(false);
	    chooser.addChoosableFileFilter(trd);
	    chooser.addChoosableFileFilter(gtr);

	    int option = chooser.showOpenDialog(frame);
	    if (option == JFileChooser.APPROVE_OPTION)
	    {
		if (chooser.getSelectedFile() != null)
		{
		    OPENDIR = chooser.getCurrentDirectory().getAbsolutePath();
		    String fname =
                        chooser.getCurrentDirectory().getAbsolutePath() + 
                        System.getProperty("file.separator") + 
                        chooser.getSelectedFile().getName();
		    gtrace.readTraceFromFile(fname); 
		}
	    }

	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class SaveAction extends AbstractAction 
    {
	
	SaveAction() 
	{
	    super(saveAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    JFileChooser chooser = new JFileChooser(SAVEDIR);
	    chooser.setMultiSelectionEnabled(false);
	    chooser.addChoosableFileFilter(trd);
	    chooser.addChoosableFileFilter(gtr);

	    int option = chooser.showSaveDialog(frame);
	    if (option == JFileChooser.APPROVE_OPTION)
	    {
		if (chooser.getSelectedFile() != null)
		{
		    SAVEDIR = chooser.getCurrentDirectory().getAbsolutePath();
		    String fname =
                        chooser.getCurrentDirectory().getAbsolutePath() + 
                        System.getProperty("file.separator") + 
                        chooser.getSelectedFile().getName();
		    if (!fname.endsWith(".gtr"))
		    {
			fname = fname.concat(".gtr");
		    }
		    gtrace.saveTraceToFile(fname); 
		}
	    }

	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class RedoAction extends AbstractAction 
    {
	
	RedoAction() 
	{
	    super(redoAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    gtrace.redoTrace();
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class StopAction extends AbstractAction 
    {
	
	StopAction() 
	{
	    super(stopAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    gtrace.incCurrentThreadGroupNumber();
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class ExitAction extends AbstractAction 
    {
	
	ExitAction() 
	{
	    super(exitAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    try
	    {
		FileOutputStream out = new FileOutputStream(GTrace.cacheFile);
		ObjectOutputStream s = new ObjectOutputStream(out);
		s.writeObject(GTrace.cache);
		s.writeObject(GTrace.cache.cache);
		s.flush();
		s.close();
		out.close();
	    }
	    catch(Exception ep) { }

	    try
 	    {
 		File comboHistory = new File(GTrace.comboFile);
		
 		if (comboHistory.exists())
 	        {
 		    comboHistory.delete();
 		}

 		FileOutputStream fptr = new FileOutputStream(comboHistory);
 		Vector elements = GTrace.comboModel.getItems();
 		String comboElements[] = new String[elements.size()];
 		int i=0;
	  	for (Enumeration enmu = elements.elements() ;
                                                     enmu.hasMoreElements() ;) 
	 	{
 		    comboElements[i++] = (String)enmu.nextElement();
 		}

 		for (i=comboElements.length-1; i>=0; i--)
 		{
 		    fptr.write(comboElements[i].getBytes());
		    fptr.write('\n');
		    fptr.flush();
 		}
		
		fptr.close();
 	    }
       	    catch(Exception ep)  
 	    { 
 		System.err.println("Did not write Combo :" + ep.getMessage());
 	    }

	    System.exit(0);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class AddAction extends AbstractAction 
    {
	
	AddAction() 
	{
	    super(addAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new MapAdd(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class DeleteAction extends AbstractAction 
    {
	
	DeleteAction() 
	{
	    super(deleteAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new MapDelete(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class WhoisAction extends AbstractAction 
    {
	
	WhoisAction() 
	{
	    super(whoisAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    GTrace.whoisDialog.showDialog();
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class PingAction extends AbstractAction 
    {
	
	PingAction() 
	{
	    super(pingAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new ping(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class NslookupAction extends AbstractAction 
    {
	
	NslookupAction() 
	{
	    super(nslookupAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new nslookup(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class DCacheAction extends AbstractAction 
    {
	
	DCacheAction() 
	{
	    super(dCacheAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    Object[] options = {"Delete",
                                "Cancel"};
	    int n = JOptionPane.showOptionDialog(frame,
                                                 "Are you sure you want to delete the Cache?",
                                                 "Cache",
                                                 JOptionPane.YES_NO_OPTION,
                                                 JOptionPane.QUESTION_MESSAGE,
                                                 null,
                                                 options,
                                                 options[1]);

	    if (n == 0)
	    {
		File f = new File(GTrace.cacheFile);
		boolean result = f.delete();
		Cache.deleteCache();
		    
		if (!result)
		{
		    JOptionPane.showMessageDialog(frame,
                              "Could not delete the cache. Try deleting\n" +
                              "the file " + GTrace.cacheFile,      
                              "Delete Cache error",
                              JOptionPane.ERROR_MESSAGE);
		}
	    }
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class DbNewAction extends AbstractAction 
    {
	
	DbNewAction() 
	{
	    super(dbNewAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new DbNew(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class DbQAMDAction extends AbstractAction 
    {
	
	DbQAMDAction() 
	{
	    super(dbQAMDAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    new DbQAMD(frame);
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class HelpAction extends AbstractAction 
    {
	
	HelpAction() 
	{
	    super(helpAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{

	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class BallLegendAction extends AbstractAction 
    {
	
	BallLegendAction() 
	{
	    super(ballLegendAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
            gtrace.showLocationLegend();
	}
    }


/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class LineLegendAction extends AbstractAction 
    {
	
	LineLegendAction() 
	{
	    super(lineLegendAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
            gtrace.showMapLineLegend();
	}
    }

/*---------------------------------------------------------------------------
 *  
 *---------------------------------------------------------------------------*/
    
    class AboutAction extends AbstractAction 
    {
	
	AboutAction() 
	{
	    super(aboutAction);
	}
	
        public void actionPerformed(ActionEvent e) 
	{
	    JOptionPane.showMessageDialog(frame, ABOUTMSG, "About GTrace",
                                          JOptionPane.INFORMATION_MESSAGE);
	}
    }
}
