/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    ParseHop.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.Timer;
import com.sleepycat.db.*;

public class ParseHop implements Runnable
{       
    private int hNum;

    private Thread t;
    Process p;
    private String hStr;
    private GTraceTableModel model;
    
    private Integer currentSearchStep;
    private final Integer Private = new Integer(80);
    private final Integer UNKNOWN = new Integer(0);
    private final Integer CLEARBALL = new Integer(1);
    public static final Integer STEP1 = new Integer(10);
    public static final Integer STEP2 = new Integer(20);
    public static final Integer STEP3 = new Integer(30);
    public static final Integer STEP4 = new Integer(40);
    public static final Integer STEP4STATE = new Integer(41);
    public static final Integer STEP4COUNTRY = new Integer(42);
    public static final Integer STEP5 = new Integer(50);
    public static final Integer STEP6 = new Integer(60);
    public static final Integer STEP7 = new Integer(70);

    private ThreadGroup tg;
    
    private static int TimedOut = 0;
    private static boolean lastDisplayedSearchStep = false;
    private Timer timer;
    private Timer nsTimer;

    NetGeoClient netgeo = new NetGeoClient(GTrace.VERSION);


//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------


/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:              int hNum, String hStr, GTraceTableModel model
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     ParseHop(int hNum, String hStr, GTraceTableModel model, ThreadGroup tg)
     {
	  this.hNum = hNum;
	  this.hStr = hStr;
	  this.model = model;
	  this.tg = tg;
	  t = new Thread(tg, this, ""+hNum);
	  t.start();			
     }


//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
//                             private utility Methods
//---------------------------------------------------------------------------
	
/******************************************************************************
**
**     Method name:             replaceSplChars
**
**     Purpose:                 
**    
**     Parameters:              String str
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     private String replaceSplChars(String str)
     {
 	  StringBuffer temp = new StringBuffer(str);
	  for (int i=0; i < temp.length(); i++)
	  {
	       if (!(Character.isLetterOrDigit(temp.charAt(i))))
	       {
		    temp.setCharAt(i,'$');
	       }
	  }
	  str = temp.toString();
	  return str;
     }

/******************************************************************************
**
**     Method name:             replace
**
**     Purpose:                 
**    
**     Parameters:              String str, String seq
**
**     Return Values:           String
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
    
     private String replace(String str, String seq)
     {
	  int loc;
	  int strlen = str.length();
	  while((loc=str.indexOf(seq)) != -1)
	  {
	      StringBuffer temp = new StringBuffer(str);
	      
	      do
	      {
		  temp.setCharAt(loc,' ');
		  if (++loc == strlen)
		  {
		      break;
		  }
	      }
	      while(temp.charAt(loc) != ' ');
	      
	      str = temp.toString();
	  }
	  return(str);
     }


/******************************************************************************
**
**     Method name:             run
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

    public void run()
    {    			
	String[] tableColumns = new String[5];
	String hostname = new String();
	String ipaddr = new String();
	float minRTT;
	StringBuffer sb;
	String cityLOC;
	LocationDetails lDetail;
	
/*---------------------------------------------------------------------------
 *  Eliminate the (ttl=XXX!) that show up in some traceroutes 
 *  jn1-atm0-0-15.hay.vbns.net (204.147.131.214)  27.1 ms (ttl=251!)  
 *  27.1 ms (ttl=251!)  27.1 ms (ttl=251!)
 *---------------------------------------------------------------------------*/

	int ttlindex;
	while((ttlindex = hStr.indexOf("(ttl=")) > 0)
	{
	    sb = new StringBuffer(hStr);
	    for(int cur=ttlindex; cur < hStr.length() ; cur++)
	    {  
		if (sb.charAt(cur) == ')')
		{   
		    sb.setCharAt(cur,' ');
		    break;
		}
		sb.setCharAt(cur,' ');
	    }

	    hStr = sb.toString();
	}

/*---------------------------------------------------------------------------
 *  To handle the case when there is more that one ip address and hostname eg.
 *  sfo-brdr-01.inet.qwest.net (205.171.18.34)  27.526 ms  
 *  27.285 ms sfo-brdr-01.inet.qwest.net (205.171.18.38)  27.921 ms 
 *---------------------------------------------------------------------------*/
	
	int fst = hStr.indexOf(')');
	int lst = hStr.lastIndexOf(')');
	
	while ( lst != fst)
	{
	    sb = new StringBuffer(hStr);
	    int secondspace = -1;
	    
	    for(int cur=lst; cur > fst ; cur--)
	    {  
		if (sb.charAt(cur) == ' ')
		{   
		    secondspace++;   
		    if (secondspace == 1)
		    {
			break;
		    }
		    else
		    {
			while(sb.charAt(--cur) == ' ');
		    }
		}
		sb.setCharAt(cur,' ');
	    }
	    hStr = sb.toString();
	    fst = hStr.indexOf(')');
	    lst = hStr.lastIndexOf(')');
	}

/*---------------------------------------------------------------------------
 *  Eliminate the words 'ms' that appear in the traceroute output 
 *---------------------------------------------------------------------------*/

	int msi; // ms index
	while ((msi = hStr.indexOf(" ms")) > 0)
	{
	    sb = new StringBuffer(hStr);
	    sb.setCharAt(msi+1,' ');
	    sb.setCharAt(msi+2,' ');
	    hStr = sb.toString();
	}


	if (hStr.indexOf("*") == -1 && hStr.indexOf("!") == -1)
	{			
 	    String hStrTokens[] = Util.tokenizeToString(hStr);
	    
	    String rtt = new String();
	    for (int i=3; i<hStrTokens.length; i++)  /* 4th element onwards
                                                        in array are rtts
                                                     */
	    {
		if (!hStrTokens[i].trim().startsWith("ms"))
		{
		    rtt += hStrTokens[i] + " ";
		}
	    }
						
	    try
	    {
		float rtts[] = Util.tokenizeTofloat(rtt);

		float temp;
		
		for(int i=0; i<rtts.length; i++)
		{
		    for(int j=i+1; j<rtts.length; j++)
		    {
			if(rtts[i] > rtts[j])
			{
			    temp = rtts[i];
			    rtts[i] = rtts[j];
			    rtts[j] = temp;
			}
		    }
		}
			  
		tableColumns[0] = hStrTokens[0];    //hop number
		ipaddr = tableColumns[1] = hStrTokens[2];    //ip address
		hostname = tableColumns[2] = hStrTokens[1];    //node name
		tableColumns[3] = "";		    //location
		tableColumns[4] = new Float(rtts[0]).toString();
		model.addRow(tableColumns, hNum, Color.black, tg.getName());
	    }
	    catch(NumberFormatException e)
	    {
		//error condition handle later
		System.out.println("Number Format Exception Error");
	    }								
	}
	else
	{
	    int strlen = hStr.length();
	    int numAsterics = 0;
	    for(int i=0;i<strlen; i++)
	    {
		if(hStr.charAt(i) == '*')
		{
		    numAsterics++;
		}
	    }

	    hStr = hStr.replace('*',' ');
	    hStr = replace(hStr,"!");

 	    String hStrTokens[] = Util.tokenizeToString(hStr);
	    if (numAsterics == GTrace.NUMOFRTTS)
	    {
		tableColumns[0] = new Integer(hStrTokens[0]).toString();
		tableColumns[1] = "Timed Out";
		for(int i=2; i<5; i++)
		{
		    tableColumns[i] = "";
		} 
		model.addRow(tableColumns, hNum, Color.red, tg.getName());
                model.addLocation(tg.getName());
		return;
	    }
	    else
	    {
		float rtts[];
		String rtt = new String();
		for (int i=3; i<hStrTokens.length; i++)  /* 4th element
                                                            onwards in array
                                                            are rtts
                                                         */
		{
		    if (!hStrTokens[i].trim().startsWith("ms"))
		    {
			rtt += hStrTokens[i] + " ";
		    }
		}
						
		try
		{
		    rtts = Util.tokenizeTofloat(rtt);

		    float temp;
		    
		    for(int i=0; i<rtts.length; i++)
		    {
			for(int j=i+1; j<rtts.length; j++)
		        {
			    if(rtts[i] > rtts[j])
			    {
				temp = rtts[i];
				rtts[i] = rtts[j];
				rtts[j] = temp;
			    }
			}
		    }

		    tableColumns[0] = hStrTokens[0];   // hop number
		    ipaddr = tableColumns[1] = hStrTokens[2];  // IP address 
		    hostname = tableColumns[2] = hStrTokens[1]; // hostname
		    tableColumns[3] = "";
		    tableColumns[4] = new Float(rtts[0]).toString();

		    if(numAsterics == 1) 
		    {
			model.addRow(tableColumns, hNum, Color.blue,
                                     tg.getName());
		    }
		    else
		    {
			model.addRow(tableColumns, hNum, Color.red,
                                     tg.getName());
		    }
		}
		catch(NumberFormatException e)
		{
		    System.out.println("Number Format Exception Error");
		}							
	    }
	}


/*---------------------------------------------------------------------------
 *  Check if the IP address fall in the ranges 
 *	   
 *     10.0.0.0 - 10.255.255.255
 *     172.16.0.0.- 172.31.255.255
 *     192.168.0.0 - 192.168.255.255
 *		 
 *  These ranges are reserved for private internet use
 *---------------------------------------------------------------------------*/

	hostname = hostname.toLowerCase();
	ipaddr = Util.ipaddrToString(ipaddr);
	
	if (ipaddr != null)
	{
	    int octets[] = Util.tokenizeToint(ipaddr,".");
	    
	    if (octets[0] == 10
		|| (octets[0] == 172 && octets[1] >= 16 && octets[1] <= 31) 
		|| (octets[0] == 192 && octets[1] == 168))
	    {
		lDetail = new LocationDetails(Float.NaN,Float.NaN, Private);
		model.addLocation(lDetail, hNum, Private, tg.getName());
		model.addCurrentSearchStep(hNum, CLEARBALL, tg.getName());
		return;
	    }
	}


/*---------------------------------------------------------------------------
 *  Step 1 - Check if the location for the IP Address exists in the cache
 *---------------------------------------------------------------------------*/

	currentSearchStep = STEP1;

	LocationDetails cacheEntry = GTrace.cache.lookup(ipaddr);
	if (cacheEntry != null)
	{
            cacheEntry.incrementHits();
	    model.addCurrentSearchStep(hNum, cacheEntry.getSearchStep(),
                                                                 tg.getName());
	    model.addLocation(cacheEntry, hNum, 
			      cacheEntry.getSearchStep(), tg.getName());
	    model.addLOCFound(hNum, tg.getName());
	    return;
	} 


	timer = new Timer(500,new iconTimer());
	timer.start();

	nsTimer = new Timer(5000,new nsTimer());
	nsTimer.start();
        
/*---------------------------------------------------------------------------
 *  Step 2 - Check for a LOC Record
 *---------------------------------------------------------------------------*/
	
 	currentSearchStep = STEP2;
	resetTimer();

	String lookupName;
	String hstTokens[] = Util.tokenizeToString(hostname,".");
	if (hstTokens.length > 1)
	{
	    String hostnames[];
	    String lastDomain = Util.getDomain(hostname, 1);
	    int end=0;
	    
	    if (lastDomain != null && lastDomain.length() == 2)
	    {
		end = hstTokens.length-2;
		hostnames = new String[end];

	    }
	    else
	    {
		end = hstTokens.length-1;
		hostnames = new String[end];
	    }

	    for(int i=0 ; i<end ; i++)
	    {
		lookupName = "";
		for (int j=i; j<hstTokens.length; j++)
		{
		    if (j != hstTokens.length-1)
		    {
			lookupName = lookupName + hstTokens[j].toLowerCase()
                                                                         + ".";
		    }
		    else
		    {
			lookupName = lookupName + hstTokens[j].toLowerCase();
		    }
		}
		hostnames[i] = lookupName;
	    }


	    BufferedReader inStream;

	    for (int i=0; i<hostnames.length; i++)
	    {
		try
		{    
		    p = Runtime.getRuntime().exec(
                           GTrace.getGTraceResource("NSLOOKUP") + " -type=LOC "
                           + hostnames[i]);
		    try
		    {
			p.waitFor();
		    }
		    catch(InterruptedException ie) { continue; }
		    
		    inStream = new BufferedReader(new InputStreamReader(
                                                       p.getInputStream()));
    String str; 	
    while( (str=inStream.readLine()) != null)
    {
        if (str.endsWith("record type LOC, interpreted as:"))
        { 
            str=inStream.readLine();
            int index = str.indexOf("LOC");
            if (index > 0)
            {
                str = str.substring(index+3).trim();
                String[] loctokens = Util.tokenizeToString(
                                                           str);

                timer.stop();

                model.addCurrentSearchStep(hNum, STEP2, tg.getName());
                float lat = Util.degminsecTofloat(loctokens[0],loctokens[1],
                                                  loctokens[2],loctokens[3]);
                float lon = Util.degminsecTofloat(loctokens[4],loctokens[5],
                                                  loctokens[6],loctokens[7]);

                lDetail = new LocationDetails(lat,lon,STEP2);
                model.addLocation(lDetail, hNum, STEP2, tg.getName());
                model.addLOCFound(hNum, tg.getName());
                GTrace.cache.add(ipaddr,lDetail);
                return;
            }
        }
    }
		}
		catch(IOException ioe) { }
	    }
	}
	nsTimer.stop();

/*---------------------------------------------------------------------------
 *  Step 3 - Search for a complete match in the databases and files 
 *           present in the GTrace Configuration File.
 *---------------------------------------------------------------------------*/

	currentSearchStep = STEP3;
	resetTimer();

	DbRecord dbRecord = null;

	for (Enumeration e = GTrace.dbtxts.keys() ; e.hasMoreElements() ;) 
	{
	    String dbtxtname = (String)e.nextElement();
	    if (dbtxtname.compareTo(GTrace.CITIES) != 0
                && dbtxtname.compareTo(GTrace.AIRPORT) != 0)
	    {
		try
		{
		    dbRecord = ((DbTxt)GTrace.dbtxts.get(
                                       dbtxtname)).lookup(hostname,ipaddr);
		    if (dbRecord != null)
		    {
			break;
		    }
		}
		catch(DbException dbe) { }
	    }
	}
      
	if (dbRecord != null)
        {
	    timer.stop();

	    lDetail = new LocationDetails(dbRecord, STEP3);

	    model.addLocation(lDetail, hNum, STEP3, tg.getName());
	    model.addCurrentSearchStep(hNum, STEP3, tg.getName());
	    model.addLOCFound(hNum, tg.getName());
	    GTrace.cache.add(ipaddr,lDetail);
	    return;
	}

/*---------------------------------------------------------------------------
 *  Step 4 - Check for a Domain Parsing File
 *---------------------------------------------------------------------------*/

	currentSearchStep = STEP4;
	resetTimer();

	String firstDomain = Util.getDomain(hostname,1);
	RegexRecord regexRecord;
       
	String domains[] = Util.tokenizeToString(hostname,".");
	String secondDomain = new String();
	if (domains[domains.length-1].length() == 2)
	{
	    secondDomain = Util.getDomain(hostname,3)
                                                  + Util.getDomain(hostname,2);
	}
	else 
	{
	    secondDomain = Util.getDomain(hostname,2);
	}
	DomainParsingFile dpf;
	//System.out.println("domain = " + secondDomain + firstDomain);
	if (firstDomain != null && secondDomain != null)
	{
	    dpf = (DomainParsingFile)GTrace.dpfs.get(secondDomain
                                                               + firstDomain);

	    if (dpf != null)
	    {
		regexRecord = dpf.lookup(hostname);

                if (regexRecord != null)
		{
		    timer.stop();
		    lDetail = new LocationDetails(regexRecord);

		    model.addLocation(lDetail,hNum,
                                      lDetail.getSearchStep(), tg.getName());
		    model.addCurrentSearchStep(hNum,
                                      lDetail.getSearchStep(), tg.getName());
		    model.addLOCFound(hNum, tg.getName());
		    GTrace.cache.add(ipaddr,lDetail);
		    return;
		}
	    }
	}

/*---------------------------------------------------------------------------
 *  Step 5 - Repeat step 3 with reduction technique
 *---------------------------------------------------------------------------*/
	
 	currentSearchStep = STEP5;
	resetTimer();
	
	if (!Character.isDigit(hostname.charAt(hostname.length()-1)))
	{
	    for (Enumeration e = GTrace.dbtxts.keys() ; e.hasMoreElements() ;) 
	    {
		String dbtxtname = (String)e.nextElement();
		if (dbtxtname.compareTo("CITIES") != 0
                                       && dbtxtname.compareTo("AIRPORTS") != 0)
		{
		    try
		    {
			dbRecord = ((DbTxt)GTrace.dbtxts.get(
                                                dbtxtname)).rlookup(hostname);
			if (dbRecord != null)
			{
			    break;
			}
		    }
		    catch(DbException dbe) { }
		}
	    }
	
      
	    if (dbRecord != null)
	    {
		timer.stop();

		lDetail = new LocationDetails(dbRecord, STEP5);
		
		model.addLocation(lDetail, hNum, STEP5, tg.getName());
		model.addCurrentSearchStep(hNum, STEP5, tg.getName());
		model.addLOCFound(hNum, tg.getName());
		GTrace.cache.add(ipaddr,lDetail);
		return;
	    }
	}


/*---------------------------------------------------------------------------
 *  Step 6 - Query NetGeo server with whois server with IP Address
 *---------------------------------------------------------------------------*/

 	currentSearchStep = STEP6;
	resetTimer();
	
	if (ipaddr != null)
	{
	    Hashtable netgeoHash = netgeo.getRecord(ipaddr);

	    float lat = NetGeoClient.getLat( netgeoHash );
	    float lon = NetGeoClient.getLong( netgeoHash );
	    String city = (String)netgeoHash.get("CITY");
	    String state = (String)netgeoHash.get("STATE");
	    String country = ((String)netgeoHash.get("COUNTRY")).toUpperCase();
	    timer.stop();

	    if (!(lat == 0 && lon == 0)) /* xxx change these to nan check
                                            with Jim
                                         */
	    {
		String formattedCity;

		if (city != null && state != null && country != null && 
		    city.length() > 0 && 
		    state.length() > 0 && 
		    country.length() > 0) 
		{
		    if (country.compareTo("US") == 0)
		    {
			formattedCity = Util.formatCityLOC(city + "," + 
							   state + "," +
							   country);
		    }
		    else
		    {
			formattedCity = Util.formatCityLOC(city + ","
                                                           + country);
		    }

		    lDetail = new LocationDetails(lat,lon, STEP6,
                                                  formattedCity);
		}
		else if (country != null && country.length() > 0)
		{
		    lDetail = new LocationDetails(lat,lon, STEP6, country);
		}
		else
		{
		    lDetail = new LocationDetails(lat,lon, STEP6);
		}

		model.addLocation(lDetail, hNum, STEP6, tg.getName());
		model.addCurrentSearchStep(hNum, STEP6, tg.getName());
		model.addLOCFound(hNum, tg.getName());
		GTrace.cache.add(ipaddr,lDetail);
		return;
	    }
	}
	

/*---------------------------------------------------------------------------
 *  Step 7 - Check if last two characters end in a country code
 *---------------------------------------------------------------------------*/
	
 	currentSearchStep = STEP7;
	resetTimer();

	String lastDomain = Util.getDomain(hostname, 1);
	if (lastDomain != null)
	{
	    LatLon country = GTrace.getCountryLatLon(lastDomain);
	    if (country != null)
	    {
		timer.stop();
		lDetail = new LocationDetails(country.getLat(),
                                              country.getLon(),
					      STEP7, lastDomain);
		model.addLocation(lDetail, hNum, STEP7, tg.getName());
		model.addCurrentSearchStep(hNum, STEP7, tg.getName());
		model.addLOCFound(hNum, tg.getName());
		GTrace.cache.add(ipaddr,lDetail);
		return;
	    }
	}

	// all search heuritics fail
	timer.stop(); 
	lDetail = new LocationDetails(Float.POSITIVE_INFINITY,
                                      Float.POSITIVE_INFINITY,
				      UNKNOWN);
	model.addLocation(lDetail, hNum, UNKNOWN, tg.getName());
	model.addCurrentSearchStep(hNum, CLEARBALL, tg.getName());
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

     private void resetTimer()
     {
	 timer.restart();
	 model.addCurrentSearchStep(hNum, currentSearchStep, tg.getName());
	 lastDisplayedSearchStep = true;
     }

/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    class iconTimer implements ActionListener
    {
	public void actionPerformed(ActionEvent  e)
	{
	    if (lastDisplayedSearchStep)
	    {
		model.addCurrentSearchStep(hNum, CLEARBALL, tg.getName());
		lastDisplayedSearchStep = false;
	    }
	    else
	    {
 		model.addCurrentSearchStep(hNum, currentSearchStep,
                                                                 tg.getName());
		lastDisplayedSearchStep = true;
	    }
	}
    }

    class nsTimer implements ActionListener
    {
	public void actionPerformed(ActionEvent  e)
	{
            if (p != null)
            {
                p.destroy();
            }
	}
    }
    
}
