/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    ParseTrace.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, May 12th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import java.io.*;

public class ParseTrace implements Runnable
{
     private String name;
     private String parseDest;
     private GTraceTableModel model;
     private static int TimedOut = 0;
     private ThreadGroup parseTraceThreadGroup;
     public Thread t; 
     Process p;
     boolean traceFromFile = false;
	

/******************************************************************************
**
**     Constructor:             for class ParseTrace
**
**     Purpose:                 
**    
**     Parameters:              String name, String dest,GTraceTableModel model
**
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/

     ParseTrace(String name, String dest, GTraceTableModel model,
                                                                ThreadGroup tg)
     {
	  this.name  = name;
	  parseDest  = dest;
	  this.model = model;
	  parseTraceThreadGroup = tg;
	  traceFromFile = false;
	  t = new Thread(tg, this, name);
	  t.start();
     }	


/******************************************************************************
**
**     Purpose:                 
**    
**     Parameters:             
**
**     Author/Last Modified:    Ram Periakaruppan, Sept 13th, 1999
**
******************************************************************************/

     ParseTrace(String fname, GTraceTableModel model, ThreadGroup tg)
     {
	  this.name  = fname;
	  this.model = model;
	  parseTraceThreadGroup = tg;
	  traceFromFile = true;
	  t = new Thread(tg, this, name);
	  t.start();
     }	



/******************************************************************************
**
**     Method name:             
**
**     Purpose:                 
**    
**     Parameters:              
**
**     Return Values:           
**                                           
**     Author/Last Modified:    Ram Periakaruppan, Sept. 8th, 1999
**
******************************************************************************/

    private boolean checkUnknownHost(String s)
    {
	if (s.trim().toLowerCase().indexOf("unknown host") > 0)
	{
	    return true;
	}
	return false;
    }

/******************************************************************************
**
**     Method name:             run  (class ParseTrace)
**
**     Purpose:                 
**    
**     Parameters:              NONE
**
**     Return Values:           void
**                                           
**     Side effects:            NONE
**
**     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
**
******************************************************************************/
    
    public void run()
    {
	boolean firstHop = false;
	String lineTokens[];
        int hNum=0;

	if (traceFromFile)
	{
	    try
	    {
		RandomAccessFile fp = new RandomAccessFile(name,"r");
		String line;

		while((line = fp.readLine()) != null)
		{
		    if (!firstHop && line != null)
		    {
			lineTokens = Util.tokenizeToString(line);
			
			if (lineTokens[0].trim().compareTo("1") == 0)
			{
			    firstHop = true;
			    new ParseHop(++hNum, line, model,
                                         parseTraceThreadGroup);
			}
		    }
		    else
		    {
			if (line != null && !line.equals(""))
			{
			    lineTokens = Util.tokenizeToString(line);

			    if (lineTokens[0].trim().compareTo(
                                         new Integer(hNum+1).toString()) == 0)
			    {
				new ParseHop(++hNum, line, model,
                                             parseTraceThreadGroup);
			    }
			}
		    }
		}
		fp.close();
	    }
	    catch(IOException e)
	    {
		//could not open and read file
	    }

	    GTrace.displayTextSBar("Waiting for threads to finish "
                                   + "determining locations");
	    while(!model.isTraceComplete(parseTraceThreadGroup.getName()))
	    {
		try { Thread.sleep(500); } catch(InterruptedException ie) { }
	    }
	
	    GTrace.displayTextSBar("Done");
	}
	else
	{
	    boolean firstOutputLine = true;  // xxx - changed to false

  try
  {
      String command = GTrace.getGTraceResource("TRACEROUTE")
          + " " + parseDest;
      GTrace.displayTextSBar(command);
      p = Runtime.getRuntime().exec(command);	
      try
      {
          BufferedReader inStream = new BufferedReader(
                                    new InputStreamReader(p.getInputStream()));
		     
          String line;
          boolean flag = true;				
          int temp=99;

          while(flag)
          {	
              try
              {
                  if ((temp=p.exitValue()) == 0)
                  {
                      flag = false;
                  }
              }
              catch(IllegalThreadStateException e)
              {
                  line = inStream.readLine();
		     
                  if ((line == null && firstOutputLine)) 
                  {
                      GTrace.displayErrorDialog("Invalid Destination");
                      return;
                  }
                  else if(line.toLowerCase().indexOf("unknown host") > 0
                          && firstOutputLine)
                  {
                      GTrace.displayTextSBar(line);
                      GTrace.displayErrorDialog("Unknown Host");
                      return;
                  }
                  else
                  {
                      firstOutputLine = false;

                      if (!firstHop && line != null)
                      {
                          lineTokens = Util.tokenizeToString(line);
				     
                          if (lineTokens[0].trim().compareTo("1") == 0)
                          {
                              firstHop = true;
                              GTrace.displayTextSBar(line);
                              new ParseHop(++hNum, line, model,
                                           parseTraceThreadGroup);
                              model.setCurrentHop(hNum,
                                              parseTraceThreadGroup.getName());
                          }
                      }
                      else
                      {
                          if (line != null && !line.equals(""))
                          {
                              lineTokens = Util.tokenizeToString(line);
					 
                              if (lineTokens[0].trim().compareTo(
                                          new Integer(hNum+1).toString()) == 0)
                              {
                                  GTrace.displayTextSBar(line);
                                  new ParseHop(++hNum, line, model,
                                               parseTraceThreadGroup);
                                  model.setCurrentHop(hNum,
                                              parseTraceThreadGroup.getName());
                              }
                          }
                      }

                      try
                      {
                          Thread.sleep(10);
                      }
                      catch(InterruptedException ie)
                      {    						
                          System.out.println("Interrupted Exception 1");
                      }
                  }
              }
          }
		     
          while((line = inStream.readLine())!=null)
          {
              GTrace.displayTextSBar(line);
              new ParseHop(++hNum, line, model, parseTraceThreadGroup);
              model.setCurrentHop(hNum, parseTraceThreadGroup.getName());
          }
      }
      catch (IOException e)
      {
          System.out.println("BufferedReader Failed" + e.getMessage());
      }
  }
	     catch(IOException e)
	     {
		 GTrace.displayTextSBar("Command Failed" + e.getMessage());
		 System.out.println("Command Failed" + e.getMessage());
	     }

	     GTrace.displayTextSBar("Waiting for threads to finish "
                                    + "determining locations");
	     while(!model.isTraceComplete(parseTraceThreadGroup.getName()))
	     {

		 try { Thread.sleep(500); } catch(InterruptedException ie) { }
	     }

	     if (!GTrace.getVerifierStatus())
	     {
		 GTrace.displayTextSBar("Starting Verifier......");
		 new LocationVerifier(model, parseTraceThreadGroup);
	     }
	     else
	     {
		 GTrace.displayTextSBar("Done");
	     }
	 }
     }

    public void killProcess()
    {
	if (t != null)
	{
	    t.stop();
	}
    }

}
