/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    RegexRecord.java
**
**     Purpose:                 Handles the contents of a domain parsing file
**                              lookup. 
**                              
**
**     Author/Date:             Ram Periakaruppan, Oct. 17th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
******************************************************************************/
package caida.GTrace;

import java.io.Serializable;

public class RegexRecord implements Serializable
{
    String regex;         // the regex used for the lookup
    String matchResult;   // the result of the regex on the hostname
    String DPFData;       // data used from the domain parsing file
    DbRecord dbRecord;    // database record if a db was queried


//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables to the
 **                              parameters specified.
 **    
 **     Parameters:              String regex - the regular expression
 **                              String result - the result of match  
 **                              String DPFData - local domain parsing file
 **                                               data used
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/

    public RegexRecord(String regex, String result, String DPFData)
    {
	this.regex = regex;
	matchResult = result;
	this.DPFData = DPFData;
	dbRecord = null;
    }

/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables to the
 **                              parameters specified.
 **    
 **     Parameters:              String regex - the regular expression
 **                              String result - the result of match  
 **                              String DPFData - local domain parsing file
 **                                               data used
 **                              DbRecord record - the record queried
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/
	
    public RegexRecord(String regex, String result, String DPFData,
                       DbRecord record)
    {
	this.regex = regex;
	matchResult = result;
	this.DPFData = DPFData;
	dbRecord = record;
    }

//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Method name:             getRegex
 **
 **     Purpose:                 returns the regular expression used.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - regex 
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/

    public String getRegex()
    {
	return regex;
    }

/******************************************************************************
 **
 **     Method name:             getMatchResult
 **
 **     Purpose:                 returns the result of the regex match
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - the match result
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/

    public String getMatchResult()
    {
	return matchResult;
    }

/******************************************************************************
 **
 **     Method name:             getDPFData
 **
 **     Purpose:                 returns the local domain parsing file data
 **                              used
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - domain parsing file data
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/

    public String getDPFData()
    {
	return DPFData;
    }

/******************************************************************************
 **
 **     Method name:             getDbRecord
 **
 **     Purpose:                 returns the database record queried for the
 **                              location match.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           DbRecord - the database record
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 ******************************************************************************/

    public DbRecord getDbRecord()
    {
	return dbRecord;
    }

/******************************************************************************
 **
 **     Method name:             toString
 **
 **     Purpose:                 returns a string representation of the
 **                              contents of this object.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String 
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 18th, 1999
 **
 ******************************************************************************/

    public String toString()
    {
	String retString;

	retString = new String(
			       "Domain Parsing File Details:\n\n" +
			       "Regex Used\t: " + regex + "\n" +
			       "Match Result\t: " + matchResult + "\n" +
			       "Local Data Used: " + DPFData + "\n\n");

	if (dbRecord != null)
	{
	    retString = retString.concat(dbRecord.toString());
	}
	
	return retString;
    }
	
}
