/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    SaveTrace.java
**
**     Purpose:                 Saves a graphical trace to a file to be
**                              restored for future use.
**
**     Author/Date:             Ram Periakaruppan, Oct. 22nd, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.io.Serializable;
import java.util.Hashtable;
import java.awt.Color;

public class SaveTrace implements Serializable
{
    int rows;              // the number of hops that make up the trace
    String mapName;        /* the name of the map the user was viewing at the
                              time of saving the trace
                           */
    public Hashtable path; // hashtable storing all the details about the path

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables
 **    
 **     Parameters:              String mapName - name of map at point of
 **                                               saving
 **                              GTraceTableModel dataModel - object used to
 **                                    access contents of textual traceroute
 **                                    table.
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public SaveTrace(String mapName, GTraceTableModel dataModel)
    {
        this.mapName = mapName;
        rows = dataModel.getRowCount();
        path = new Hashtable();

/*---------------------------------------------------------------------------
 *  Create a HopData object for each hop and add it to the hashtable with
 *  the hop number as the key.
 *---------------------------------------------------------------------------*/
        
        for (int i=0; i<rows; i++)
        {
            path.put(new Integer(i),
                     new HopData(dataModel.getData(i),
                                 dataModel.getCellColor(i),
                                 dataModel.getLocationColor(i),
                                 dataModel.getLocationDetails(i),
                                 dataModel.getTypeofLOC(i)
                                 )
                     );
        }

    }

//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Method name:             getNumHops
 **
 **     Purpose:                 Returns the number of hops in the saved trace
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           int - number of hops
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public int getNumHops()
    {
        return rows;
    }

/******************************************************************************
 **
 **     Method name:             getRow
 **
 **     Purpose:                 Returns the contents of the traceroute text
 **                              display for the specified row
 **    
 **     Parameters:              int rowNum - the hop for which the search step
 **                                           is to be returned.
 **
 **     Return Values:           Object[] - the six fields of each row
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public Object[] getRow(int rowNum)
    {
        return ((HopData)path.get(new Integer(rowNum))).getRow();
    }

/******************************************************************************
 **
 **     Method name:             getTimeColor
 **
 **     Purpose:                 Returns the color of the time field for the
 **                              specified row
 **    
 **     Parameters:              int rowNum - the hop for which the search step
 **                                           is to be returned.
 **
 **     Return Values:           Color - time color
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/

    public Color getTimeColor(int rowNum)
    {
        return ((HopData)path.get(new Integer(rowNum))).getTimeColor();
    }

/******************************************************************************
 **
 **     Method name:             getLocationColor      
 **
 **     Purpose:                 Returns the color of the location field for 
 **                              the specified row
 **    
 **     Parameters:              int rowNum - the hop for which the search step
 **                                           is to be returned.
 **
 **     Return Values:           Color - Location color
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public Color getLocationColor(int rowNum)
    {
        return ((HopData)path.get(new Integer(rowNum))).getLocationColor();
    }

/******************************************************************************
 **
 **     Method name:             getLocationDetails
 **
 **     Purpose:                 Returns the location detail object for the
 **                              specified row.
 **    
 **     Parameters:              int rowNum - the hop for which the search step
 **                                           is to be returned.
 **
 **     Return Values:           LocationDetails - the location detail object
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public LocationDetails getLocationDetails(int rowNum)
    {
        return ((HopData)path.get(new Integer(rowNum))).getLocationDetails();
    }

/******************************************************************************
 **
 **     Method name:             getStep       
 **
 **     Purpose:                 Returns the search step that produced the
 **                              location for the specified row
 **    
 **     Parameters:              int rowNum - the hop for which the search step
 **                                           is to be returned.
 **
 **     Return Values:           Integer - the search step
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public Integer getStep(int rowNum)
    {
        return ((HopData)path.get(new Integer(rowNum))).getStep();
    }

/******************************************************************************
 **
 **     Method name:             getMapName
 **
 **     Purpose:                 Returns the name of the map that was showing
 **                              the trace at the point of saving.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           String - name of map
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
    
    public String getMapName()
    {
        return mapName;
    }

//---------------------------------------------------------------------------
//                             inner classes
//---------------------------------------------------------------------------

    
public class HopData implements Serializable
{
    Object[] row = new Object[6];  /* contains the values stored in the fields
                                      of each row
                                   */
    Color time_color;              // the color of the RTT value
    Color location_color;          // the color of the location
    LocationDetails lDetail;       /* the Location Detail object maintaining
                                      history of how the location was obtained
                                   */
    Integer step;                  /* the search step that produced the
                                      location
                                   */

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------

    
/******************************************************************************
 **
 **     Purpose:                 Initializes the member variables
 **    
 **     Parameters:              Object[] row - same as described above
 **                              Color time_c - same as described above
 **                              Color location_c - same as described above
 **                              LocationDetails lDetail - same as above
 **                              Integer step - same as described above
 **
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public HopData(Object[] row, Color time_c,
                   Color location_c, LocationDetails lDetail,
                   Integer step)
    {
        this.row = row;
        time_color = time_c;
        location_color = location_c;
        this.lDetail = lDetail;
        this.step = step;
    }

//---------------------------------------------------------------------------
//                             getxxx Methods
//---------------------------------------------------------------------------

    
/******************************************************************************
 **
 **     Method name:             getRow
 **
 **     Purpose:                 Returns the contents of each row of the
 **                              traceroute text display
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           Object[] - the six fields of each row
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public Object[] getRow()
    {
        return row;
    }

/******************************************************************************
 **
 **     Method name:             getTimeColor
 **
 **     Purpose:                 Returns the color of the time field
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           Color - time color
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public Color getTimeColor()
    {
        return time_color;
    }

/******************************************************************************
 **
 **     Method name:             getLocationColor      
 **
 **     Purpose:                 Returns the color of the location field
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           Color - Location color
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public Color getLocationColor()
    {
        return location_color;
    }

/******************************************************************************
 **
 **     Method name:             getLocationDetails
 **
 **     Purpose:                 Returns the location detail object.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           LocationDetails - the location detail object
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public LocationDetails getLocationDetails()
    {
        return lDetail;
    }

/******************************************************************************
 **
 **     Method name:             getStep       
 **
 **     Purpose:                 Returns the search step that produced the
 **                              location
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           Integer - the search step
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 22nd, 1999
 **
 *****************************************************************************/
        
    public Integer getStep()
    {
        return step;
    }
}

}
