/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    Util.java
**
**     Purpose:                 Provides common utility methods that are
**                              accessed by most other GTrace classes.
**
**     Author/Date:             Ram Periakaruppan, Sept 5th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.util.StringTokenizer;
import java.util.Vector;

public final class Util
{

/******************************************************************************
 **
 **     Method name:             tokenizeToString
 **
 **     Purpose:                 tokenizes a string and returns the tokens
 **                              using space as the delimiter
 **    
 **     Parameters:              String data - the stirng to be tokenized
 **  
 **     Return Values:           String[] - the tokens
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
 **
 *****************************************************************************/

    public static String[] tokenizeToString(String data) 
    {
	return tokenizeToString(data,null);
    }

/******************************************************************************
 **
 **     Method name:             tokenizeToString
 **
 **     Purpose:                 tokenizes a string and returns the the tokens
 **                              using the delimeter specified.
 **    
 **     Parameters:              String data  - the string to be tokenized.
 **                              String seperator - the delimiter
 **
 **     Return Values:           String[] - the tokens
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
 **
 *****************************************************************************/

    public static String[] tokenizeToString(String data,String seperator) 
    {
	Vector v = new Vector();
	StringTokenizer tokenizer;

	if (data == null)
	{
	    return null;
	}

	if (seperator == null)
	{
	    tokenizer = new StringTokenizer(data);
	}
	else
	{
	    tokenizer = new StringTokenizer(data,seperator);
	}

	while(tokenizer.hasMoreTokens())
	{
	    v.addElement(tokenizer.nextElement());
	}
	
	String retStr[] = new String[v.size()];
	for (int i=0; i<retStr.length; i++)
	{
	    retStr[i] = (String)v.elementAt(i);
	}
	
	return retStr;
    }

/******************************************************************************
 **
 **     Method name:             tokenizeTofloat
 **
 **     Purpose:                 tokenizes a string into float values using
 **                              space as a delimiter
 **    
 **     Parameters:              String data - the string to be tokenized.
 **
 **     Return Values:           float[] - the tokens
 **
 **     Side Effects:            throws NumberFormatException - occurs when
 **                              the string contains an invalid charater that
 **                              can't be converted to float
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
 **
 *****************************************************************************/

    public static float[] tokenizeTofloat(String data) 
        throws NumberFormatException
    {
	return tokenizeTofloat(data,null);
    }

/******************************************************************************
 **
 **     Method name:             tokenizeTofloat
 **
 **     Purpose:                 tokenizes a string into float values using
 **                              the delimeter specified.
 **    
 **     Parameters:              String data  - the string to be tokenized.
 **                              String seperator - the delimiter
 **
 **     Return Values:           float[] - the tokens
 **
 **     Side Effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to float
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 5th, 1999
 **
 *****************************************************************************/

    public static float[] tokenizeTofloat(String data,String seperator) 
        throws NumberFormatException
    {
	Vector v = new Vector();
	StringTokenizer tokenizer;
	if (seperator == null)
	{
	    tokenizer = new StringTokenizer(data);
	}
	else
	{
	    tokenizer = new StringTokenizer(data,seperator);
	}

	while(tokenizer.hasMoreTokens())
	{
	    v.addElement(tokenizer.nextElement());
	}
	
	float floatElements[] = new float[v.size()];
	for (int i=0; i<floatElements.length; i++)
	{
	    floatElements[i] = new Float((String)v.elementAt(i)).floatValue();
	}
	
	return floatElements;
    }

/******************************************************************************
 **
 **     Method name:             tokenizeToint
 **
 **     Purpose:                 tokenizes a string into int values using
 **                              the delimeter specified.
 **    
 **     Parameters:              String data  - the string to be tokenized.
 **                              String seperator - the delimiter
 **
 **     Return Values:           int[] - the tokens
 **
 **     Side Effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to int
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static int[] tokenizeToint(String data,String seperator) 
        throws NumberFormatException
    {
	Vector v = new Vector();
	StringTokenizer tokenizer;
	if (seperator == null)
	{
	    tokenizer = new StringTokenizer(data);
	}
	else
	{
	    tokenizer = new StringTokenizer(data,seperator);
	}

	while(tokenizer.hasMoreTokens())
	{
	    v.addElement(tokenizer.nextElement());
	}
	
	int intElements[] = new int[v.size()];
	for (int i=0; i<intElements.length; i++)
	{

            intElements[i] = new Integer((String)v.elementAt(i)).intValue();
	}
	
	return intElements;
    }


/******************************************************************************
 **
 **     Method name:             stringTofloat
 **
 **     Purpose:                 converts a string to a float
 **    
 **     Parameters:              String data - the string to be converted
 ** 
 **     Return Values:           float - the float equivalent
 **                                           
 **     Side effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to float
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public static float stringTofloat(String data) throws NumberFormatException
    {
	return new Float(data).floatValue();
    }

/******************************************************************************
 **
 **     Method name:             stringToint 
 **
 **     Purpose:                 converts a string to a int
 **    
 **     Parameters:              String data - the string to be converted
 **
 **     Return Values:           int - the int equivalent
 **                                           
 **     Side effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to int
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 3th, 1999
 **
 *****************************************************************************/

    public static int stringToint(String data) throws NumberFormatException
    {
	return new Integer(data).intValue();
    }

/******************************************************************************
 **
 **     Method name:             stringToLat
 **
 **     Purpose:                 converts a string representation of a
 **                              latitude value to its float equivalent
 **    
 **     Parameters:              String lat - the latitude to be converted
 **
 **     Return Values:           float - the float equivalent
 **                                           
 **     Side effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to float
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static float stringToLat(String lat) throws NumberFormatException
    {
	StringTokenizer tokenizer = new StringTokenizer(lat);
	  
	if (lat.endsWith("N")) 
	{
	    return stringTofloat( tokenizer.nextToken() );
	}
	else
	{
	    return ( stringTofloat( tokenizer.nextToken() ) * -1 );
	}
    }

/******************************************************************************
 **
 **     Method name:             stringToLon
 **
 **     Purpose:                 converts a string representation of a
 **                              longitude value to its float equivalent
 **    
 **     Parameters:              String lon - the longitude to be converted
 **
 **     Return Values:           float - the float equivalent
 **                                           
 **     Side effects:            throws NumberFormatException - is thrown when
 **                              the string contains an invalid charater that
 **                              can't be converted to float
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static float stringToLon(String lon) throws NumberFormatException
    {
	StringTokenizer tokenizer = new StringTokenizer(lon);
	if (lon.endsWith("E")) 
	{
	    return stringTofloat( tokenizer.nextToken() );
	}
	else
	{
	    return ( stringTofloat( tokenizer.nextToken() ) * -1 );
	}
    }

/******************************************************************************
 **
 **     Method name:             roundNum
 **
 **     Purpose:                 rounds of a number to two decimal places
 **    
 **     Parameters:              float d - the number to be rounded
 **
 **     Return Values:           String - rounded number is returned as a
 **                                       string.
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static String roundNum(float d)
    {
        int i;
        d = Math.abs(d);
        i = (int)(d*100.00)%100;
        return (int)d + "." + i;
    }


/******************************************************************************
 **
 **     Method name:             comparelon  
 **
 **     Purpose:                 compares two longitude values returning the
 **                              sum if both are not on the same east/west
 **                              side
 **    
 **     Parameters:              String lon1 - longitude 1
 **                              String lon2 - longitude 2 
 **
 **     Return Values:           int - the sum if the two longitudes are east
 **                                    and west, 0 otherwise
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static int comparelon(String lon1, String lon2) 
    {
	if (lon1.charAt(lon1.length()-1) != lon2.charAt(lon2.length()-1))
	{
	    return new Float(stringTofloat(
                             lon1.substring(0,lon1.trim().lastIndexOf(' ')+1)) 
			     + stringTofloat(
                             lon2.substring(0,lon2.trim().lastIndexOf(' ')+1))
                             ).intValue();
	}
	
	return 0;
    }
   
/******************************************************************************
 **
 **     Method name:             difflon  
 **
 **     Purpose:                 returns the difference between two longitude
 **                              values.
 **    
 **     Parameters:              String lon1 - longitude 1
 **                              String lon2 - longitude 2  
 **
 **     Return Values:           float - the difference
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static float difflon(String lon1, String lon2) 
        throws NumberFormatException
    {
        if (lon1.charAt(lon1.length()-1) == lon2.charAt(lon2.length()-1))
        {
            return Math.abs(stringTofloat(
                            lon1.substring(0,lon1.trim().lastIndexOf(' ')))
                            - stringTofloat(
                            lon2.substring(0,lon2.trim().lastIndexOf(' '))));
        }
        else
        {
            return (360 - (stringTofloat(
                           lon1.substring(0,lon1.trim().lastIndexOf(' '))) 
                           + stringTofloat(
                           lon2.substring(0,lon2.trim().lastIndexOf(' ')))));
        }
    }

/******************************************************************************
 **
 **     Method name:             difflat  
 **
 **     Purpose:                 returns the difference between two latitude
 **                              values.
 **    
 **     Parameters:              String lat1 - latitude 1
 **                              String lat2 - latitude 2  
 **
 **     Return Values:           float - the difference
 **
 **     Author/Last Modified:    Ram Periakaruppan, May 12th, 1999
 **
 *****************************************************************************/

    public static float difflat(String lat1, String lat2) 
        throws NumberFormatException
    {
	if (lat1.charAt(lat1.length()-1) == lat2.charAt(lat2.length()-1))
	{
	    return Math.abs(stringTofloat(
                            lat1.substring(0,lat1.trim().lastIndexOf(' ')))
			    - stringTofloat(
                            lat2.substring(0,lat2.trim().lastIndexOf(' '))));
	}
        else
	{
	    return (stringTofloat(
                    lat1.substring(0,lat1.trim().lastIndexOf(' ')))
		    + stringTofloat(
                    lat2.substring(0,lat2.trim().lastIndexOf(' '))));
	}
    }


/******************************************************************************
 **
 **     Method name:             latlonToString
 **
 **     Purpose:                 converts float values of lat and lon to a
 **                              their string equivalent.
 **    
 **     Parameters:              float lat - latitude
 **                              float lon - longitude
 **
 **     Return Values:           String - the string equivalent
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static String latlonToString(float lat, float lon)
    {
	return roundNum(lat) + ((lat < 0.0) ? " S" :" N") 
            + ", " + roundNum(lon) 
            + ((lon < 0.0) ? " W" :" E");
    }

/******************************************************************************
 **
 **     Method name:             latlonToString
 **
 **     Purpose:                 converts a latlon object to a
 **                              its string equivalent.
 **    
 **     Parameters:              LatLon latlon
 **
 **     Return Values:           String - the string equivalent
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static String latlonToString(LatLon latlon)
    {
	return latlonToString(latlon.getLat(), latlon.getLon());
    }

/******************************************************************************
 **
 **     Method name:             toUpperCase
 **
 **     Purpose:                 converts the first character of each word to
 **                              uppercase and the remaining are in losercase.
 **    
 **     Parameters:              String city - the string to be converted
 **
 **     Return Values:           String - the converted string.
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/

    public static String toUpperCase(String city)
    {
	String upperCaseCity;
	String cityTokens[] = tokenizeToString(city);

	for (int i=0; i<cityTokens.length; i++)
	{
	    cityTokens[i] = cityTokens[i].toLowerCase();
	    char cityChars[] = cityTokens[i].toCharArray();
	    cityChars[0] = Character.toUpperCase(cityChars[0]);
	    cityTokens[i] = new String(cityChars);
	}

	upperCaseCity = cityTokens[0];
	for (int i=1; i<cityTokens.length; i++)
	{
	    upperCaseCity = upperCaseCity.concat(" " + cityTokens[i]);
	}

	return upperCaseCity;
    }

/******************************************************************************
 **
 **     Method name:             formatCityLOC
 **
 **     Purpose:                 formats a city,state,country to a standard
 **                              format.
 **    
 **     Parameters:              String city - the string to be formatted
 **
 **     Return Values:           String - the formatted string.
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 17th, 1999
 **
 *****************************************************************************/
    
    public static String formatCityLOC(String city)
    {
	String formattedCity = new String("UNKNOWN");
	city = city.replace('$',' ');
	String tokens[] = tokenizeToString(city,",");
	
	if (tokens.length == 3)
	{
	    tokens[0] = toUpperCase(tokens[0]);
	    if (tokens[1].trim().length() == 2)
	    {
		tokens[1] = tokens[1].toUpperCase();
 	    }
	    else
	    {
		tokens[1] = toUpperCase(tokens[1]);
	    }
	    if (tokens[2].trim().length() == 2
                || tokens[2].trim().toLowerCase().compareTo("usa") == 0)
	    {
		tokens[2] = tokens[2].toUpperCase();
	    }
	    else
	    {
		tokens[2] = toUpperCase(tokens[2]);
	    }
	    
	    formattedCity = new String(tokens[0] + "," + tokens[1]
                                       + "," + tokens[2]);
	}
	else if (tokens.length == 2)
	{
	    tokens[0] = toUpperCase(tokens[0]);
	    if (tokens[1].trim().length() == 2
                || tokens[1].trim().toLowerCase().compareTo("usa") == 0)
	    {
		tokens[1] = tokens[1].toUpperCase();
 	    }
	    else
	    {
		tokens[1] = toUpperCase(tokens[1]);
	    }

	    formattedCity = new String(tokens[0] + "," + tokens[1]);
	}
	else if (tokens.length == 1)
	{
	    if ( tokens[0].trim().toLowerCase().compareTo("usa") == 0 ||
		 tokens[0].length() == 2)
	    {
		formattedCity = tokens[0].toUpperCase();
	    }
	    else
	    {
		formattedCity = toUpperCase(tokens[0]);
	    }
        }
	

	return formattedCity;
    }

/******************************************************************************
 **
 **     Method name:             ipaddrToString
 **
 **     Purpose:                 removes all extra characters like '(' returned
 **                              by traceroute and returns a string of just
 **                              the four octets of the IP address.
 **    
 **     Parameters:              String ipaddr - the IP address to be formatted
 **
 **     Return Values:           String - the fomatted IP address
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static String ipaddrToString(String ipaddr)
    {
	StringBuffer temp = new StringBuffer(ipaddr);
	int length = ipaddr.length();

	for (int i=0; i<length; i++)
	{
	    if (!Character.isDigit(ipaddr.charAt(i))
                && ipaddr.charAt(i) != '.')
	    {
		temp.setCharAt(i,' ');
	    }
	}

	ipaddr = temp.toString().trim();

	try
	{
	    tokenizeToint(ipaddr,".");
	}
	catch(NumberFormatException nfe) 
	{
	    return null;
	}

	return ipaddr;
    }

/******************************************************************************
 **
 **     Method name:             getDomain
 **
 **     Purpose:                 
 **    
 **     Parameters:              String hostname
 **                              int domainnum
 **
 **     Return Values:           String 
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static String getDomain(String hostname, int domainnum)
    {
	if (hostname.length() > 0)
	{
	    String domains[] = tokenizeToString(hostname,".");
	    if (domains.length-domainnum >= 0)
	    {
		if (Character.isDigit(domains[domains.length-(
                                                        domainnum)].charAt(0)))
		{
		    return null;
		}
		
		return domains[domains.length-(domainnum)];
	    }
	}
	return null;
    }

/******************************************************************************
 **
 **     Method name:             getDomain
 **
 **     Purpose:                 returns the first level domain of a hostname
 **    
 **     Parameters:              String hostname
 **
 **     Return Values:           String - the first level domain
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 15th, 1999
 **
 *****************************************************************************/

    public static String getDomain(String hostname)
    {
	if (hostname.length() > 0)
	{
	    String domains[] = tokenizeToString(hostname,".");
	    if (domains.length > 2)
	    {
                //international site
		if (domains[domains.length-1].length() == 2) 
		{
		    return domains[domains.length-3] + "." +
                        domains[domains.length-2] + "." +
                        domains[domains.length-1];
		}
		else if (domains[domains.length-1].length() == 3)
		{
		    return domains[domains.length-2] + "." +
                        domains[domains.length-1];			   
		}
	    }
            else
            {
                return null;
            }
	    return hostname;
	}
	return null;
    }

/******************************************************************************
 **
 **     Method name:             degminsecTofloat
 **
 **     Purpose:                 converts a degree minute second representation
 **                              to its float equivalent.
 **    
 **     Parameters:              String deg - degrees
 **                              String min - minutes
 **                              String sec - seconds
 **                              String dir - direction
 **
 **     Return Values:           float - the float equivalent
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 16th, 1999
 **
 *****************************************************************************/

    public static float degminsecTofloat(String deg, String min,
                                         String sec, String dir)
    {
	float latlon = stringTofloat(deg) + (stringTofloat(min))/60
                       + (stringTofloat(sec))/3600;
	
	if (dir.toLowerCase().compareTo("s") == 0
            || dir.toLowerCase().compareTo("w") == 0)
	{
	    latlon *= -1;
	}

	return latlon;
    }

/******************************************************************************
 **
 **     Method name:             getNumLinesMaxWidth
 **
 **     Purpose:                 returns the number of lines in a string and
 **                              max width of any line.
 **    
 **     Parameters:              String text 
 **
 **     Return Values:           int[] - the number of lines and max width
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Oct. 18th, 1999
 **
 *****************************************************************************/

    public static int[] getNumLinesMaxWidth(String text)
    {
	char chars[] = text.toCharArray();

	int numLines=0;
	int maxWidth=0;
	int tmpWidth=0;
	for (int i=0; i<chars.length; i++)
	{
	    if (chars[i] == '\n')
	    {
		numLines++;
		if (tmpWidth > maxWidth)
		{
		    maxWidth = tmpWidth;
		}
		tmpWidth = 0;
	    }
	    else
	    {
		tmpWidth++;
	    }
	}

	return new int[] {numLines,maxWidth};
    }
}

