/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    nslookup.java
**
**     Purpose:                 
**                              
**
**     Author/Date:             Ram Periakaruppan, Sept. 10th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class nslookup extends JDialog implements ActionListener, Runnable
{
    JDialog nslookupDialog;     //the dialog
    JTextArea textArea;         //text area displaying output of nslookup
    JTextField hostField;       // name of host/ipaddress
    JTextField optionsField;
    JButton cancel;

    private Thread t = null;
    private Process p;

//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Purpose:                Constructs and implements the nslookup dialog 
 **                             that displays dns records for hostnames/
 **                             ipaddrs. 
 **    
 **     Parameters:             JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 10th, 1999
 **
 *****************************************************************************/

    public nslookup(JFrame frame) 
    {
/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
	nslookupDialog = new JDialog(frame, "nslookup", false);
	nslookupDialog.setSize(410, 460);
	nslookupDialog.setResizable(false);

	JPanel nslookup = new JPanel();
	nslookup.setLayout(new BorderLayout());
	nslookup.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	nslookup.setMinimumSize(new Dimension(410,460));
	nslookup.setPreferredSize(new Dimension(410,460));
	nslookup.setMaximumSize(new Dimension(410,460));

	textArea = new JTextArea(15,28);
	textArea.setEditable(false);
        textArea.setLineWrap(false);
	JScrollPane scrollPane = new JScrollPane(textArea);
        scrollPane.setVerticalScrollBarPolicy(
                                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setHorizontalScrollBarPolicy(
                                      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);


	JPanel bottomPanel = new JPanel();
        bottomPanel.setBorder(BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
                                                                 "Output"),
                                 BorderFactory.createEmptyBorder(5,5,5,5)));
        bottomPanel.add(scrollPane);
	
	hostField = new JTextField(18);
	hostField.setMinimumSize(new Dimension(200,25));
	hostField.setPreferredSize(new Dimension(200,25));
	hostField.setMaximumSize(new Dimension(200,25));
	hostField.addActionListener(this);

	JPanel hostPanel = new JPanel();
        hostPanel.setBorder(BorderFactory.createCompoundBorder(
                              BorderFactory.createTitledBorder(
                                                               "Host"),
                              BorderFactory.createEmptyBorder(5,5,5,5)));
        hostPanel.add(hostField);

	optionsField = new JTextField(18);
	optionsField.setMinimumSize(new Dimension(200,25));
	optionsField.setPreferredSize(new Dimension(200,25));
	optionsField.setMaximumSize(new Dimension(200,25));
	optionsField.addActionListener(this);

	JPanel optionsPanel = new JPanel();
        optionsPanel.setBorder(BorderFactory.createCompoundBorder(
                                 BorderFactory.createTitledBorder(
                                                                  "Options"),
                                 BorderFactory.createEmptyBorder(5,5,5,5)));
        optionsPanel.add(optionsField);
	
/*---------------------------------------------------------------------------
 *  create the cancel button and add a action listener that will dispose the
 *  dialog if clicked on.
 *---------------------------------------------------------------------------*/
        
	cancel = new JButton("Cancel");
	cancel.setMnemonic('C');
	cancel.setBorder(BorderFactory.createRaisedBevelBorder());
	cancel.setMinimumSize(new Dimension(100,30));
	cancel.setPreferredSize(new Dimension(100,30));
	cancel.setMaximumSize(new Dimension(100,30));
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    nslookupDialog.setVisible(false);
                    nslookupDialog.dispose();
                }
	});
	cancel.setAlignmentX(Component.CENTER_ALIGNMENT);    


	JPanel leftTopPanel = new JPanel();
        leftTopPanel.setLayout(new BoxLayout(leftTopPanel, BoxLayout.Y_AXIS));
	leftTopPanel.setAlignmentX(0);
        leftTopPanel.add(hostPanel);
        leftTopPanel.add(Box.createRigidArea(new Dimension(0,5)));
	leftTopPanel.add(optionsPanel);

	JPanel rightTopPanel = new JPanel();
        rightTopPanel.setLayout(new BoxLayout(rightTopPanel,
                                              BoxLayout.Y_AXIS));
        rightTopPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	rightTopPanel.add(cancel);

	JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
	topPanel.setAlignmentX(0);
        topPanel.add(leftTopPanel);
        topPanel.add(Box.createRigidArea(new Dimension(40,0)));
	topPanel.add(rightTopPanel);

        nslookup.add(topPanel,BorderLayout.NORTH);
        nslookup.add(Box.createRigidArea(new Dimension(0,10)));
        nslookup.add(bottomPanel,BorderLayout.SOUTH);

/*---------------------------------------------------------------------------
 *  add the add nslookup panel to the content pane of the dialog and set its
 *  postion relative to the parent frame and display the dialog.
 *---------------------------------------------------------------------------*/
        
	nslookupDialog.getContentPane().add(nslookup);
	nslookupDialog.setLocationRelativeTo(frame);
	nslookupDialog.setVisible(true);
    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                          
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
 **
 ****************************************************************************/

    public void actionPerformed(ActionEvent e)
    {
	t = new Thread(this, "Nslookup");
	t.start();
    }

/******************************************************************************
 **
 **     Method name:             run
 **
 **     Purpose:                 method is called when thread t is started.
 **                              Executes nslookup to the specified host and
 **                              displays the output in the text area.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
 **
 *****************************************************************************/
    
    public void run()
    {
	try
	{
	    if (hostField.getText().length() > 0)
	    {
		if (optionsField.getText().length() > 0)
		{
		    p = Runtime.getRuntime().exec(
                                          GTrace.getGTraceResource("NSLOOKUP") 
					  + " " + optionsField.getText() 
					  + " " + hostField.getText());
		}
		else
		{
		    p = Runtime.getRuntime().exec(
                                          GTrace.getGTraceResource("NSLOOKUP") 
					  + " " + hostField.getText());
		}

		BufferedReader inStream = new BufferedReader(
                                    new InputStreamReader(p.getInputStream()));
		String str;
		boolean flag = true;				
		int temp=99;	
		textArea.setText("");	

		while(flag)
		{	
		    try
		    {
			if ((temp=p.exitValue()) == 0)
			{
			    flag = false;
			}
		    }
		    catch(IllegalThreadStateException e)
		    {

			str=inStream.readLine();

			if (str == null)
			{
			    textArea.append("Error");
			    return;
			}
			textArea.append(str+"\n");
		    }
		}

		boolean TRUE = true;
		while(TRUE)
	        {
		    str = inStream.readLine();
		    
		    if (str == null)
		    {
			return;
		    }
		    textArea.append(str+"\n");
		}
	    }
	}
	catch(IOException io)
	{
	    System.out.println("Io exception");
	}		       
    }
}
