/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    ping.java
**
**     Purpose:                 Constructs and implements the whois dialog that
**                              displays whois records for hostnames/ipaddrs.
**
**     Author/Date:             Ram Periakaruppan, Sept. 13th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class ping extends JDialog implements ActionListener, Runnable
{
    private JDialog pingDialog;       // the dialog
    private JTextArea textArea;       // text area that displays output
    private JTextField hostField;     // name of host to ping
    private JTextField optionsField;  // options to ping
    private JButton stop;             // stop button
    private JButton cancel;           // cancel button

    private Thread t = null;   // thread that is spawned to executed ping
    private Process p;         // process that starts ping

    // The width and height of the dialog    
    public static int WIDTH = 410;
    public static int HEIGHT = 460;
    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Purpose:                 
 **    
 **     Parameters:             
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 10th, 1999
 **
 *****************************************************************************/

    public ping(JFrame frame) 
    {

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
	pingDialog = new JDialog(frame, "ping", false);
	pingDialog.setSize(WIDTH, HEIGHT);
	pingDialog.setResizable(false);

	JPanel ping = new JPanel();
	ping.setLayout(new BorderLayout());
	ping.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
        setMPMSize(ping,new Dimension(WIDTH,HEIGHT));

	textArea = new JTextArea(15,28);
	textArea.setEditable(false);
        textArea.setLineWrap(false);
	JScrollPane scrollPane = new JScrollPane(textArea);
        scrollPane.setVerticalScrollBarPolicy(
                                      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        scrollPane.setHorizontalScrollBarPolicy(
                                    JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

	JPanel bottomPanel = new JPanel();
        bottomPanel.setBorder(BorderFactory.createCompoundBorder(
                                BorderFactory.createTitledBorder(
                                "Output"),
                                BorderFactory.createEmptyBorder(5,5,5,5)));
        bottomPanel.add(scrollPane);
	
	hostField = new JTextField(18);
        setMPMSize(hostField,new Dimension(200,25));
	hostField.addActionListener(this);

	JPanel hostPanel = new JPanel();
        hostPanel.setBorder(BorderFactory.createCompoundBorder(
                              BorderFactory.createTitledBorder(
                              "Host"),
                              BorderFactory.createEmptyBorder(5,5,5,5)));
        hostPanel.add(hostField);

	optionsField = new JTextField(18);
        setMPMSize(optionsField,new Dimension(200,25));
	optionsField.addActionListener(this);

	JPanel optionsPanel = new JPanel();
        optionsPanel.setBorder(BorderFactory.createCompoundBorder(
                                 BorderFactory.createTitledBorder(
                                 "Options"),
                                 BorderFactory.createEmptyBorder(5,5,5,5)));
        optionsPanel.add(optionsField);
	
/*---------------------------------------------------------------------------
 *  create the stop button and add a action listener that will kill the 
 *  tread if its not already completed.
 *---------------------------------------------------------------------------*/
        
	stop = new JButton("Stop");
	stop.setMnemonic('S');
	stop.setBorder(BorderFactory.createRaisedBevelBorder());
        setMPMSize(stop,new Dimension(100,30));
	stop.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    if (t != null)
                    {
                        t.stop();

                        if (p != null)
                        {
                            p.destroy();
                        }
                    }
                }
	});
	stop.setAlignmentX(Component.CENTER_ALIGNMENT);    

/*---------------------------------------------------------------------------
 *  create the cancel button and add a action listener that will dispose the
 *  dialog if clicked on.
 *---------------------------------------------------------------------------*/
        
	cancel = new JButton("Cancel");
	cancel.setMnemonic('C');
	cancel.setBorder(BorderFactory.createRaisedBevelBorder());
        setMPMSize(cancel,new Dimension(100,30));
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    pingDialog.setVisible(false);
                    pingDialog.dispose();
                }
	});
	cancel.setAlignmentX(Component.CENTER_ALIGNMENT);    


	JPanel leftTopPanel = new JPanel();
        leftTopPanel.setLayout(new BoxLayout(leftTopPanel, BoxLayout.Y_AXIS));
	leftTopPanel.setAlignmentX(0);
        leftTopPanel.add(hostPanel);
        leftTopPanel.add(Box.createRigidArea(new Dimension(0,5)));
	leftTopPanel.add(optionsPanel);

	JPanel rightTopPanel = new JPanel();
        rightTopPanel.setLayout(new BoxLayout(rightTopPanel,
                                              BoxLayout.Y_AXIS));
        rightTopPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	rightTopPanel.add(stop);
        rightTopPanel.add(Box.createRigidArea(new Dimension(0,10)));
	rightTopPanel.add(cancel);


	JPanel topPanel = new JPanel();
        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
	topPanel.setAlignmentX(0);
        topPanel.add(leftTopPanel);
        topPanel.add(Box.createRigidArea(new Dimension(40,0)));
	topPanel.add(rightTopPanel);

        ping.add(topPanel,BorderLayout.NORTH);
        ping.add(Box.createRigidArea(new Dimension(0,15)));
        ping.add(bottomPanel,BorderLayout.SOUTH);

/*---------------------------------------------------------------------------
 *  add the add ping panel to the content pane of the dialog and set its
 *  postion relative to the parent frame and display the dialog.
 *---------------------------------------------------------------------------*/
        
	pingDialog.getContentPane().add(ping);
	pingDialog.setLocationRelativeTo(frame);
	pingDialog.setVisible(true);
    }

/******************************************************************************
 **
 **     Method name:             actionPerformed
 **
 **     Purpose:                 is the action listener for components that 
 **                              have registered this method as their action 
 **                              listener
 **                       
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                          
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
 **
 *****************************************************************************/

    public void actionPerformed(ActionEvent e)
    {
	t = new Thread(this, "Ping");
	t.start();
    }

/******************************************************************************
 **
 **     Method name:             run
 **
 **     Purpose:                 method is called when thread t is started.
 **                              Executes ping to the specified host and
 **                              displays the output in the text area.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 13th, 1999
 **
 *****************************************************************************/

    public void run()
    {
	try
	{
	    if (hostField.getText().length() > 0)
	    {
		if (optionsField.getText().length() > 0)
		{
		    p = Runtime.getRuntime().exec(
                                             GTrace.getGTraceResource("PING") 
					     + " " + optionsField.getText() 
					     + " " + hostField.getText());
		}
		else
		{
		    p = Runtime.getRuntime().exec(
                                             GTrace.getGTraceResource("PING") 
					     + " " + hostField.getText());
		}

		BufferedReader inStream = new BufferedReader(
                                       new InputStreamReader(
                                                          p.getInputStream()));
		String str;
		boolean flag = true;				
		int temp=99;	
		textArea.setText("");	

		while(flag)
		{	
		    try
		    {
			if ((temp=p.exitValue()) == 0)
			{
			    flag = false;
			}
		    }
		    catch(IllegalThreadStateException e)
		    {

			str=inStream.readLine();

			if (str == null)
			{
			    textArea.append("Error");
			    return;
			}
			textArea.append(str+"\n");
		    }
		}

		boolean TRUE = true;
		while(TRUE)
	        {
		    str = inStream.readLine();
		    
		    if (str == null)
		    {
			return;
		    }
		    textArea.append(str+"\n");
		}

	    }
	}
	catch(IOException io)
	{
	    System.out.println("Io exception");
	}		       
    }

/******************************************************************************
 **
 **     Method name:             setMPMSize
 **
 **     Purpose:                 Sets the Minimum, Preferred and Maximimum
 **                              sizes for a component
 **    
 **     Parameters:              JComponent component - the component whose
 **                                       size attributes are to be set.
 **                              Dimension d - the dimension to which they are
 **                                       to be set.
 **                              
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/

    public void setMPMSize(JComponent component, Dimension d)
    {
	component.setMinimumSize(d);
	component.setPreferredSize(d);
	component.setMaximumSize(d);
    }	
}
