/***************************************************************************
Copyright 1999 The Regents of the University of California
All Rights Reserved

Permission to use, copy, modify and distribute any part of the
GTrace tool for educational, research and non-profit
purposes, without fee, and without a written agreement is hereby
granted, provided that the above copyright notice, this paragraph
and the following paragraphs appear in all copies.

Those desiring to incorporate this into commercial products or use
for commercial purposes should contact the Technology Transfer
Office, University of California, San Diego, 9500 Gilman Drive, La
Jolla, CA 92093-0910, Ph: (619) 534-5815, FAX: (619) 534-7345.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
SOFTWARE, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
THE POSSIBILITY OF SUCH DAMAGE.

THE SOFTWARE PROVIDED HEREIN IS ON AN "AS IS" BASIS, AND THE
UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE UNIVERSITY
OF CALIFORNIA MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES
OF ANY KIND, EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED
TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE, OR THAT THE USE OF THE SOFTWARE WILL NOT INFRINGE
ANY PATENT, TRADEMARK OR OTHER RIGHTS.

The GTrace tool is developed by Ram Periakaruppan at the University 
of California, San Diego under the Cooperative Association for Internet 
Data Analysis (CAIDA) Program. Support for this effort is provided by 
the CAIDA grant ANI-9711092, and by CAIDA members.

Report bugs and suggestions to gtrace-bugs@caida.org.
**************************************************************************/

/******************************************************************************
**
**     File:                    whois.java
**
**     Purpose:                 Constructs and implements the whois dialog that
**                              displays whois records for hostnames/ipaddrs.
**
**     Author/Date:             Ram Periakaruppan, Sept. 9th, 1999.
**
**     References:
**      Naughton, P., Schildt, H., JAVA - The Complete Reference. McGraw Hill.
**      Eckstein, R., Loy, M., Wood, D., JAVA Swing. O'Reilly.
**      The Source For Java Technology, http://www.java.sun.com.
**
*****************************************************************************/
package caida.GTrace;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

public class whois extends JDialog implements ActionListener
{
    JDialog whoisDialog;     //the dialog
    JTextArea textArea;      //text area displaying whois record
    JTextField nameField;    /*name of host/ipaddress for which whois info
                               is to be displayed
                             */
    JTextField asField;      /*name of autonomous system for which whois 
                               info is to be displayed
                             */
    JComboBox hostCombo;     // whois server 
    JButton showWhoisRecord; // show whois button

    Process p;   // Process that is spawned to execute whois

    // The width and height of the frame
    public static int WIDTH = 600;
    public static int HEIGHT = 370;

    // Constants
    public final String Cancel = new String("Cancel");
    public final String showRec = new String("Show whois Record");
    
//---------------------------------------------------------------------------
//                             constructors
//---------------------------------------------------------------------------
    
/******************************************************************************
 **
 **     Purpose:                 Invokes the overloaded construtor setting the
 **                              name of the host to be looked up to null.
 **    
 **     Parameters:              JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 10th, 1999
 **
 *****************************************************************************/

    public whois(JFrame frame)
    {
	this(frame, null);
    }

/******************************************************************************
 **
 **     Purpose:                 Constructs and laysout components in dialog
 **                              setting the name of the host to be looked up.
 **    
 **     Parameters:              JFrame frame - the parent frame on which the
 **                                             dialog is displayed on.
 **                              String name - the host/ipaddr to be looked up
 **
 **     Author/Last Modified:    Ram Periakaruppan, Sept 10th, 1999
 **
 *****************************************************************************/

    public whois(JFrame frame, String name)
    {

/*---------------------------------------------------------------------------
 *  Instantiate the dialog making it non-modal
 *---------------------------------------------------------------------------*/
        
	whoisDialog = new JDialog(frame, "whois", false);
	whoisDialog.setSize(WIDTH, HEIGHT);
	whoisDialog.setResizable(false);

/*---------------------------------------------------------------------------
 *  the panel contains all the components
 *---------------------------------------------------------------------------*/
        
	JPanel whois = new JPanel();
	whois.setLayout(new BorderLayout());
	whois.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	setMPMSize(whois,new Dimension(WIDTH,HEIGHT));
	
	textArea = new JTextArea();
	textArea.setEditable(false);

	JScrollPane scrollPane = new JScrollPane(textArea,
                                   JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                                   JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
	setMPMSize(scrollPane,new Dimension(310, 280));

	JPanel rightPanel = new JPanel();
	rightPanel.setBorder(BorderFactory.createCompoundBorder(
                               BorderFactory.createTitledBorder("Record"),
                               BorderFactory.createEmptyBorder(5,5,5,5)));
	rightPanel.add(scrollPane);

	showWhoisRecord = new JButton(showRec);
	showWhoisRecord.setMnemonic('S');
	showWhoisRecord.setDefaultCapable(true);
	showWhoisRecord.setBorder(BorderFactory.createRaisedBevelBorder());
	setMPMSize(showWhoisRecord,new Dimension(180,30));
	showWhoisRecord.setAlignmentX(Component.CENTER_ALIGNMENT);
	showWhoisRecord.addActionListener(this);

	JButton cancel = new JButton(Cancel);
	cancel.setMnemonic('C');
	cancel.setBorder(BorderFactory.createRaisedBevelBorder());
	setMPMSize(cancel,new Dimension(100,30));
	cancel.setAlignmentX(Component.CENTER_ALIGNMENT);    
	cancel.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) 
                {
                    whoisDialog.setVisible(false);
                    whoisDialog.dispose();
                }
	});

	nameField = new JTextField(15);
	setMPMSize(nameField,new Dimension(200,25));
	nameField.addActionListener(this);
	
	JPanel namePanel = new JPanel();
	namePanel.setBorder(BorderFactory.createCompoundBorder(
                              BorderFactory.createTitledBorder("Name"),
                              BorderFactory.createEmptyBorder(5,5,5,5)));
	namePanel.add(nameField);

	asField = new JTextField(15);
	setMPMSize(asField,new Dimension(200,25));
	asField.addActionListener(this);
	
	JPanel asPanel = new JPanel();
	asPanel.setBorder(BorderFactory.createCompoundBorder(
                            BorderFactory.createTitledBorder("AS Number"),
                            BorderFactory.createEmptyBorder(5,5,5,5)));
	asPanel.add(asField);
	
	    
	hostCombo = new JComboBox();
	hostCombo.setEditable(true);
	setMPMSize(hostCombo,new Dimension(200,25));
	hostCombo.addItem("whois.arin.net");
	hostCombo.addItem("whois.ripe.net");
	hostCombo.addItem("whois.apnic.net");
	hostCombo.addItem("None");	

	JPanel hostPanel = new JPanel();
	hostPanel.setBorder(BorderFactory.createTitledBorder("Host"));
	hostPanel.setBorder(BorderFactory.createCompoundBorder(
                              BorderFactory.createTitledBorder("Host"),
                              BorderFactory.createEmptyBorder(5,5,5,5)));
	hostPanel.add(hostCombo);
	

	JPanel leftPanel = new JPanel();
	leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
	leftPanel.setAlignmentX(0);
	leftPanel.add(namePanel);
	leftPanel.add(Box.createRigidArea(new Dimension(0,10)));
	leftPanel.add(asPanel);
	leftPanel.add(Box.createRigidArea(new Dimension(0,10)));
	leftPanel.add(hostPanel);
	leftPanel.add(Box.createRigidArea(new Dimension(0,20)));
	leftPanel.add(showWhoisRecord);
	leftPanel.add(Box.createRigidArea(new Dimension(0,10)));
	leftPanel.add(cancel);
	
	whois.add(leftPanel, BorderLayout.WEST);
	whois.add(Box.createRigidArea(new Dimension(5,0)));
	whois.add(rightPanel, BorderLayout.CENTER);

/*---------------------------------------------------------------------------
 *  add the add panel to the content pane of the map dialog and set its
 *  postion relative to the parent frame.
 *---------------------------------------------------------------------------*/
        
	whoisDialog.getContentPane().add(whois);
	whoisDialog.setLocationRelativeTo(frame);
    }

/******************************************************************************
 **
 **     Method name:             showDialog
 **
 **     Purpose:                 Resets all the fields and displays the dialog.
 **    
 **     Parameters:              NONE
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/
    
    public void showDialog()
    {
	nameField.setText("");
	asField.setText("");
	textArea.setText("");
	hostCombo.setSelectedItem("whois.arin.net");
	whoisDialog.setVisible(true);
    }

/******************************************************************************
 **
 **     Method name:             showDialog
 **
 **     Purpose:                 Sets the name field to the specified parameter
 **                              sets domain to true, it name is an IP address.
 **
 **     Parameters:              String text - the hostname/ipaddress to be
 **                                            looked up.
 **                              boolean domain - decides which whois server is
 **                                               to be used for the lookup,
 **                                               internic or arin.
 **
 **     Return Values:           void 
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/

    public void showDialog(String text, boolean domain)
    {
	nameField.setText(text);
	asField.setText("");
	textArea.setText("");

	if (domain)
	{
	    hostCombo.setSelectedItem("None");
	}
	else 
	{
	    hostCombo.setSelectedItem("whois.arin.net");
	}

	showWhoisRecord.doClick();
	whoisDialog.setVisible(true);
    }


/******************************************************************************
 **
 **     Method name:             actionPerformed 
 **
 **     Purpose:                 Method invoked when the showWhoisRecord button
 **                              is clicked. Executes whois and displays the
 **                              record in the textarea.
 **    
 **     Parameters:              ActionEvent e
 **
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/

    public void actionPerformed(ActionEvent e) 
    {
	String whoisRecord = new String();

	try
	{
/*---------------------------------------------------------------------------
 *  Check if the nameField or the asField have anything in them to be looked
 *  up with prefreence being given to the nameField first.
 *---------------------------------------------------------------------------*/
            
	    if (nameField.getText().length() > 0)
	    {
		String wserver = (String)hostCombo.getSelectedItem();

		if (wserver.compareTo("None") == 0)
		{
		    p = Runtime.getRuntime().exec(
                                             GTrace.getGTraceResource("WHOIS") 
                                             + " " + nameField.getText());
		}
		else
		{
		    p = Runtime.getRuntime().exec(
                                             GTrace.getGTraceResource("WHOIS") 
                                             + " -h " + wserver  
                                             + " " + nameField.getText());
		}
	    }
	    else if (asField.getText().length() > 0)
	    {
                p = Runtime.getRuntime().exec(
                                         GTrace.getGTraceResource("WHOIS") 
                                         + " AS" + asField.getText());
	    }

/*---------------------------------------------------------------------------
 *  Read the contents of the spawned process and display the resulting text
 *  in the text area box, setting its width and heigth to the maximum width
 *  of any line and the height to the number of lines.
 *---------------------------------------------------------------------------*/
            
	    if (p!= null)
	    {	
		BufferedReader inStream = new BufferedReader(
                                 new InputStreamReader(p.getInputStream()));
		String str;
		boolean flag = true;				
		int temp=99;	/* dummy value set to temp, p.exitValue() will
                                   return 0 when the process is done.
                                */

		while(flag)
		{	
		    try
		    {
			if ((temp=p.exitValue()) == 0)
			{
			    flag = false;
			}
		    }
		    catch(IllegalThreadStateException itse)
		    {

			str=inStream.readLine();

			if (str == null)
			{
			    textArea.append("Error");
			    return;
			}
			whoisRecord = whoisRecord.concat(str+"\n");
		    }
		}

		boolean TRUE = true;
		while(TRUE)
	        {
		    str = inStream.readLine();
		    
		    if (str == null)
		    {
			break;
		    }
		    whoisRecord = whoisRecord.concat(str+"\n");
		}

		int linwidth[] = Util.getNumLinesMaxWidth(whoisRecord);

		textArea.setRows(linwidth[0]);
		textArea.setColumns(linwidth[1]);
		textArea.setText(whoisRecord);
	    }
	}
	catch(IOException io)
	{
	    System.out.println("Io exception");
	}		       
    }

/******************************************************************************
 **
 **     Method name:             setMPMSize
 **
 **     Purpose:                 Sets the Minimum, Preferred and Maximimum
 **                              sizes for a component
 **    
 **     Parameters:              JComponent component - the component whose
 **                                       size attributes are to be set.
 **                              Dimension d - the dimension to which they are
 **                                       to be set.
 **                              
 **     Return Values:           void
 **                                           
 **     Author/Last Modified:    Ram Periakaruppan, Sept. 10th, 1999
 **
 *****************************************************************************/

    public void setMPMSize(JComponent component, Dimension d)
    {
	component.setMinimumSize(d);
	component.setPreferredSize(d);
	component.setMaximumSize(d);
    }	
}
